% cond_init
%
% This script provides the initializations for cond_topo:
%
% Initialization values:
%
%    ADDCNST    Character string of additional constraints on active mask.
%    BOUND      Bounding value for slope parameter.
%    CRT_FLG    Criterion flag.  Indicates which quantity is bound.
%                  [0]  bound grad(h)/h  where h is the topography.
%                  [1]  bound grad(h)    where h is the topography.
%                  [2]  Bound both.
%    CURR_FAC   Slope factor for initial topography.
%    DXT        Zonal grid spacing at tracer points.
%    DYT        Meridional grid spacing at tracer points.
%    DXU        Zonal grid spacing at velocity points.
%    DYU        Meridional grid spacing at velocity points.
%    HSHLLW     Minimum depth above which the search is inactive.  (m)
%    HDEEP      Maximum depth below which the search is inactive.  (m)
%    ITMAX      Maximum number of iterations.
%    IPLOT      Flag for plotting sparse matrix [0] do NOT plot [1] plot
%    IPRINT     Flag for printing current cond. fac. [0] do NOT print [1] print
%    MEDFRQ     Frequency of applying median filter.
%    METXT      Zonal metric factors at tracer points.
%    METYT      Meridional metric factors at tracer points.
%    METXU      Zonal metric factors at velocity points.
%    METYU      Meridional metric factors at velocity points.
%    NESTFILE   GRIDS netCDF file for nesting super domain.
%    NETFILE    GRIDS netCDF file.
%    SUPTOPO    Topography in nesting super domain surrounding current domain.
%    TOPO       Initial topography.
%    VERTPARM   Parameters to describe double sigma system.
%                  vertparm(1):  ZC1     shallow bound
%                  vertparm(2):  ZC2     Deep bound
%                  vertparm(3):  ZREF    Reference depth.
%                  vertparm(4):  ZSLOPE  Slope bounding factor
%                  vertparm(5):  SIGRES  A measure of reduced sigma thickness
%
% Auxillary values:
%
%    TLON       Longitudes of tracer grid (topography).
%    TLAT       Latitudes of tracer grid (topography).
%    VLON       Longitudes of velocity grid (slope factor).
%    VLAT       Latitudes of velocity grid (slope factor).

%-------------------------------------------------------------------------------
% Get name of parameter input file.
%-------------------------------------------------------------------------------

[filename,pathname] = uigetfile ('*.in*','Run parameter file');

if (isstr(filename) & isstr(pathname))
   infile = fullfile (pathname,filename);
elseif isstr(filename)
   infile = filename;
else
   disp (' ');
   disp ('***Error:  COND_INIT - unspecified input file');
   disp (' ');
   return;
end;

clear filename pathname

%-------------------------------------------------------------------------------
% Read in parameters and name of GRIDS netCDF file.
%-------------------------------------------------------------------------------

[crt_flg,bound,hshllw,hdeep,itmax,medfrq,iplot,iprint, ...
                      addcnst,netfile,nestfile,status] = cond_read (infile);

clear infile;

%-------------------------------------------------------------------------------
% Get relevent data from GRIDS netCDF file.
%-------------------------------------------------------------------------------

[tlon,tlat,topo,vlon,vlat,dxt,dyt,metxt,metyt, ...
                 dxu,dyu,metxu,metyu,vertparm] = gridsread (netfile);

if isempty(vertparm),
   disp (' ');
   disp ('***Error:  COND_INIT - unable to read GRIDS netCDF file:');
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

[suptopo,status] = nestread (netfile,nestfile);

if (status<0),
   disp (' ');
   disp ('***Error:  COND_INIT - unable to read GRIDS netCDF file:');
   disp (['           ',setstr(34),netfile,setstr(34),'  or']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%-------------------------------------------------------------------------------
% Slope factor values, should be examined.
%-------------------------------------------------------------------------------

if (crt_flg==0),
   curr_fac = slopefac (topo,vertparm,dxu.*metxu,dyu.*metxu,2,0);
   bnddf = max([ 2 (max(max(curr_fac))/2) ]);
  elseif (crt_flg==1),
   bnddf = 0.06;
  else
   curr_fac = slopefac (topo,vertparm,dxu.*metxu,dyu.*metxu,2,0);
   bnddf = [max([ 2 (max(max(curr_fac))/2) ]) 0.06];
end;

ind = find (isnan(bound));
if ~isempty(ind)
   bound(ind) = bnddf(ind);
end;

curr_fac = slopefac (topo,vertparm,dxu.*metxu,dyu.*metxu,bound,crt_flg);

%-------------------------------------------------------------------------------
% Report beta values.
%-------------------------------------------------------------------------------

disp(' ');
if (crt_flg~=2),
   disp(['Bound for slope factor       = ',num2str(bound)]);
   disp(['Current maximum slope factor = ',num2str(max(max(curr_fac)))]);
 else
   disp(['Bound for slope factor                = [',num2str(bound(1)), ...
                                          ', ',num2str(bound(2)),']']);
   disp(['Current relative maximum slope factor = ', ...
                      num2str(max(max(curr_fac)))]);
end;
disp(' ');
