function [topo_new,iter] = cond_loop (topo,dxt,dyt,metxt,metyt,dxu,dyu, ...
                                      metxu,metyu,vertparm,bound,crt_flg, ...
                                     hshllw,hdeep,addcnst,lon,lat,curr_fac, ...
                                      itmax,medfrq,suptopo,iplot,iprint);
%
% function [topo_new,iter] = cond_loop (topo,dxt,dyt,metxt,metyt,dxu,dyu, ...
%                                       metxu,metyu,vertparm,bound,crt_flg, ...
%                                    hshllw,hdeep,addcnst,lon,lat,curr_fac, ...
%                                       itmax,medfrq,suptopo,iplot,iprint);
%
% This function manages the loop which drives the topographic conditioning:
%
% ------
% Input:
% ------
%
%    ADDCNST    Character string of additional constraints on active mask.
%    BOUND      Bounding value for slope parameter.
%    CRT_FLG    Criterion flag.  Indicates which quantity is bound.
%                  [0]  bound grad(h)/h  where h is the topography.
%                  [1]  bound grad(h)    where h is the topography.
%                  [2]  Bound both.
%    CURR_FAC   Slope factor for initial topography.
%    DXT        Zonal grid spacing at tracer points.
%    DYT        Meridional grid spacing at tracer points.
%    DXU        Zonal grid spacing at velocity points.
%    DYU        Meridional grid spacing at velocity points.
%    HSHLLW     Minimum depth above which the search is inactive.  (m)
%    HDEEP      Maximum depth below which the search is inactive.  (m)
%    ITMAX      Maximum number of iterations.
%    MEDFRQ     Frequency of applying median filter.
%    IPLOT      Flag for plotting sparse matrix [0] do NOT plot [1] plot
%    IPRINT     Flag for printing current cond. fac. [0] do NOT print [1] print
%    LAT        Latitudes of topography
%    LON        Longitudes of topography
%    METXT      Zonal metric factors at tracer points.
%    METYT      Meridional metric factors at tracer points.
%    METXU      Zonal metric factors at velocity points.
%    METYU      Meridional metric factors at velocity points.
%    SUPTOPO    Topography in nesting super domain surrounding current domain.
%    TOPO       Initial topography.
%    VERTPARM   Parameters to describe double sigma system.
%                  vertparm(1):  ZC1     shallow bound
%                  vertparm(2):  ZC2     Deep bound
%                  vertparm(3):  ZREF    Reference depth.
%                  vertparm(4):  ZSLOPE  Slope bounding factor
%                  vertparm(5):  SIGRES  A measure of reduced sigma thickness
%
% -------
% Output:
% -------
%
%    TOPO_NEW   Final, conditioned topography
%    ITER       Actual number of iterations used in the loop.

%-------------------------------------------------------------------------------
% Initialize the variables.
%-------------------------------------------------------------------------------

topo_new = topo;

[ny nx] = size(topo);
[Imat,Jmat] = meshgrid((1:nx),(1:ny));
clear nx ny;

iter = 0;

if (iplot),
   vstr   = version;
   vnum   = str2num(vstr(1));
   itt    = iter;
   facvec = max(max(curr_fac));
   clear vstr;
end;

%-------------------------------------------------------------------------------
% Condition the topography.
%-------------------------------------------------------------------------------

while ( slopetest(crt_flg,bound,curr_fac) & (iter<itmax)),

   %---------------------------------
   %--- Compute next iteration of ---
   %---  conditioned topography.  ---
   %---------------------------------

   topo_old = topo_new;
   topo_new = new_topo (topo_old,dxt,dyt,metxt,metyt,dxu,dyu,metxu, ...
                        metyu,vertparm,bound,crt_flg,hshllw, ...
                        hdeep,addcnst,lon,lat,suptopo,Imat,Jmat);

   %----------------------------
   %--- Apply median filter. ---
   %----------------------------

   test = rem((iter+1),medfrq);
   if ( ~isnan(test) & (test==0) ),
      if isempty(suptopo)
         topo_wk           = topo_new;
         [topo_new,mfiter] = med_filt2D (topo_wk,200,2);
         clear topo_wk mfiter;
       else
         topo_wk           = topo_new;
         [topo_new,mfiter] = med_filt2Dd (topo_wk,200,2);
%         topo_wk           = suptopo;
%         [suptopo,mfiter] = med_filt2Dd (topo_wk,200,2);
         clear topo_wk mfiter;
      end;
   end;

   %-----------------------------------
   %--- Compute iteration controls. ---
   %-----------------------------------

   iter = iter + 1;
   curr_fac = slopefac (topo_new,vertparm,dxu.*metxu,dyu.*metxu, ...
                        bound,crt_flg);

   %----------------------------------
   %--- Display status information ---
   %----------------------------------

   if (iprint),
      disp (['Max slope fac:  ',num2str(max(max(curr_fac)))]);
   end;
   if (iplot),
      itt    = [itt iter];
      facvec = [facvec max(max(curr_fac))];
      plot (itt,facvec);
      set  (gca,'XLim',[0 iter],'FontName','Times');
      if (min(min(facvec))~=max(max(facvec))),
         set (gca,'YLim',[min(min(facvec)) max(max(facvec))]);
      end;
      if (vnum>=5),
         set (gca,'FontSize',14);
      end;
      xlabel ('Iteration');
      if (crt_flg~=2),
         ylabel ('Max. Slope Factor');
         title ([' Bound: ',num2str(bound), ...
                 '  |  Best Slope Fac: ',num2str(min(min(facvec))), ...
                 '  |  Max Iter: ',num2str(itmax)]);
        else
         ylabel ('Max. Rel. Slope Factor');
         title ([' Bound: [',num2str(bound(1)),', ',num2str(bound(2)), ...
                 ']  |  Best Slope Fac: ',num2str(min(min(facvec))), ...
                 '  |  Max Iter: ',num2str(itmax)]);
      end;
      figure(gcf);
   end;

end;
