% cond_topo
%
% This script performs the topographic conditioning:
%
% Expected values, already in memory:
%
%    ADDCNST    Character string of additional constraints on active mask.
%    BOUND      Bounding value for slope parameter.
%    CRT_FLG    Criterion flag.  Indicates which quantity is bound.
%                  [0]  bound grad(h)/h  where h is the topography.
%                  [1]  bound grad(h)    where h is the topography.
%    CURR_FAC   Slope factor for initial topography.
%    DXT        Zonal grid spacing at tracer points.
%    DYT        Meridional grid spacing at tracer points.
%    DXU        Zonal grid spacing at velocity points.
%    DYU        Meridional grid spacing at velocity points.
%    HSHLLW     Minimum depth above which the search is inactive.  (m)
%    HDEEP      Maximum depth below which the search is inactive.  (m)
%    ITMAX      Maximum number of iterations.
%    IPLOT      Flag for plotting sparse matrix [0] do NOT plot [1] plot
%    IPRINT     Flag for printing current cond. fac. [0] do NOT print [1] print
%    TLAT       Latitudes of tracer grid (topography).
%    TLON       Longitudes of tracer grid (topography).
%    MEDFRQ     Frequency of applying median filter.
%    METXT      Zonal metric factors at tracer points.
%    METYT      Meridional metric factors at tracer points.
%    METXU      Zonal metric factors at velocity points.
%    METYU      Meridional metric factors at velocity points.
%    NESTFILE   GRIDS netCDF file for nesting super domain.
%    NETFILE    GRIDS netCDF file.
%    SUPTOPO    Topography in nesting super domain surrounding current domain.
%    TOPO       Initial topography.
%    VERTPARM   Parameters to describe double sigma system.
%                  vertparm(1):  ZC1     shallow bound
%                  vertparm(2):  ZC2     Deep bound
%                  vertparm(3):  ZREF    Reference depth.
%                  vertparm(4):  ZSLOPE  Slope bounding factor
%                  vertparm(5):  SIGRES  A measure of reduced sigma thickness
%    VLAT       Latitudes of velocity grid (slope factor).
%    VLON       Longitudes of velocity grid (slope factor).

%-------------------------------------------------------------------------------
% Condition the topography.
%-------------------------------------------------------------------------------

%-----------------------------------------------------
%--- Set-up to display elapsed time, if requested. ---
%-----------------------------------------------------

if (iplot | iprint),
   tic;
end;

%-----------------------------------------------
%--- Ensure conditioning plot is on its own. ---
%-----------------------------------------------

if (iplot)
   figure;
end;

%---------------------------------
%--- Condition the topography. ---
%---------------------------------

[topo_new,iter] = cond_loop (topo,dxt,dyt,metxt,metyt,dxu,dyu, ...
                             metxu,metyu,vertparm,bound,crt_flg, ...
                             hshllw,hdeep,addcnst,tlon,tlat, ...
                             curr_fac,itmax,medfrq,suptopo,iplot,iprint);

curr_fac = slopefac (topo_new,vertparm,dxu.*metxu,dyu.*metyu,bound,crt_flg);

%-------------------------------------------
%--- Display elapsed time, if requested. ---
%-------------------------------------------

if (iplot | iprint),
   t=toc;
   val = fix(t/3600);
   if (val>=10),
      tstr = num2str(val);
    else
      tstr = ['0',num2str(val)];
   end;
   val = fix(rem(t,3600)/60);
   if (val>=10),
      tstr = [tstr,':',num2str(val)];
    else
      tstr = [tstr,':0',num2str(val)];
   end;
   val = rem(t,60);
   if (val>=10),
      tstr = [tstr,':',num2str(val)];
    else
      tstr = [tstr,':0',num2str(val)];
   end;
   clear t val;
end;

if (iplot),
   XLim = get (gca,'XLim');
   YLim = get (gca,'YLim');
   h = text (0.5*max(XLim),(min(YLim)+0.9.*(max(YLim)-min(YLim))), ...
             ['Elapsed time - ',tstr],'FontName','Times');
   vstr = version;
   vnum = str2num(vstr(1));
   if (vnum>=5),
      set (h,'FontSize',14);
   end;   
   clear XLim YLim vstr vnum h;
end;

if (iprint),
   disp (['Elapsed time - ',tstr]);
end;

%-------------------------------------------------------------------------------
% Save critical work space, protection against crashes between now
% and possible rewriting of GRIDS file.
%-------------------------------------------------------------------------------

save    cond_topo                                                       ...
        addcnst         bound           crt_flg         curr_fac        ...
        dxt             dxu             dyt             dyu             ...
        hdeep           hshllw          iplot           iprint          ...
        iter            itmax           tlat            tlon            ...
        medfrq          metxt           metxu           metyt           ...
        metyu           nestfile        netfile         suptopo         ...
        topo            topo_new        vertparm        vlon            ...
        vlat
