% cond_write
%
% This script writes the conditioned topography to the GRIDS netCDF
% file.  It also records the necessary parameters to reproduce it.
%
% Expected values, already in memory:
%
%    ADDCNST    Character string of additional constraints on active mask.
%    BOUND      Bounding value for slope parameter.
%    CRT_FLG    Criterion flag.  Indicates which quantity is bound.
%                  [0]  bound grad(h)/h  where h is the topography.
%                  [1]  bound grad(h)    where h is the topography.
%                  [2]  Bound both.
%    DXU        Zonal grid spacing at velocity points.
%    DYU        Meridional grid spacing at velocity points.
%    HSHLLW     Minimum depth above which the search is inactive.  (m)
%    HDEEP      Maximum depth below which the search is inactive.  (m)
%    ITMAX      Maximum number of iterations.
%    MEDFRQ     Frequency of applying median filter.
%    METXU      Zonal metric factors at velocity points.
%    METYU      Meridional metric factors at velocity points.
%    NESTFILE   GRIDS netCDF file for nesting super domain.
%    NETFILE    GRIDS netCDF file.
%    SUPTOPO    Topography in nesting super domain surrounding current domain.
%    TOPO_NEW   Conditioned topography.
%    VERTPARM   Parameters to describe double sigma system.
%                  vertparm(1):  ZC1     shallow bound
%                  vertparm(2):  ZC2     Deep bound
%                  vertparm(3):  ZREF    Reference depth.
%                  vertparm(4):  ZSLOPE  Slope bounding factor
%                  vertparm(5):  SIGRES  A measure of reduced sigma thickness
%

%-------------------------------------------------------------------------------
% Initialize netCDF constants.  Set string for filter attribute.
%-------------------------------------------------------------------------------

status    = mexcdf ('setopts',0);

if (crt_flg==0),
   filt_string = 'Gradient conditioning:  reduced slope';
  elseif (crt_flg==1),
   filt_string = 'Gradient conditioning:  slope';
  else,
   filt_string = 'Gradient conditioning:  both';
end;

%-------------------------------------------------------------------------------
% Open GRIDS netCDF file.  Put into define mode.
%-------------------------------------------------------------------------------

ncid = netcdf (netfile,'write');
if isempty(ncid),
   disp('***Error:  COND_WRITE - unable to open file:');
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

%status = mexcdf ('ncredef',ncid);
%if (status < 0),
%   disp([setstr(7),'***Error:  COND_WRITE - unable to put file:',setstr(7)]);
%   disp([setstr(7),'           ',setstr(34),netfile,setstr(34),setstr(7)]);
%   disp([setstr(7),'           into define mode.',setstr(7)]);
%   return
%end;

%-------------------------------------------------------------------------------
% Write global condition attribute attribute.
%-------------------------------------------------------------------------------

date_info = datenum(now);

histstamp = ['cond_topo (Matlab) Version 3.6 - ', ...
                     datestr(date_info,'dddd'),' - ', ...
                     datestr(date_info,'mmmm'),' ', ...
         datestr(date_info,'dd'),', ',datestr(date_info,'yyyy'),' - ', ...
         strjust(lower(datestr(date_info,'HH:MM:SSPM')),'left')];

ncid.condition(1:length(histstamp)) = histstamp;

clear date_info histstamp;

%-------------------------------------------------------------------------------
% Look for dimension grad_pass.  If not present, define it along with the
% filter descriptor variables.
%-------------------------------------------------------------------------------

if isempty(ncid('grad_pass')),
   redefcdf (ncid,length(addcnst));
end;

%-------------------------------------------------------------------------------
% Get netCDF id for topography.  Add filter attribute.  Take out of define mode.
%-------------------------------------------------------------------------------

%--------------------------------
%--- Determine filter number. ---
%--------------------------------

varname = 'tbath';
nfilt = 0;
eval(['theAtt = ncid{varname}.filter_',num2str(nfilt),';']);
while ~isempty(theAtt),
   nfilt = nfilt + 1;
   eval(['theAtt = ncid{varname}.filter_',num2str(nfilt),';']);
end;

%-----------------------------------------------
%--- Add new filter attribute to topography. ---
%-----------------------------------------------

eval(['ncid{varname}.filter_',num2str(nfilt),' = filt_string;']);

%----------------------------------------------------
%--- Take GRIDS netCDF file out of "define" mode. ---
%----------------------------------------------------

%status = mexcdf ('ncendef',ncid);
%if (status < 0),
%   disp([setstr(7),'***Error:  COND_WRITE - unable to take file:',setstr(7)]);
%   disp([setstr(7),'           ',setstr(34),netfile,setstr(34),setstr(7)]);
%   disp([setstr(7),'           out of define mode.',setstr(7)]);
%   return
%end;

%-------------------------------------------------------------------------------
% Write new topography.
%-------------------------------------------------------------------------------

%----------------------------------
%--- Topography on Tracer grid. ---
%----------------------------------

ncid{varname}(:) = topo_new;

%------------------------------------
%--- Topography on Velocity grid. ---
%------------------------------------

name = 'vbath';

if ~isempty(ncid{name})
   [ny nx] = size(topo_new);
   n = [2:ny ny];
   s = 1:ny;
   e = [2:nx nx];
   w = 1:nx;
   vnew = (topo_new(n,e) + topo_new(n,w) + topo_new(s,e) + topo_new(n,w))./4;
   ncid{name}(:) = vnew;
   clear e n s vnew w;
end;

%-------------------------------------------------------------------------------
% Compute coordinate interface.  If double sigma system, write new interface.
%-------------------------------------------------------------------------------

favg = 0.5*(vertparm(1)+vertparm(2));
fdif = 0.5*(vertparm(2)-vertparm(1));
if (fdif~=0),
   fac = vertparm(4)/fdif;
  else,
   fac = 0;
end;

iface = favg + fdif.*tanh( fac.*(topo_new-vertparm(3)) );

if (vertparm(1)~=vertparm(2)),

   %--------------------
   %--- Tracer grid. ---
   %--------------------

   name   = 'interface';
   ncid{name}(:) = iface;

   %----------------------
   %--- Velocity grid. ---
   %----------------------

   name   = 'viface';
   if ~isempty(ncid{name})
      [ny nx] = size(iface);
      n = [2:ny ny];
      s = 1:ny;
      e = [2:nx nx];
      w = 1:nx;
      vnew = (iface(n,e) + iface(n,w) + iface(s,e) + iface(n,w))./4;
      ncid{name}(:) = vnew;
      clear e n s vnew w;
   end;

end;

%-------------------------------------------------------------------------------
% Write new hydrostatic consistency values.
%-------------------------------------------------------------------------------

%-------------------------------------
%--- Read vertical discretization. ---
%-------------------------------------

name = 'kc';
kc   = ncid{name}(:);
if isempty(kc),
   disp(['***Error:  COND_WRITE - unable to read variable ', ...
                        setstr(34),name,setstr(34),' in file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

name = 'level';
nz   = length (ncid(name));
if isempty(nz),
   disp(['***Error:  COND_WRITE - unable to read dimension ', ...
                     setstr(34),name,setstr(34),' in file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

name = 'hz';
dsig = ncid{name}(:);
if isempty(dsig),
   disp(['***Error:  COND_WRITE - unable to read variable ', ...
                        setstr(34),name,setstr(34),' in file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

%--------------------------------
%--- Scale sigma thicknesses. ---
%--------------------------------

dsig = dsig';
if (kc>0),
   dsig(1:kc) = dsig(1:kc)./sum(dsig(1:kc));
end;
dsig((kc+1):nz) = dsig((kc+1):nz)./sum(dsig((kc+1):nz));

%------------------------------------------
%--- Compute sigma levels at mid-boxes. ---
%------------------------------------------

testmat = 0.5.*diag(ones(nz,1)) + triu(ones(nz,nz),1);
sigval = dsig*testmat;

%------------------------------------------
%--- Compute slopes at velocity points. ---
%------------------------------------------

[ifx,ify] = gradt2u (iface,dxu.*metxu,dyu.*metyu);
[hx,hy]   = gradt2u (topo_new,dxu.*metxu,dyu.*metyu);

%--------------------------------------------------------
%--- Compute topography & interface at tracer points. ---
%--------------------------------------------------------

w = 1:nx;
e = [2:nx nx];
s = 1:ny;
n = [2:ny ny];


dum=0.25.*(topo_new(n,e)+topo_new(n,w)+topo_new(s,e)+topo_new(s,w));
iface=0.25.*(iface(n,e)+iface(n,w)+iface(s,e)+iface(s,w));

%--------------------------------------
%--- Reshape for matrix operations. ---
%--------------------------------------

dum = reshape(dum',1,nx*ny);
hx = reshape(hx',1,nx*ny);
hy = reshape(hx',1,nx*ny);
iface = reshape(iface',1,nx*ny);
ifx = reshape(ifx',1,nx*ny);
ify = reshape(ify',1,nx*ny);

%----------------------------------------
%--- Read netCDF variable identifier. ---
%----------------------------------------

name  = 'hcf';
if isempty(ncid{name}),
   name  = 'hcf_up';
   if isempty(ncid{name}),
      disp(['***Error:  COND_WRITE - unable to find variable ',setstr(34), ...
            'hcf',setstr(34),' or ',setstr(34),name,setstr(34),' in file:']);
      disp(['           ',setstr(34),netfile,setstr(34)]);
      return
   end;
end;

%---------------------------------------
%--- Read special value for filling. ---
%---------------------------------------

attname = '_FillValue';
spval   = ncid{name}.FillValue_(:);
if isempty(spval),
   disp(['***Error:  COND_WRITE - unable to find attribute ', ...
         setstr(34),name,':',attname,setstr(34),' in file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

%---------------------------------------------
%--- Work in upper system of double sigma. ---
%---------------------------------------------

if (vertparm(1)~=vertparm(2)),

   %-------------------------------------------------
   %--- Compute Hyd. Const. Fac. in upper system. ---
   %-------------------------------------------------

   hcfx = (ones(size(sigval(1:kc)')))*(spval.*ones(size(ifx)));
   hcfy = hcfx;

   ind = find (iface<0);
   hcfx(:,ind) = ((sigval(1:kc)./dsig(1:kc))')*abs(ifx(ind)./iface(ind));
   hcfy(:,ind) = ((sigval(1:kc)./dsig(1:kc))')*abs(ify(ind)./iface(ind));

   hcf = max(hcfx);
   ind = find (hcf < max(hcfy) );
   hcf(ind) = max(hcfy(:,ind));
   hcf = reshape(hcf,nx,ny);

   %-------------------------------------------------
   %--- Write Hyd. Const. Fac. in upper system. ---
   %-------------------------------------------------

   if strcmp(name,'hcf');
      ncid{name}(:,:,1) = hcf;
     else
      ncid{name}(:) = hcf;
   end;

end;

%---------------------------------------------
%--- Work in lower system of double sigma. ---
%---------------------------------------------

if ~strcmp(name,'hcf');
   name  = 'hcf_dn';
   if isempty(ncid{name}),
      disp(['***Error:  COND_WRITE - unable to find variable ', ...
                       setstr(34),name,setstr(34),' in file:']);
      disp(['           ',setstr(34),netfile,setstr(34)]);
      return
   end;

   attname = '_FillValue';
   spval   = ncid{name}.FillValue_(:);
   if (status < 0),
      disp(['***Error:  COND_WRITE - unable to find attribute ', ...
            setstr(34),name,':',attname,setstr(34),' in file:']);
      disp(['           ',setstr(34),netfile,setstr(34)]);
      return
   end;
end;

%-------------------------------------------------
%--- Compute Hyd. Const. Fac. in lower system. ---
%-------------------------------------------------

knd = (kc+1):nz;
hcfx = (ones(size(sigval(knd)')))*(spval.*ones(size(ifx)));
hcfy = hcfx;

dum = dum - iface;
ind = find (dum<0);
hcfx(:,ind) = (((2-sigval(knd))./dsig(knd))')*abs(ifx(ind)./dum(ind)) + ...
              (((sigval(knd)-1)./dsig(knd))')*abs(hx(ind)./dum(ind));
hcfy(:,ind) = (((2-sigval(knd))./dsig(knd))')*abs(ify(ind)./dum(ind)) + ...
              (((sigval(knd)-1)./dsig(knd))')*abs(hy(ind)./dum(ind));

hcf = max(hcfx);
ind = find (hcf < max(hcfy) );
hcf(ind) = max(hcfy(:,ind));
hcf = reshape(hcf,nx,ny);

%-----------------------------------------------
%--- Write Hyd. Const. Fac. in lower system. ---
%-----------------------------------------------

if strcmp(name,'hcf');
   ncid{name}(:,:,2) = hcf;
  else
   ncid{name}(:) = hcf;
end;

%-------------------------------------------------------------------------------
% Write 3D thicknesses, if present.
%-------------------------------------------------------------------------------

%---------------------------
%--- T-grid thicknesses. ---
%---------------------------

status = sync(ncid);

name  = 'interface';
iface = ncid{name}(:);
if isempty(iface),
   disp(['***Error:  COND_WRITE - unable to read variable ', ...
                 setstr(34),name,setstr(34),' in file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

name = 'dzt';
if ~isempty(ncid{name}),
   for j = 1:ny
   for i = 1:nx
      if (kc>0),
         dz(1:kc) = dsig(1:kc).*abs(iface(j,i));
      end;
      dz((kc+1):nz) = dsig((kc+1):nz).*(abs(topo_new(j,i))-abs(iface(j,i)));
      ncid{name}(j,i,:) = dz;
   end;
   end;
end;

%---------------------------
%--- V-grid thicknesses. ---
%---------------------------

status = sync(ncid);

name = 'dzv';
if ~isempty(ncid{name}),
   w = 1:nx;
   e = [2:nx nx];
   s = 1:ny;
   n = [2:ny ny];

   for nn = 1:nz
      dzt = squeeze(ncid{'dzt'}(:,:,nn));
      dzv = 0.25.*(dzt(n,e)+dzt(n,w)+dzt(s,e)+dzt(s,w))';
      ncid{name}(:,:,nn) = dzv;
   end;
end;

%-------------------------------------------------------------------------------
% Write new search parameters.  Close netCDF file.
%-------------------------------------------------------------------------------

%-----------------------------------------------
%--- Get current number of condition passes. ---
%-----------------------------------------------

name = 'grad_pass';
pass = length (ncid(name));
if isempty(pass),
   disp(['***Error:  COND_WRITE - unable to read dimension ', ...
                 setstr(34),'grad_pass',setstr(34),' in file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

pass = pass + 1;

%----------------------
%--- Criteria flag. ---
%----------------------

name   = 'crt_flg';
ncid{name}(pass) = crt_flg;

%---------------------------
%--- Slope factor bound. ---
%---------------------------

name   = 'slope_bound';
if (crt_flg==0),
   ncid{name}(pass,1) = bound;
  elseif (crt_flg==1)
   ncid{name}(pass,2) = bound;
  else
   ncid{name}(pass,:) = bound;
end;

%---------------------------------------
%--- Shallow limit for conditioning. ---
%---------------------------------------

name   = 'hshllw';
ncid{name}(pass) = hshllw;

%------------------------------------
%--- Deep limit for conditioning. ---
%------------------------------------

name   = 'hdeep';
ncid{name}(pass) = hdeep;

%-------------------------------------
%--- Maximum number of iterations. ---
%-------------------------------------

name   = 'itmax';
ncid{name}(pass) = iter;

%--------------------------------------------
%--- Frequency of applying median filter. ---
%--------------------------------------------

name   = 'medfrq';
ncid{name}(pass) = medfrq;

%-------------------------------
%--- Additional constraints. ---
%-------------------------------

name   = 'addcnst';
if isempty(addcnst),
   addcnst = 'None';
end;
ncid{name}(pass,1:length(addcnst)) = addcnst;

%-------------------------------
%--- Additional constraints. ---
%-------------------------------

name   = 'nestfile';
if isempty(suptopo),
   nestfile = 'None';
end;
ncid{name}(pass,1:length(nestfile)) = nestfile;

%-------------------------------------------------------------------------------
% Close Grids netCDF file.
%-------------------------------------------------------------------------------

status = close (ncid);
if ~isempty(status),
   disp(['***Error:  COND_WRITE - unable to close file:']);
   disp(['           ',setstr(34),netfile,setstr(34)]);
   return
end;

%-------------------------------------------------------------------------------
% Clear temporary variables.
%-------------------------------------------------------------------------------

clear datatype dimid dum hx hy filt_string fac favg fdif hist_str hcf;
clear hcfx hcfy dsig iface ifx ify ind n s e w kc knd len nc_char nc_global;
clear nc_write ncid nfilt nz nx ny sigval spval status;
clear testmat varid;
