function [dfdx,dfdy] = gradt2u (f,dx,dy);
%
% function [dfdx,dfdy] = gradt2u (f,dx,dy);
%
% This function computes the gradient at the velocity points of an
% Arakawa B-grid of a field located at tracer points.
%
% ------
% Input:
% ------
%
%    f......Field whose gradient is sought.
%    dx.....Grid spacing in x-direction.  [Default:  1]
%    dy.....Grid spacing in y-direction.  [Default:  dx]
%
% -------
% Output:
% -------
%
%    dfdx...Slope component in x-direction.
%    dfdy...Slope component in y-direction.

%-------------------------------------------------------------------------------
% Set default values.
%-------------------------------------------------------------------------------

[ny nx] = size (f);

%--------------------------
%--- Set x-grid spacing ---
%--------------------------

if (nargin<2),
   dx = ones(ny,nx);
 elseif (length(dx)==1),
   dx = dx.*ones(ny,nx);
 else,
   [ny2 nx2] = size (dx);
   if (ny2==1),
      dx = ones(ny,1)*dx;
    elseif (nx2==1),
      dx = dx*ones(1,nx);
   end;
end;

[ny2 nx2] = size (dx);

if ((ny~=ny2) | (nx~=nx2)),
   disp (' ');
   error('***Error:  GRADT2U - size of dx does not match size of field.');
end;

%--------------------------
%--- Set y-grid spacing ---
%--------------------------

if (nargin<2),
   dy = dx;
 elseif (length(dy)==1),
   dy = dy.*ones(ny,nx);
 else,
   [ny2 nx2] = size (dy);
   if (ny2==1),
      dy = ones(ny,1)*dy;
    elseif (nx2==1),
      dy = dx*ones(1,ny);
   end;
end;

[ny2 nx2] = size (dy);

if ((ny~=ny2) | (nx~=nx2)),
   disp (' ');
   error ('***Error:  GRADT2U - size of dy does not match size of field.');
end;

clear nx2 ny2

%-------------------------------------------------------------------------------
% Compute gradient.
%-------------------------------------------------------------------------------

%--------------------
%--- Set indices. ---
%--------------------

n = [2:ny ny];
s = 1:ny;

e = [2:nx nx];
w = 1:nx;

%----------------------------------------
%--- Compute x-component of gradient. ---
%----------------------------------------

dfdx = 0.5 .* ((f(n,e)-f(n,w)) + (f(s,e)-f(s,w)))./dx;

%----------------------------------------
%--- Compute y-component of gradient. ---
%----------------------------------------

dfdy = 0.5 .* ((f(n,e)-f(s,e)) + (f(n,w)-f(s,w)))./dy;
