function [p,iter] = med_filt2Dd (data,itmax,intflg)
%
%function [p,iter] = med_filt2Dd (data,itmax,intflg)
%
% This function performs repeated median filtering of the supplied data until
% a steady state is reached or no further changes occur.  The patterns used are
% the "local, integral weighted" schemes.  This version uses "Dirichlet"
% boundary conditions (ie: boundaries are unfiltered).
%
% Input:
%
%    data.....2D array.  The data to be filtered.
%               [required]
%    itmax....Integer.  The maximum number of
%               times to apply the filter.
%               [Default: 200]
%    intflg...Integer.  Flag to indicate whether data should
%             be taken to be integer.
%             [0]  not integer data  (default)
%             [1]  integer data
%             [2]  only work to single precision
%
% Output:
%
%    p........2D array.  The filtered data.
%    iter.....Integer.  The actual number of iterations.
%
% Note:  The data should be 3x3 or larger.  Otherwise
%        the filter simply returns the data.

%-------------------------------------------------------------------------------
% Set defaults (if necessary).
%-------------------------------------------------------------------------------

if (nargin<3), intflg=0; end;
if (intflg>2), intflg=0; end;
if (nargin<2), itmax=200; end;

if (intflg==2),
   tol = sqrt (eps);
end;

%-------------------------------------------------------------------------------
% Get size of data.
%-------------------------------------------------------------------------------

[n m] = size(data);

%-------------------------------------------------------------------------------
% Make sure data array is large enough for filter to make sense.
%-------------------------------------------------------------------------------

if ( min(min([n m])) >= 3),

%-------------------------------------------------------------------------------
%    Initialize working variables.
%-------------------------------------------------------------------------------

   p = data;
   test = 1;
   iter = 0;

   mm1 = m - 1;
   mm2 = m - 2;
   nm1 = n - 1;

   ind = 2:mm1;
   indp1 = ind-1;
   indm1 = ind+1;
   wind = 1:mm2;

%-------------------------------------------------------------------------------
%    Repeatedly filter data.
%-------------------------------------------------------------------------------

   while (test & (iter<itmax)),

      pold = p(1,:);
      test = 0;

      for j = 2: nm1

         poldr = pold;
         pold = p(j,:);
         jp1 = j + 1;

         wk(1:16,wind) =[poldr(indm1)' poldr(ind)'  poldr(ind)' poldr(indp1)'...
                         pold(indm1)' pold(ind)'  pold(ind)' pold(indp1)' ...
                         pold(indm1)' pold(ind)'  pold(ind)' pold(indp1)' ...
                         p(jp1,indm1)' p(jp1,ind)' p(jp1,ind)' p(jp1,indp1)']';

         p(j,ind) = median(wk);
         if (intflg==1),
            p(j,ind) = round(p(j,ind));
         end;
         if (intflg~=2),
            test = any([test (p(j,ind)~=pold(ind))]);
          else,
            test = any([test (abs(p(j,ind)-pold(ind)) > abs(tol.*pold(ind))) ]);
         end;

      end

   end;

  else,

%-------------------------------------------------------------------------------
%    Data is too small to filter, simply pass it out.
%-------------------------------------------------------------------------------

   p = data;
   iter = 0;

end;
