function [suptopo,status] = nestread (netfile,nestfile);
%
% function [suptopo,status] = nestread (netfile,nestfile);
%
% This function reads the portion of the topography in the nesting super
% domain which covers the current domain.
%
% ------
% Input:
% ------
%
%    NESTFILE   Name of GRIDS netCDF file for nesting super domain.
%    NETFILE    Name of GRIDS netCDF file for current domain.
%
% -------
% Output:
% -------
%
%    SUPTOPO    Topography in nesting super domain surrounding current domain.
%    STATUS     Exit status.  [0] no error  [<0] error

%-------------------------------------------------------------------------------
% Set useful parameters.
%-------------------------------------------------------------------------------

status = mexcdf ('setopts',0);

%-------------------------------------------------------------------------------
% Initialize output.
%-------------------------------------------------------------------------------

suptopo = [];
status  = 0;

%-------------------------------------------------------------------------------
% Open GRIDS netCDF file.
%-------------------------------------------------------------------------------

%----------------------------------------------------------
%--- Try to open nesting file.  If can't, none is used. ---
%----------------------------------------------------------

nclid = netcdf (nestfile);

if isempty(nclid),
   return
end;

status = -1;

%---------------------------------
%--- Open current domain file. ---
%---------------------------------

ncsid = netcdf (netfile);

if isempty(ncsid),
   disp (' ');
   disp ('***Error:  NESTREAD - unable to open GRIDS netCDF file:');
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

%-------------------------------------------------------------------------------
% Read dimensions.
%-------------------------------------------------------------------------------

%-------------------------------
%--- Number of zonal points. ---
%-------------------------------

name = 'tlon';

nxs = length (ncsid(name));
if isempty(nxs),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read dimension ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

nxl = length (nclid(name));
if isempty(nxl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read dimension ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%------------------------------------
%--- Number of meridional points. ---
%------------------------------------

name = 'tlat';

nys = length (ncsid(name));
if isempty(nys),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read dimension ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

nyl = length (nclid(name));
if isempty(nyl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read dimension ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%-------------------------------------------------------------------------------
% Read domain parameters.
%-------------------------------------------------------------------------------

%--------------------
%--- Domain type. ---
%--------------------

name = 'coord';

crds = ncsid{name}(:);
if isempty(crds),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

crdl = nclid{name}(:);
if isempty(crdl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%--------------------------------
%--- Transformation centroid. ---
%--------------------------------

name = 'rlngd';

lons = ncsid{name}(:);
if isempty(lons),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

lonl = nclid{name}(:);
if isempty(lonl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

name = 'rlatd';

lats = ncsid{name}(:);
if isempty(lats),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

latl = nclid{name}(:);
if isempty(latl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%----------------------
%--- Domain offset. ---
%----------------------

name = 'delx';

dlxs = ncsid{name}(:);
if isempty(dlxs),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

dlxl = nclid{name}(:);
if isempty(dlxl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

name = 'dely';

dlys = ncsid{name}(:);
if isempty(dlys),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

dlyl = nclid{name}(:);
if isempty(dlyl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%---------------------
%--- Grid spacing. ---
%---------------------

name = 'meandx';

dxs = ncsid{name}(:);
if isempty(dxs),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

dxl = nclid{name}(:);
if isempty(dxl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

name = 'meandy';

dys = ncsid{name}(:);
if isempty(dys),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

dyl = nclid{name}(:);
if isempty(dyl),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%------------------------
%--- Domain rotation. ---
%------------------------

name = 'thetad';

thetas = ncsid{name}(:);
if isempty(thetas),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   return
end;

thetal = nclid{name}(:);
if isempty(thetal),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%-------------------------------------------------------------------------------
% Determine positions of overlap topography.
%-------------------------------------------------------------------------------

[lon,lat] = xy2ll (3,3,crds,nxs,nys,dxs,dys,lons,lats,dlxs,dlys,thetas);
[ill,jll] = ll2xy (lon,lat,crdl,nxl,nyl,dxl,dyl,lonl,latl,dlxl,dlyl,thetal);
ill = round(ill) - 2;
jll = round(jll) - 2;

[lon,lat] = xy2ll ((nxs-2),(nys-3),crds,nxs,nys,dxs,dys,lons, ...
                   lats,dlxs,dlys,thetas);
[iur,jur] = ll2xy (lon,lat,crdl,nxl,nyl,dxl,dyl,lonl,latl,dlxl,dlyl,thetal);
iur = round(iur) + 2;
jur = round(jur) + 2;

%-------------------------------------------------------------------------------
% Read topography.
%-------------------------------------------------------------------------------

name    = 'tbath';
suptopo = nclid{name}((jll:jur),(ill:iur));

if isempty(suptopo),
   disp (' ');
   disp (['***Error:  NESTREAD - unable to read variable ',setstr(34), ...
                              name,setstr(34),' in GRIDS netCDF file:']);
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   return
end;

%-------------------------------------------------------------------------------
% Close GRIDS netCDF file.
%-------------------------------------------------------------------------------

status = close (ncsid);

if ~isempty(status),
   disp (' ');
   disp ('***Error:  NESTREAD - unable to close GRIDS netCDF file:');
   disp (['           ',setstr(34),netfile,setstr(34)]);
   disp (' ');
   status = -1;
   return
end;

status = close (nclid);

if ~isempty(status),
   disp (' ');
   disp ('***Error:  NESTREAD - unable to close GRIDS netCDF file:');
   disp (['           ',setstr(34),nestfile,setstr(34)]);
   disp (' ');
   status = -1;
   return
end;

status  = 0;
