function topo_new = new_topo (topo_old,dxt,dyt,metxt,metyt,dxu,dyu,metxu, ...
                              metyu,vertparm,bndconst,criteria,hshllw, ...
                              hdeep,addcnst,lon,lat,suptopo,Imat,Jmat);
%
% function topo_new = new_topo (topo_old,dxt,dyt,metxt,metyt,dxu,dyu,metxu, ...
%                               metyu,vertparm,bndconst,criteria,hshllw, ...
%                               hdeep,addcnst,lon,lat,suptopo,Imat,Jmat);
%
% This function conditions the topography according to the current derivative
% constraints.
%
% ------
% Input:
% ------
%
%    TOPO_OLD   Original topography.
%    DXT        Width of a tracer box in the x-direction.
%    DYT        Width of a tracer box in the y-direction.
%    METXT      X-metric terms evaluated at tracer points.
%    METYT      Y-metric terms evaluated at tracer points.
%    DXU        Width of a velocity box in the x-direction.
%    DYU        Width of a velocity box in the y-direction.
%    METXU      X-metric terms evaluated at velocity points.
%    METYU      Y-metric terms evaluated at velocity points.
%    VERTPARM   Parameters to describe double sigma system.
%                  vertparm(1):  zc1     shallow bound
%                  vertparm(2):  zc2     Deep bound
%                  vertparm(3):  zref    Reference depth.
%                  vertparm(4):  zslope  Slope bounding factor
%                  vertparm(5):  sigres  A measure of reduced sigma thickness
%    BNDCONST   Constant to bound slope factor.
%    CRITERIA   Bounding criteria.
%                  [0]  Bound reduced slope.
%                  [1]  Bound slope.
%                  [2]  Bound both.
%    HSHLLW     Shallow bound on topography available for modification.
%    HDEEP      Deep bound on topography available for modification.
%    ADDCNST    Additional constraints on topography available for
%               modification.  String of "untouchable" constraints to be
%               "anded" to list.
%    LON        Longitudes of topography
%    LAT        Latitudes of topography
%    SUPTOPO    Topography in nesting super domain surrounding current domain.
%    IMAT       I-indices of topography, for additional constraints.
%    JMAT       J-indices of topography, for additional constraints.
%
% -------
% Output:
% -------
%
%    TOPO_NEW   Conditioned topography

%-------------------------------------------------------------------------------
% Initialize with identity matrix.
%-------------------------------------------------------------------------------

[ny nx] = size (topo_old);

rhs = reshape (topo_old,nx*ny,1);

diag0     = ones (nx*ny,1);

diagp1    = zeros (nx*ny,1);
diagpnym1 = zeros (nx*ny,1);
diagpny   = zeros (nx*ny,1);
diagpnyp1 = zeros (nx*ny,1);

diagm1    = zeros (nx*ny,1);
diagmnyp1 = zeros (nx*ny,1);
diagmny   = zeros (nx*ny,1);
diagmnym1 = zeros (nx*ny,1);

if ~isempty(suptopo)
   [nyl,nxl]  = size(suptopo);
   ill        = 3;
   jll        = 3;
   iur        = nxl - 2;
   jur        = nyl - 2;
   diagm3nym1 = zeros (nx*ny,1);
   diagm3ny   = zeros (nx*ny,1);
   diagm3nyp1 = zeros (nx*ny,1);
   diagm2nym1 = zeros (nx*ny,1);
   diagm2ny   = zeros (nx*ny,1);
   diagm2nyp1 = zeros (nx*ny,1);
   diagmnyp2  = zeros (nx*ny,1);
   diagmnyp3  = zeros (nx*ny,1);
   diagm2     = zeros (nx*ny,1);
   diagp2     = zeros (nx*ny,1);
   diagp3     = zeros (nx*ny,1);
   diagpnyp2  = zeros (nx*ny,1);
   diagpnyp3  = zeros (nx*ny,1);
   diagp2nym1 = zeros (nx*ny,1);
   diagp2ny   = zeros (nx*ny,1);
   diagp2nyp1 = zeros (nx*ny,1);
   diagp3nym1 = zeros (nx*ny,1);
   diagp3ny   = zeros (nx*ny,1);
   diagp3nyp1 = zeros (nx*ny,1);
end

%-------------------------------------------------------------------------------
% Apply boundary conditions.
%-------------------------------------------------------------------------------

if isempty(suptopo)

   %===================================
   %=== Apply no-normal constraint. ===
   %===================================

   %--------------------
   %--- Western Edge ---
   %--------------------

   j = 2:(ny-1);
   i = ones(size(j));

   ip = j + (i-1).*ny;

   diagpny(ip+ny) = -ones(size(j));
   rhs    (ip) = zeros(size(j));

   %--------------------
   %--- Eastern Edge ---
   %--------------------

   j = 2:(ny-1);
   i = nx.*ones(size(j));

   ip = j + (i-1).*ny;

   diagmny(ip-ny) = -ones(size(j));
   rhs    (ip) = zeros(size(j));

   %---------------------
   %--- Southern Edge ---
   %---------------------

   i = 2:(nx-1);
   j = ones(size(i));

   ip = j + (i-1).*ny;

   diagp1(ip+1) = -ones(size(j));
   rhs   (ip) = zeros(size(j));

   %---------------------
   %--- Northern Edge ---
   %---------------------

   i = 2:(nx-1);
   j = ny.*ones(size(i));

   ip = j + (i-1).*ny;

   diagm1(ip-1) = -ones(size(j));
   rhs   (ip) = zeros(size(j));

   %-----------------
   %--- SW corner ---
   %-----------------

   ip = 1;

   diagpnyp1(ip+ny+1) = -1;
   rhs   (ip) = 0;

   %-----------------
   %--- SE corner ---
   %-----------------

   ip = 1 + (nx-1)*ny;

   diagmnyp1(ip-ny+1) = -1;
   rhs      (ip) = 0;

   %-----------------
   %--- NW corner ---
   %-----------------

   ip = ny;

   diagpnym1(ip+ny-1) = -1;
   rhs      (ip) = 0;

   %-----------------
   %--- NE corner ---
   %-----------------

   ip = nx*ny;

   diagmnym1(ip-ny-1) = -1;
   rhs      (ip) = 0;

 else

   %=======================================
   %=== Apply compatibility constraint. ===
   %=======================================

   %--------------------
   %--- Western Edge ---
   %--------------------

   j = 4:3:(ny-2);
   i = ones(size(j));
   ip = j + (i-1).*ny;
   dj = fix(j./3) - 1;
   diagm1(ip-1) = -2/3;
   diagp2(ip+2) = -1/3;
   rhs(ip) = 0;

   j = 5:3:(ny-1);
   i = ones(size(j));
   ip = j + (i-1).*ny;
   dj = fix(j./3) - 1;
   diagm2(ip-2) = -1/3;
   diagp1(ip+1) = -2/3;
   rhs(ip) = 0;

   j = 3:3:(ny-3);
   i = ones(size(j));
   ip = j + (i-1).*ny;
   dj = fix(j./3) - 1;
   diagpnym1(ip-1+ny)    = -(1/27);
   diagp2nym1(ip-1+2*ny) = -(1/27);
   diagp3nym1(ip-1+3*ny) = -(1/27);
   diagpny(ip+ny)        = -(1/27);
   diagp2ny(ip+2*ny)     = -(1/27);
   diagp3ny(ip+3*ny)     = -(1/27);
   diagpnyp1(ip+1+ny)    = -(1/27);
   diagp2nyp1(ip+1+2*ny) = -(1/27);
   diagp3nyp1(ip+1+3*ny) = -(1/27);
   rhs(ip) = suptopo(jll+dj+(ill-1-1)*nyl).*2./3;

   %--------------------
   %--- Eastern Edge ---
   %--------------------

   j = 4:3:(ny-2);
   i = nx.*ones(size(j));
   ip = j + (i-1).*ny;
   dj = fix(j./3) - 1;
   diagm1(ip-1) = -2/3;
   diagp2(ip+2) = -1/3;
   rhs(ip) = 0;

   j = 5:3:(ny-1);
   i = nx.*ones(size(j));
   ip = j + (i-1).*ny;
   dj = fix(j./3) - 1;
   diagm2(ip-2) = -1/3;
   diagp1(ip+1) = -2/3;
   rhs(ip) = 0;

   j = 3:3:(ny-3);
   i = nx.*ones(size(j));
   ip = j + (i-1).*ny;
   dj = fix(j./3) - 1;
   diagmnym1(ip-1-ny)    = -(1/27);
   diagm2nym1(ip-1-2*ny) = -(1/27);
   diagm3nym1(ip-1-3*ny) = -(1/27);
   diagmny(ip-ny)        = -(1/27);
   diagm2ny(ip-2*ny)     = -(1/27);
   diagm3ny(ip-3*ny)     = -(1/27);
   diagmnyp1(ip+1-ny)    = -(1/27);
   diagm2nyp1(ip+1-2*ny) = -(1/27);
   diagm3nyp1(ip+1-3*ny) = -(1/27);
   rhs(ip) = suptopo(jll+dj+(iur+1-1)*nyl).*2./3;

   %---------------------
   %--- Southern Edge ---
   %---------------------

   i = 4:3:(nx-4);
   j = ones(size(i));
   ip = j + (i-1).*ny;
   di = fix(i./3) - 1;
   diagmny(ip-ny)    = -2/3;
   diagp2ny(ip+2*ny) = -1/3;
   rhs(ip) = 0;

   i = 5:3:(nx-3);
   j = ones(size(i));
   ip = j + (i-1).*ny;
   di = fix(i./3) - 1;
   diagpny(ip+ny)    = -2/3;
   diagm2ny(ip-2*ny) = -1/3;
   rhs(ip) = 0;

   i = 3:3:(nx-2);
   j = ones(size(i));
   ip = j + (i-1).*ny;
   di = fix(i./3) - 1;
   diagmnyp1(ip+1-ny) = -(1/27);
   diagp1(ip+1)       = -(1/27);
   diagpnyp1(ip+1+ny) = -(1/27);
   diagmnyp2(ip+2-ny) = -(1/27);
   diagp2(ip+2)       = -(1/27);
   diagpnyp2(ip+2+ny) = -(1/27);
   diagmnyp3(ip+3-ny) = -(1/27);
   diagp3(ip+3)       = -(1/27);
   diagpnyp3(ip+3+ny) = -(1/27);
   rhs(ip) = suptopo(jll-1+(ill+di-1)*nyl).*2./3;

   %---------------------
   %--- Northern Edge ---
   %---------------------

   i = 1:3:(nx-1);
   j = ny.*ones(size(i));
   ip = j + (i-1).*ny;
   di = fix(i./3) - 1;
   rhs(ip) = suptopo(jur+1+(ill+di-1)*nyl).*2./3 + ...
             suptopo(jur+1+(ill+di+1-1)*nyl)./3;

   i = 2:3:nx;
   j = ny.*ones(size(i));
   ip = j + (i-1).*ny;
   di = fix(i./3) - 1;
   rhs(ip) = suptopo(jur+1+(ill+di-1)*nyl)./3 + ...
             suptopo(jur+1+(ill+di+1-1)*nyl).*2./3;

   i = 3:3:(nx-2);
   j = ny.*ones(size(i));
   ip = j + (i-1).*ny;
   di = fix(i./3) - 1;
   rhs(ip) = suptopo(jur+1+(ill+di-1)*nyl);

   %------------------------
   %--- SouthWest corner ---
   %------------------------

   j = 1;
   i = 1;
   ip = j + (i-1).*ny;
   diagp2(ip+2) = -1/3;
   rhs(ip) = suptopo(jll-1+(ill-1-1)*nyl).*4./9 + ...
             suptopo(jll-1+(ill-1)*nyl).*2./9;

   j = 2;
   i = 1;
   ip = j + (i-1).*ny;
   diagp1(ip+1) = -2/3;
   rhs(ip) = suptopo(jll-1+(ill-1-1)*nyl).*2./9 + ...
             suptopo(jll-1+(ill-1)*nyl)./9;

   j = 1;
   i = 2;
   ip = j + (i-1).*ny;
   diagpny(ip+ny) = -2/3;
   rhs(ip) = suptopo(jll-1+(ill-1-1)*nyl).*2./9 + ...
             suptopo(jll+(ill-1-1)*nyl)./9;

   %------------------------
   %--- SouthEast corner ---
   %------------------------

   j = 1;
   i = nx;
   ip = j + (i-1).*ny;
   diagp2(ip+2) = -1/3;
   rhs(ip) = suptopo(jll-1+(iur+1-1)*nyl).*4./9 + ...
             suptopo(jll-1+(iur-1)*nyl).*2./9;

   j = 2;
   i = nx;
   ip = j + (i-1).*ny;
   diagp1(ip+1) = -2/3;
   rhs(ip) = suptopo(jll-1+(iur+1-1)*nyl).*2./9 + ...
             suptopo(jll-1+(iur-1)*nyl)./9;

   j = 1;
   i = nx-1;
   ip = j + (i-1).*ny;
   diagmny(ip-ny) = -2/3;
   rhs(ip) = suptopo(jll-1+(iur+1-1)*nyl).*2./9 + ...
             suptopo(jll+(iur+1-1)*nyl)./9;

end

clear i ip j;

%-------------------------------------------------------------------------------
% Apply gradient constraint.
%-------------------------------------------------------------------------------

%-------------------------------------
%--- Compute bounds for gradients. ---
%-------------------------------------

[bndx,bndy,bndxy] = slopebound (topo_old,vertparm,dxu.*metxu,dyu.*metyu, ...
                                bndconst,criteria);

%------------------------
%--- Compute gradient ---
%------------------------

[dtopodx,dtopody] = gradt2u (topo_old,dxu.*metxu,dyu.*metyu);

%----------------------------
%--- Compute test factor. ---
%----------------------------

if (criteria==0),
   testfac  = bndx - abs(dtopodx);
   testfac2 = bndy - abs(dtopody);
   ind = find (testfac2 < testfac);
   if ~isempty(ind),
      testfac(ind) = testfac2(ind);
   end;
   clear testfac2;
  elseif (criteria==1),
   gradtop = sqrt ( dtopodx.*dtopodx + dtopody.*dtopody );
   testfac = bndxy - gradtop;
  else,
   testfac  = (bndx - abs(dtopodx))./bndx;
   testfac2 = (bndy - abs(dtopody))./bndy;
   ind = find (testfac2 < testfac);
   if ~isempty(ind),
      testfac(ind) = testfac2(ind);
   end;
   testfac2 = testfac;
   gradtop   = sqrt ( dtopodx.*dtopodx + dtopody.*dtopody );
   testfacxy = (bndxy - gradtop)./bndxy;
   ind = find (testfacxy < testfac);
   if ~isempty(ind),
      testfac(ind) = testfacxy(ind);
   end;
end;

%-----------------------------------
%--- Find replacement locations. ---
%-----------------------------------

s = [1 1:(ny-1)];
w = [1 1:(nx-1)];

if isempty (addcnst),
   [jrep irep] = find ( ( (testfac<0) |(testfac(s,:)<0) ...
                           | (testfac(:,w)<0)| (testfac(s,w)<0) ) ...
                         & (topo_old<hshllw) & (topo_old>hdeep) );
  else,
   eval (['[jrep irep] = find ( ( (testfac<0) |(testfac(s,:)<0) ' ...
                         ' | (testfac(:,w)<0)| (testfac(s,w)<0) ) ' ...
                         ' & (topo_old<hshllw) & (topo_old>hdeep) ' ...
                         ' & ' addcnst ');' ]);
end;

remind = find ( (jrep==1) | (irep==1) | (jrep==ny) | (irep==nx) );

if ~isempty(remind),
   jrep(remind) = [];
   irep(remind) = [];
end;

ndrp = jrep + (irep-1).*ny;

clear irep jrep remind s w;

%-----------------------------------------
%--- Load Laplacian portion of matrix. ---
%-----------------------------------------

myomxu = metyu./(dxu.*metxu);
mxomyu = metxu./(dyu.*metyu);

diagpnyp1(ndrp+ny+1) = ( myomxu(ndrp)./dxt(ndrp) + ...
                         mxomyu(ndrp)./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagp1(ndrp+1) = ( (mxomyu(ndrp)+mxomyu(ndrp-ny))./dyt(ndrp) - ...
                   (myomxu(ndrp)+myomxu(ndrp-ny))./dxt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagmnyp1(ndrp-ny+1) = ( myomxu(ndrp-ny)./dxt(ndrp) + ...
                         mxomyu(ndrp-ny)./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagpny(ndrp+ny) = ( (myomxu(ndrp)+myomxu(ndrp-1))./dxt(ndrp) - ...
                     (mxomyu(ndrp)+mxomyu(ndrp-1))./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diag0(ndrp) = - ( (myomxu(ndrp)+myomxu(ndrp-1)+myomxu(ndrp-ny)+ ...
                                 myomxu(ndrp-1-ny))./dxt(ndrp) + ...
                  (mxomyu(ndrp)+mxomyu(ndrp-1)+mxomyu(ndrp-ny)+ ...
                                 mxomyu(ndrp-1-ny))./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagmny(ndrp-ny) = ( (myomxu(ndrp-ny)+myomxu(ndrp-ny-1))./dxt(ndrp) - ...
                     (mxomyu(ndrp-ny)+mxomyu(ndrp-ny-1))./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagpnym1(ndrp+ny-1) = ( myomxu(ndrp-1)./dxt(ndrp) + ...
                         mxomyu(ndrp-1)./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagm1(ndrp-1) = ( (mxomyu(ndrp-1)+mxomyu(ndrp-ny-1))./dyt(ndrp) - ...
                   (myomxu(ndrp-1)+myomxu(ndrp-ny-1))./dxt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

diagmnym1(ndrp-ny-1) = ( myomxu(ndrp-ny-1)./dxt(ndrp) + ...
                         mxomyu(ndrp-ny-1)./dyt(ndrp) ) ...
                        ./ (4.*metxt(ndrp).*metyt(ndrp));

clear mxomyu myomxu;

%-----------------------
%--- Bound gradient. ---
%-----------------------

if (criteria==0),
   ind = find (abs(dtopodx) > bndx);
   if ~isempty(ind),
      dtopodx(ind) = bndx(ind).*sign(dtopodx(ind));
   end;
   ind = find (abs(dtopody) > bndy);
   if ~isempty(ind),
      dtopody(ind) = bndy(ind).*sign(dtopody(ind));
   end;
  elseif (criteria==1),
   ind = find (gradtop > bndxy);
   if ~isempty(ind),
      dtopodx(ind) = dtopodx(ind).*bndxy(ind)./gradtop(ind);
      dtopody(ind) = dtopody(ind).*bndxy(ind)./gradtop(ind);
   end;
   clear gradtop;
  else,
   dhx = zeros(size(gradtop));
   dhy = dhx;
   ind = find (gradtop>0);
   if ~isempty(ind),
      dhx(ind) = dtopodx(ind).*bndxy(ind)./gradtop(ind);
      dhy(ind) = dtopody(ind).*bndxy(ind)./gradtop(ind);
   end;

   ind = find (abs(dtopodx) > bndx);
   if ~isempty(ind),
      dtopodx(ind) = bndx(ind).*sign(dtopodx(ind));
   end;
   ind = find (abs(dtopody) > bndy);
   if ~isempty(ind),
      dtopody(ind) = bndy(ind).*sign(dtopody(ind));
   end;

   ind = find ( (gradtop > bndxy) & (testfacxy < testfac2) );
   if ~isempty(ind),
      dtopodx(ind) = dhx(ind);
      dtopody(ind) = dhy(ind);
   end;
   clear gradtop testfac2 testfacxy;
end;

clear bndx bndy bndxy ind;

%----------------------------------------------
%--- Compute forcing for replacement areas. ---
%----------------------------------------------

dhx = metyu.*dtopodx;
dhy = metxu.*dtopody;

rhs(ndrp) = ( ((dhx(ndrp)-dhx(ndrp-ny))+(dhx(ndrp-1)-dhx(ndrp-ny-1))) ...
              ./dxt(ndrp) + ...
              ((dhy(ndrp)-dhy(ndrp-1))+(dhy(ndrp-ny)-dhy(ndrp-ny-1))) ...
              ./dyt(ndrp) ) ...
            ./ (2.*metxt(ndrp).*metyt(ndrp));

clear dhx dhy dtopodx dtopody;

%-------------------------------------------------------------------------------
% Solve conditioning problem.
%-------------------------------------------------------------------------------

if isempty(suptopo)

   CondMat = spdiags ( [diagmnym1 diagmny diagmnyp1 ...
                        diagm1    diag0   diagp1    ...
                        diagpnym1 diagpny diagpnyp1 ], ...
                        [(-ny-1):(1-ny) -1:1 (ny-1):(ny+1)], ...
                        (nx*ny), (nx*ny) );

   clear diagmnym1 diagmny diagmnyp1 ...
         diagm1    diag0   diagp1    ...
         diagpnym1 diagpny diagpnyp1;

 else

   CondMat = spdiags ( [diagm3nym1 diagm3ny diagm3nyp1 ...
                        diagm2nym1 diagm2ny diagm2nyp1 ...
                        diagmnym1 diagmny diagmnyp1 diagmnyp2 diagmnyp3 ...
                        diagm2 diagm1    diag0   diagp1 diagp2 diagp3  ...
                        diagpnym1 diagpny diagpnyp1 diagpnyp2 diagpnyp3 ...
                        diagp2nym1 diagp2ny diagp2nyp1 ...
                        diagp3nym1 diagp3ny diagp3nyp1], ...
                        [(-3*ny-1):(1-3*ny) (-2*ny-1):(1-2*ny) ...
                        (-ny-1):(3-ny) -2:3 (ny-1):(ny+3) ...
                        (2*ny-1):(2*ny+1) (3*ny-1):(3*ny+1)], ...
                        (nx*ny), (nx*ny) );

   clear diagm3nym1 diagm3ny diagm3nyp1 ...
         diagm2nym1 diagm2ny diagm2nyp1 ...
         diagmnym1 diagmny diagmnyp1 diagmnyp2 diagmnyp3 ...
         diagm2 diagm1    diag0   diagp1 diagp2 diagp3  ...
         diagpnym1 diagpny diagpnyp1 diagpnyp2 diagpnyp3 ...
         diagp2nym1 diagp2ny diagp2nyp1 ...
         diagp3nym1 diagp3ny diagp3nyp1;

end;

soln = CondMat\rhs;

topo_new = reshape (soln,ny,nx);
