function [bndx,bndy,bndxy] = slopebound (h,vertparm,dx,dy,bndconst,criteria);
%
% function [bndx,bndy,bndxy] = slopebound (h,vertparm,dx,dy,bndconst,criteria);
%
% This function computes upper bound for the slope under the given criteria.
%
% ------
% Input:
% ------
%
%    h..........topography whose bound is sought.
%    vertparm...Parameters to describe double sigma system.
%                  vertparm(1):  zc1     shallow bound
%                  vertparm(2):  zc2     Deep bound
%                  vertparm(3):  zref    Reference depth.
%                  vertparm(4):  zslope  Slope bounding factor
%                  vertparm(5):  sigres  A measure of reduced sigma thickness
%    dx.........Grid spacing in x-direction.
%    dy.........Grid spacing in y-direction.
%    bndconst...Constant to bound slope factor.
%    criteria...Bounding criteria.
%                  [0]  Bound reduced slope.
%                  [1]  Bound slope.
%                  [2]  Bound both.
%
% -------
% Output:
% -------
%
%    bndx....Slope bound in x-direction.      (reduced slope)
%    bndy....Slope bound in y-direction.      (reduced slope)
%    bndxy...Slope bound in both directions.  (slope)

%-------------------------------------------------------------------------------
% Initialize output.
%-------------------------------------------------------------------------------

bndx  = [];
bndy  = bndx;
bndxy = bndx;

%-------------------------------------------------------------------------------
% Select type of bound.
%-------------------------------------------------------------------------------

if (criteria~=1),

   %----------------------------
   %--- Reduced slope bound. ---
   %---  Compute interface.  ---
   %----------------------------

   favg = 0.5 * sum(vertparm(1:2));
   fdif = 0.5 * diff(vertparm(2:-1:1));

   if (abs(fdif)>0),
      fac = vertparm(4)/fdif;
     else
      fac = 0;
   end;

   iface = favg + fdif .* tanh( fac.*(h-vertparm(3)) );

   %---------------------------------------
   %--- Move fields to velocity points. ---
   %---------------------------------------

   [ny nx] = size(h);

   n = [2:ny ny];
   s = 1:ny;
   e = [2:nx nx];
   w = 1:nx;

   iface = 0.25.*(iface(n,e)+iface(n,w)+iface(s,e)+iface(s,w));
   h     = 0.25.*(h(n,e)+h(n,w)+h(s,e)+h(s,w));

   %----------------------------
   %--- Compute slope bound. ---
   %----------------------------

   bndx = abs(h - iface) ./ (vertparm(5).*dx) .* bndconst(1);
   bndy = abs(h - iface) ./ (vertparm(5).*dy) .* bndconst(1);

end;

if (criteria~=0),

   %----------------------------
   %---     Slope bound.     ---
   %--- Compute slope bound. ---
   %----------------------------

   bndxy = bndconst(length(bndconst)).*ones(size(h));

end;
