function fac = slopefac (h,vertparm,dx,dy,bndconst,criteria);
%
% function fac = slopefac (h,vertparm,dx,dy,bndconst,criteria);
%
% This function computes the slope factor at velocity points from topography
% at tracer points.
%
% ------
% Input:
% ------
%
%    h..........topography whose bound is sought.
%    vertparm...Parameters to describe double sigma system.
%                  vertparm(1):  zc1     shallow bound
%                  vertparm(2):  zc2     Deep bound
%                  vertparm(3):  zref    Reference depth.
%                  vertparm(4):  zslope  Slope bounding factor
%                  vertparm(5):  sigres  A measure of reduced sigma thickness
%    dx.........Grid spacing in x-direction.
%    dy.........Grid spacing in y-direction.
%    bndconst...Constant to bound slope factor.
%    criteria...Bounding criteria.
%                  [0]  Bound reduced slope.
%                  [1]  Bound slope.
%                  [2]  Bound both.
%
% -------
% Output:
% -------
%
%    fac....Slope factor.

%-------------------------------------------------------------------------------
% Get slope bound.
%-------------------------------------------------------------------------------

[bndx,bndy,bndxy] = slopebound (h,vertparm,dx,dy,bndconst,criteria);

%-------------------------------------------------------------------------------
% Get slope.
%-------------------------------------------------------------------------------

[dhdx,dhdy] = gradt2u (h,dx,dy);

%-------------------------------------------------------------------------------
% Compute slope factor.
%-------------------------------------------------------------------------------

if (criteria==0),
   fac  = abs(dhdx)./bndx.*bndconst;
   fac2 = abs(dhdy)./bndy.*bndconst;
   ind  = find (fac2>fac);
   if ~isempty(ind),
      fac(ind) = fac2(ind);
   end
  elseif (criteria==1),
   fac = sqrt( dhdx.*dhdx + dhdy.*dhdy );
  else
   fac  = abs(dhdx)./bndx;
   fac2 = abs(dhdy)./bndy;
   ind  = find (fac2>fac);
   if ~isempty(ind),
      fac(ind) = fac2(ind);
   end
   fac2 = sqrt( dhdx.*dhdx + dhdy.*dhdy )./bndconst(2);
   ind  = find (fac2>fac);
   if ~isempty(ind),
      fac(ind) = fac2(ind);
   end
end;
