      subroutine rmed_filt (ndat,pos,data,nbin,itmax,work,work2,iwrk)
c
c-------------------------------------------------------------------------------
c     This routine applies an N-point median filter to the data.               |
c                                                                              |
c     Calls:  q_init, rcond_rep, rget_val, rinsert, rq_in, rq_out              |
c                                                                              |
c     Arguments:                                                               |
c                                                                              |
c     data....Real vector.  The data to be filtered.   {input/output}          |
c     itmax...Integer.  Maximum number of iterations.   {input}                |
c     iwrk....Integer vector.  Work space of dimension (nbin+1).               |
c     nbin....Integer.  Filter width  (odd integer > 3).   {input}             |
c     ndat....Integer.  Number of data points.   {input}                       |
c     pos.....Real vector.  Positions of data points.   {input}                |
c     work....Real vector.  Work space of dimension (nbin).                    |
c     work2...Real vector.  Work space of dimension ( (nbin+1)/2 ).            |
c-------------------------------------------------------------------------------
c
      integer itmax,ndat,nm1,nm2,iter,i,nin,nbin,nwk,nh,nhm1,qbgn,qend,
     &        iwrk(nbin+1),fbgn,fend,nhm2
      real data(ndat),pos(ndat),work(nbin),d1,dn,p5,dlin,val,slpfac,
     &        work2((nbin+1)/2),slope,denom,small,c1
      logical ever,error
c
      parameter (p5=0.5, slpfac=2.0, small=2.0e-7, c1=1.0)
c
c=======================================================================
c     Begin executable code.                                           #
c=======================================================================
c
c-----------------------------------------------------------------------
c     Only filter data with at least 3 points.                         |
c-----------------------------------------------------------------------
c
      if (ndat.gt.2) then
c
c-----------------------------------------------------------------------
c        Make sure working bin width is an odd integer >= 3.  Compute  |
c        maximum bin width allowed by number of data points.  Take the |
c        minimum of these as the working bin width.  Get half width.   |
c-----------------------------------------------------------------------
c
         nwk = max(2*((nbin+1)/2) - 1 , 3)
         i = max(2*((ndat+1)/2) - 1 , 3)
         nwk = min( nwk, i )
c
         nh = (nwk+1)/2
         nhm1 = nh - 1
         nhm2 = nh - 2
c
c-----------------------------------------------------------------------
c        Initialize working variables.                                 |
c-----------------------------------------------------------------------
c
         d1 = data(1)
         dn = data(ndat)
c
         iter = 0
         ever = .true.
c
         nm1 = ndat - 1
         nm2 = ndat - 2
c
         call q_init (nh,iwrk,qbgn,qend,fbgn,fend,error)
c
c-----------------------------------------------------------------------
c        Repeatedly filter data.                                       |
c-----------------------------------------------------------------------
c
         do 40 while (ever .and. (iter.lt.itmax))
c
c-----------------------------------------------------------------------
c           Filter first point with a 2-point median.                  |
c-----------------------------------------------------------------------
c
            ever = .false.
            call rq_in (data(1),nh,work2,iwrk,qbgn,qend,fbgn,fend,error)
            data(1) = p5*(data(1) + data(2))
c
c-----------------------------------------------------------------------
c           Fan out the median filter width.                           |
c-----------------------------------------------------------------------
c
            do 10 i = 2, nhm1
c
               call rq_in (data(i),nh,work2,iwrk,qbgn,qend,fbgn,fend,
     &                                                            error)
               call rget_val (nh,work2,iwrk,qbgn,qend,ndat,data,i,nwk,
     &                                                   work,val,error)
               call rcond_rep (data(i),val,ever)
c
 10         continue
c
c-----------------------------------------------------------------------
c           Filter with the desired width.                             |
c-----------------------------------------------------------------------
c
c
            do 20 i = nh, ndat-nhm1
c
               call rq_in (data(i),nh,work2,iwrk,qbgn,qend,fbgn,fend,
     &                                                            error)
               call rget_val (nh,work2,iwrk,qbgn,qend,ndat,data,i,nwk,
     &                                                   work,val,error)
               call rcond_rep (data(i),val,ever)
               call rq_out (val,nh,work2,iwrk,qbgn,qend,fbgn,fend,error)
c
 20         continue
c
c-----------------------------------------------------------------------
c           Fan in the median filter width.                            |
c-----------------------------------------------------------------------
c
            do 30 i = ndat-nhm2, nm1
c
               call rq_out (val,nh,work2,iwrk,qbgn,qend,fbgn,fend,error)
               call rq_in (data(i),nh,work2,iwrk,qbgn,qend,fbgn,fend,
     &                                                            error)
               call rget_val (nh,work2,iwrk,qbgn,qend,ndat,data,i,nwk,
     &                                                   work,val,error)
               call rcond_rep (data(i),val,ever)
               call rq_out (val,nh,work2,iwrk,qbgn,qend,fbgn,fend,error)
c
 30         continue
c
c-----------------------------------------------------------------------
c           Filter last point with a 2-point median.                   |
c-----------------------------------------------------------------------
c
            call rq_out (val,nh,work2,iwrk,qbgn,qend,fbgn,fend,error)
            data(ndat) = p5*(val + data(ndat))
            iter = iter + 1
c
c-----------------------------------------------------------------------
c           Check endpoints to see if they can change the next point in|
c-----------------------------------------------------------------------
c
            if ( ((data(2)-data(1))*(data(2)-data(3))) .gt. 0 )
     &         ever=.true.
            if ( ((data(nm1)-data(ndat))*(data(nm1)-data(nm2))) .gt. 0 )
     &         ever=.true.
c
 40      continue
c
c-----------------------------------------------------------------------
c        Handle end points.                                            |
c-----------------------------------------------------------------------
c
         denom = pos(3)-pos(2)
         if (abs(denom).lt.small) denom = c1
         slope = (data(3)-data(2))/denom * slpfac
         dlin = slope*(pos(1)-pos(2)) + data(2)
         nin = 0
         call rinsert (dlin,work,nin,3,error)
         call rinsert (d1,work,nin,3,error)
         call rinsert (data(2),work,nin,3,error)
         data(1) = work(2)
c
         denom = pos(nm1)-pos(nm2)
         if (abs(denom).lt.small) denom = c1
         slope = (data(nm1)-data(nm2))/denom * slpfac
         dlin = slope*(pos(ndat)-pos(nm1)) + data(nm1)
         nin = 0
         call rinsert (dlin,work,nin,3,error)
         call rinsert (dn,work,nin,3,error)
         call rinsert (data(nm1),work,nin,3,error)
         data(ndat) = work(2)
c
      end if
c
      return
      end
