      subroutine chk_cdf (fname,coord,no_x,no_y,dx,dy,lonc,latc,delx,
     &                    dely,rotang,no_lvl,hybrid)
c
c=======================================================================
c                                                                    ===
c  This routine opens the indicated netCDF file and checks the given ===
c  model domain parameters with those already in the file.           ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     COORD    Flag for type of grid.                     (integer)  ===
c                 [0]  cartesian (tangent plane) grid.               ===
c                 [1]  geographic spherical grid.                    ===
c                 [2]  rotated spherical grid.                       ===
c     DELX,                                                          ===
c     DELY     Offset from transformation point to                   ===
c              grid center.                               (real)     ===
c     DX       The mean grid spacing in the x-direction.  (real)     ===
c     DY       The mean grid spacing in the y-direction.  (real)     ===
c     FNAME    The name of the netCDF file.               (character)===
c     HYBRID   Type of vertical system.                   (logical)  ===
c     LONC     The central longitude of the domain.       (real)     ===
c     LATC     The central latitude of the domain.        (real)     ===
c     NO_LVL   The number of vertical levels.             (integer)  ===
c     NO_X     The number of grids points in x-direction. (integer)  ===
c     NO_Y     The number of grids points in y-direction. (integer)  ===
c     ROTANG   The central rotation angle of the domain.  (real)     ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     NCOUTID   Identifier number for netCDF output file.   (integer)===
c     NCSTAT    Status flag for netCDF output file.         (integer)===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    GR_EXIT,          LENGTH                              ===
c     netCDF:  NF_OPEN,          NF_GET_VAR_TEXT,  NF_GET_VAR_INT,   ===
c              NF_GET_VAR_REAL,  NF_INQ_VARID                        ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
c
#include <gpconst.h>
#include <gridsio.h>
#include <netcdf.inc>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      real
     &        delx,dely,dx,dy,lonc,latc,rotang,rdum,tol,xdelx,xdely,
     &        xlatc,xlonc
      integer coord,idum,no_lvl,no_x,no_y,sbgn,send,slen,varid
      logical     hybrid,ldum,old
      character*1   cdum
      character*(*) fname
c
      parameter (tol=eps*(c1+eps))
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Open netCDF file.
c-----------------------------------------------------------------------
c
c  Make sure supplied name is valid.
c
      call length (fname,slen,sbgn,send)
c
      if (slen.lt.1) then
         write (out_unit,5)
         call gr_exit ('CHK_CDF')
      end if
c
c  Open netCDF file.
c
      if (nf_open(fname(sbgn:send),NF_WRITE,ncoutid) .eq. NF_NOERR) then
         ncstat = 2
       else
         write (out_unit,10) fname(sbgn:send)
         call gr_exit ('CHK_CDF')
      end if
c
c-----------------------------------------------------------------------
c  Check model domain parameters.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncoutid,'coord',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'coord'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_int (ncoutid,varid,idum) .ne. NF_NOERR) then
         write (out_unit,30) 'coord'
         call gr_exit ('CHK_CDF')
       elseif (coord.ne.idum) then
         write (out_unit,40) 'coord',coord,idum
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'imt',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'imt'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_int (ncoutid,varid,idum) .ne. NF_NOERR) then
         write (out_unit,30) 'imt'
         call gr_exit ('CHK_CDF')
       else if (no_x.ne.idum) then
         write (out_unit,40) 'no_x',no_x,idum
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'jmt',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'jmt'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_int (ncoutid,varid,idum) .ne. NF_NOERR) then
         write (out_unit,30) 'jmt'
         call gr_exit ('CHK_CDF')
       else if (no_y.ne.idum) then
         write (out_unit,40) 'no_y',no_y,idum
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'meandx',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'meandx'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_real (ncoutid,varid,rdum) .ne. NF_NOERR) then
         write (out_unit,30) 'meandx'
         call gr_exit ('CHK_CDF')
       else if (abs(dx-rdum).gt.tol*abs(dx)) then
         write (out_unit,50) 'dx',dx,rdum
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'meandy',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'meandy'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_real (ncoutid,varid,rdum) .ne. NF_NOERR) then
         write (out_unit,30) 'meandy'
         call gr_exit ('CHK_CDF')
       else if (abs(dy-rdum).gt.tol*abs(dy)) then
         write (out_unit,50) 'dy',dy,rdum
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'thetad',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'thetad'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_real (ncoutid,varid,rdum) .ne. NF_NOERR) then
         write (out_unit,30) 'thetad'
         call gr_exit ('CHK_CDF')
       else if (abs(rotang-rdum).gt.tol*abs(rotang)) then
         write (out_unit,50) 'rotang',rotang,rdum
         call gr_exit ('CHK_CDF')
      end if
c
c  The remaining horizontal parameters are checked as a group
c  for backwards compatibility with pre-6.1 versions.
c
      if (nf_inq_varid (ncoutid,'rlngd',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'rlngd'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_real (ncoutid,varid,xlonc) .ne. NF_NOERR) then
         write (out_unit,30) 'rlngd'
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'rlatd',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'rlatd'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_real (ncoutid,varid,xlatc) .ne. NF_NOERR) then
         write (out_unit,30) 'rlatd'
         call gr_exit ('CHK_CDF')
      end if
c
      old = .false.
c
      if (nf_inq_varid (ncoutid,'delx',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncoutid,varid,xdelx) .ne. NF_NOERR) then
            write (out_unit,30) 'delx'
            call gr_exit ('CHK_CDF')
         end if
       else
         old = .true.
      end if
c
      if (nf_inq_varid (ncoutid,'dely',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncoutid,varid,xdely) .ne. NF_NOERR) then
            write (out_unit,30) 'dely'
            call gr_exit ('CHK_CDF')
          end if
       else
         old = .true.
      end if
c
      if (old) then
         if (coord.ne.1) then
            xdelx = c0
            xdely = c0
          else
            xdelx = xlonc
            xdely = xlatc
            xlonc = c0
            xlatc = c0
         end if
      end if
c
      if (abs(lonc-xlonc).gt.tol*abs(lonc)) then
         write (out_unit,50) 'lonc',lonc,xlonc
         call gr_exit ('CHK_CDF')
      end if
c
      if (abs(latc-xlatc).gt.tol*abs(latc)) then
         write (out_unit,50) 'latc',latc,xlatc
         call gr_exit ('CHK_CDF')
      end if
c
      if (abs(delx-xdelx).gt.tol*abs(delx)) then
         write (out_unit,50) 'delx',delx,xdelx
         call gr_exit ('CHK_CDF')
      end if
c
      if (abs(dely-xdely).gt.tol*abs(dely)) then
         write (out_unit,50) 'dely',dely,xlatc
         call gr_exit ('CHK_CDF')
      end if
c
c-----------------------------------------------------------------------
c  Check vertical parameters.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncoutid,'km',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'km'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_int (ncoutid,varid,idum) .ne. NF_NOERR) then
         write (out_unit,30) 'km'
         call gr_exit ('CHK_CDF')
       elseif (no_lvl.ne.idum) then
         write (out_unit,40) 'km',no_lvl,idum
         call gr_exit ('CHK_CDF')
      end if
c
      if (nf_inq_varid (ncoutid,'hybrid',varid) .ne. NF_NOERR) then
         write (out_unit,20) 'hybrid'
         call gr_exit ('CHK_CDF')
      end if
      if (nf_get_var_text (ncoutid,varid,cdum) .ne. NF_NOERR) then
         write (out_unit,30) 'hybrid'
         call gr_exit ('CHK_CDF')
       else
         read (cdum,60) ldum
         if ((hybrid.or.ldum) .and. (.not.(hybrid.and.ldum)) ) then
            write (out_unit,70) 'hybrid',hybrid,ldum
            call gr_exit ('CHK_CDF')
         end if
      end if
c
      return
c
  5   format (/'*** Error:  CHK_CDF - invalid (empty) file name')
 10   format (/'*** Error:  CHK_CDF - unable to open file ',a)
 20   format (/'*** Error:  CHK_CDF - unable to find variable ',
     &        1h',a,1h')
 30   format (/'*** Error:  CHK_CDF - unable to read variable ',
     &        1h',a,1h')
 40   format (/'*** Error:  CHK_CDF - incompatible values for ',
     &        1h',a,1h'/12x,'input card:  ',i10/12x,'input file:  ',i10)
 50   format (/'*** Error:  CHK_CDF - incompatible values for ',
     &        1h',a,1h'/12x,'input card:  ',1pg15.8/12x,'input file:  ',
     &        1pg15.8)
 60   format (l1)
 70   format (/'*** Error:  CHK_CDF - incompatible values for ',
     &        1h',a,1h'/12x,'input card:  ',l1/12x,'input file:  ',l1)
c
      end
