      subroutine comp_fne
c
c=======================================================================
c                                                                    ===
c  This routine computes the fine grid domain definition parameters. ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /CRS_DAT/                                                         ===
c                                                                    ===
c     COORD     Grid type.                            (integer)      ===
c                  [0] cartesian                                     ===
c                  [1] geographic spherical                          ===
c                  [2] rotated spherical                             ===
c     DELX_C    X-offset, grid-trans. centers.        (real; m | deg)===
c     DELY_C    Y-offset, grid-trans. centers.        (real; m | deg)===
c     GRIDX_C   Grid spacing in x-direction.          (real; m | deg)===
c     GRIDY_C   Grid spacing in y-direction.          (real; m | deg)===
c     IMT_C     Number of grid points in x direction. (integer)      ===
c     JMT_C     Number of grid points in y direction. (integer)      ===
c     LATC      Latitude of center of transformation. (real; deg)    ===
c     LONC      Longitude of center of transformation.(real; deg)    ===
c     THETA     Angle grid is rotated WRT East.       (real; deg)    ===
c                                                                    ===
c  /FNE_DAT/                                                         ===
c                                                                    ===
c     IMT_F   Number of grid points in x direction,                  ===
c             fine grid.                                (integer)    ===
c     JMT_F   Number of grid points in y direction,                  ===
c             fine grid.                                (integer)    ===
c     LAT_F   Latitude of center of grid, fine grid.    (real; deg)  ===
c     LON_F   Longitude of center of grid, fine grid.   (real; deg)  ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /FNE_DAT/                                                         ===
c                                                                    ===
c     DDELX    Needed shift to collocate in x-direction. (real;m|deg)===
c     DDELY    Needed shift to collocate in y-direction. (real;m|deg)===
c     DELX_F   X-offset, grid-trans. centers, fine grid. (real;m|deg)===
c     DELY_F   Y-offset, grid-trans. centers, fine grid. (real;m|deg)===
c     GRIDX_F  Grid spacing in x-direction, fine grid.   (real;m|deg)===
c     GRIDY_F  Grid spacing in y-direction, fine grid.   (real;m|deg)===
c     I_LL_C,                                                        ===
c     J_LL_C   Lower left crnr of fine grid in crs grid. (integer)   ===
c     I_UR_C,                                                        ===
c     J_UR_C   Upper right crnr of fne grid in crs grid. (integer)   ===
c     IMT_FC   Number of nested coarse grid x-points.    (integer)   ===
c     JMT_FC   Number of nested coarse grid y-points.    (integer)   ===
c     WKSP     Data passing work space size.             (integer)   ===
c                                                                    ===
c  Calls:  LL2XY                                                     ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <crs_dat.h>
#include <fne_dat.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      real c1,p5,r3,r6
      parameter (c1=1.0, p5=0.5, r3=1.0/3.0, r6=p5*r3)
c
      real x1,xc,y1,yc
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Compute grid spacing in fine grid.
c-----------------------------------------------------------------------
c
      gridx_f = gridx_c*r3
      gridy_f = gridy_c*r3
c
c-----------------------------------------------------------------------
c  Compute offset to properly nest grids.
c-----------------------------------------------------------------------
c
      call ll2xy (lon_f,lat_f,coord,imt_c,jmt_c,gridx_c,gridy_c,lonc,
     &            latc,delx_c,dely_c,theta,x1,y1)
c
      ddelx = (dble(nint(x1)) - dble(x1))*dble(gridx_c)
      ddely = ((dble(nint(y1))+dble(r6)) - dble(y1))*dble(gridy_c)
c
      xc = float(imt_c+1)*p5
      yc = float(jmt_c+1)*p5
c
      delx_f =(dble(x1)-dble(xc))*dble(gridx_c)+dble(ddelx)+dble(delx_c)
      dely_f =(dble(y1)-dble(yc))*dble(gridx_c)+dble(ddely)+dble(dely_c)
c
c-----------------------------------------------------------------------
c  Compute additional output information.
c-----------------------------------------------------------------------
c
      x1 = float(nint(x1))
      y1 = float(nint(y1))
c
      i_ll_c = nint( x1-(float(imt_f+1)*r6)+c1  )
      j_ll_c = nint( y1-(float(jmt_f)*r6   -c1) )
      i_ur_c = nint( x1+(float(imt_f+1)*r6)-c1  )
      j_ur_c = nint( y1+(float(jmt_f)*r6   -c1) )
c
      imt_fc = i_ur_c - i_ll_c + 1
      jmt_fc = j_ur_c - j_ll_c + 1
c
      wksp = max( imt_fc*jmt_fc, imt_f, jmt_f )
c
      return
      end
