      subroutine depth_init (topid,nlon,lonbgn,dlon,lonend,nlat,latbgn,
     &                                             dlat,latend,periodic)
c
c-----------------------------------------------------------------------
c     This routine initializes the positional information for the      |
c     topographic extraction.                                          |
c                                                                      |
c     Calls:  GR_EXIT, LENGTH, POS_INQ, SET_BLANK                      |
c                                                                      |
c     netCDF Calls:  NCAGTC, NCDINQ, NCVID, NCVINQ                     |
c                                                                      |
c     ------                                                           |
c     Input:                                                           |
c     ------                                                           |
c                                                                      |
c     Common Blocks:                                                   |
c                                                                      |
c     /GRIDSIO/                                                        |
c                                                                      |
c     nctinid.......Identifier for topography archive file. (integer)  |
c     topo_infile...Name of topography archive file.        (character)|
c                                                                      |
c     -------                                                          |
c     Output:                                                          |
c     -------                                                          |
c                                                                      |
c     topid......Identifier for topography variable.     (integer)     |
c     nlon.......Number of longitudinal grid points.     (integer)     |
c     lonbgn.....Starting longitude.                     (deg; real)   |
c     dlon.......Longitude step size.                    (deg; real)   |
c     lonend.....Ending longitude.                       (deg; real)   |
c     nlat.......Number of latitudinal grid points.      (integer)     |
c     latbgn.....Starting latitude.                      (deg; real)   |
c     dlat.......Latitude step size.                     (deg; real)   |
c     latend.....Ending latitude.                        (deg; real)   |
c     periodic...Flag for periodic longitudinal extent.  (logical)     |
c-----------------------------------------------------------------------
c
c-----------------------------------------------------------------------
c     Define global data.
c-----------------------------------------------------------------------
c
#include <gridsio.h>
#include <gpconst.h>
#include <netcdf.inc>
c
c-----------------------------------------------------------------------
c     Define local data.
c-----------------------------------------------------------------------
c
      real    c360,dlat,dlon,latbgn,latend,lonbgn,lonend,tol
      integer dims(maxvdims),dimsize,err,n,natt,ndim,nlat,nlon,sb2,sbgn,
     &        se2,send,slen,topid,type
      logical       ctest,periodic
      character*128 name
c
      parameter (c360=360.0, tol=eps*(c1+eps))
c
c=======================================================================
c     Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c     Get topography identifier.
c-----------------------------------------------------------------------
c
      topid = ncvid (nctinid,'topo',err)
      if (err.ne.0) then
         call length (topo_infile,slen,sbgn,send)
         write (out_unit,20) 'topo',topo_infile(sbgn:send)
         call gr_exit ('DEPTH_INIT')
      end if
c
c-----------------------------------------------------------------------
c     Get size information on the topographic data.
c-----------------------------------------------------------------------
c
      call set_blank (name)
      call ncvinq (nctinid,topid,name,type,ndim,dims,natt,err)
      if (err.ne.0) then
         call length (topo_infile,slen,sbgn,send)
         write (out_unit,30) 'topo',topo_infile(sbgn:send)
         call gr_exit ('DEPTH_INIT')
      end if
c
      do 10 n = 1, ndim
c
         call set_blank (name)
         call ncdinq (nctinid,dims(n),name,dimsize,err)
         if (err.ne.0) then
            call length (topo_infile,slen,sbgn,send)
            write (out_unit,40) dims(n),topo_infile(sbgn:send)
            call gr_exit ('DEPTH_INIT')
         end if
c
         call length (name,slen,sbgn,send)
         if (index(name,'lon').ne.0) then
            nlon=dimsize
           else if (index(name,'lat').ne.0) then
            nlat=dimsize
         end if
c
 10   continue
c
c-----------------------------------------------------------------------
c     Get the position information.
c-----------------------------------------------------------------------
c
c     Try to find positions of data from the "positions" attribute.
c
      call set_blank (name)
      call ncpopt (0)
      call ncagtc (nctinid,topid,'positions',name,128,err)
      call ncpopt (ncverbos)
c
c     Determine compactness of positions data.
c
      if (err.eq.0) then
         ctest = index(name,'compact') .ne. 0
        else
         ctest = .false.
      end if
c
c     Get positions information either from attribute or default names.
c
      if (err.eq.0) then
c
c        Parse positions variables names and read data.
c
         call length (name,slen,sbgn,send)
         se2 = index(name(sbgn:send),',') + sbgn - 2
         if (index(name(sbgn:se2),'lon').ne.0) then
            call pos_inq (nctinid,topo_infile,name(sbgn:se2),ctest,
     &                                                      lonbgn,dlon)
           else
            call pos_inq (nctinid,topo_infile,name(sbgn:se2),ctest,
     &                                                      latbgn,dlat)
         end if
c
         sb2 = index(name(sbgn:send),';') + sbgn
         call length (name(sb2:send),slen,sbgn,se2)
         sb2 = sb2 + sbgn - 1
         se2 = index(name(sb2:send),',') + sb2 - 2
         if (index(name(sb2:se2),'lon').ne.0) then
            call pos_inq(nctinid,topo_infile,name(sb2:se2),ctest,lonbgn,
     &                                                             dlon)
           else
            call pos_inq(nctinid,topo_infile,name(sb2:se2),ctest,latbgn,
     &                                                             dlat)
         end if
c
        else
c
c        Use default names for positions
c
         call pos_inq (nctinid,topo_infile,'topo_lon',ctest,lonbgn,dlon)
         call pos_inq (nctinid,topo_infile,'topo_lat',ctest,latbgn,dlat)
c
      end if
c
c     Compute ending points.
c
      lonend = lonbgn + float(nlon-1)*dlon
      latend = latbgn + float(nlat-1)*dlat
c
c-----------------------------------------------------------------------
c     Determine if longitudinal extent covers the globe.
c-----------------------------------------------------------------------
c
      periodic = (c360 - float(nlon)*dlon) .lt. tol
c
      return
c
 20   format ('*** Error:  DEPTH_INIT - unable to find variable ',1h",
     &        a,1h"/12x,'in file ',1h",a,2h".)
 30   format ('*** Error:  DEPTH_INIT - unable to inquire about',
     &        ' variable ',1h",a,1h"/12x,'in file ',1h",a,2h".)
 40   format ('*** Error:  DEPTH_INIT - unable to inquire about',
     &        ' dimension ',i10/12x,'in file ',1h",a,2h".)
c
      end
