      function gcircle (lon1,lat1,lon2,lat2,dist,bearing)
c
c=======================================================================
c                                                                    ===
c  This routine computes great circle distance and initial bearing   ===
c  between two longitude and latitude points. The Earth is assumed   ===
c  to be an sphere. This approximation is valid for short distances. ===
c  Short, but not too short.  For small distances, the routine       ===
c  reverts to a tangent plane approximation for numerical accuracy.  ===
c  The square of the cut-off distance between the two algorithms is  ===
c                 2*sqrt(epsilon)                                    ===
c  where epsilon is the machine precision.                           ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     LON1     Longitude point 1.   (decimal degrees East; real)     ===
c     LAT1     Latitude  point 1.   (decimal degrees North; real)    ===
c     LON2     Longitude point 2.   (decimal degrees East; real)     ===
c     LAT2     Latitude  point 2.   (decimal degrees North; real)    ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     BEARING  Angle between points.      (decimal degrees; real)    ===
c              Counterclockwise, 1-2.
c     GCIRCLE  Great circle distance.     (decimal degrees; real)    ===
c     DIST     Great circle distance.     (kilometers; real)         ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     none                                                           ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      logical first
      real alpha,ang,bearing,c0,c2,deg2rad,dist,dtng,dtngx,dtngy,elat,
     &      elon,gcircle,lat1,lat2,lon1,lon2,p5,phi,radius,rad2deg,slat,
     &                slon,small,tng_cut
      double precision pi
c
      parameter (c0=0.0,c2=2.0,p5=0.5,radius=6371.315,
     &            small=1.0842022e-19,tng_cut=6.90533966e-4)
c
c The cut-off distance is computed from epsilon=2**(-23).
c
      save deg2rad,first,pi,rad2deg
c
      data first /.true./
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Set often used parameters, on first call.
c
      if (first) then
         pi=4.d0*datan(1.d0)
         deg2rad=pi/180.
         rad2deg=180./pi
         first=.false.
      endif
c
c  Convert to radians.
c       
      slon=lon1*deg2rad
      slat=lat1*deg2rad
      elon=lon2*deg2rad
      elat=lat2*deg2rad
c
c----------------------------------------------------------------------- 
c  Compute tangent plane approximation to angular distance.
c----------------------------------------------------------------------- 
c
      dtngx = elon-slon
      dtngy = elon-(slon+c2*pi)
      if (abs(dtngx).le.abs(dtngy)) then
         dtngx = cos(p5*(slat+elat))*dtngx
        else
         dtngx = cos(p5*(slat+elat))*dtngy
      end if
      dtngy = elat-slat
      dtng  = dtngx*dtngx + dtngy*dtngy
c
c----------------------------------------------------------------------- 
c  If the approximate distance is greater than the cutoff value, use
c  full spherical formulae.  Otherwise use tangent plane approximation
c  for numerical accuracy.
c----------------------------------------------------------------------- 
c
      if (dtng.gt.tng_cut) then
c
c----------------------------------------------------------------------- 
c  Compute distance along great circle (degrees and kilometers).
c-----------------------------------------------------------------------
c
         alpha=sin(slat)*sin(elat)+cos(slat)*cos(elat)*cos(elon-slon)
         if (abs(alpha).gt.1.) alpha=sign(1.,alpha)
         alpha=acos(alpha)      
         gcircle=alpha*rad2deg
         dist=radius*alpha
c
c-----------------------------------------------------------------------
c  Compute bearing angle: angle from the first point to the second 
c  point clockwise from true NORTH.
c-----------------------------------------------------------------------
c
         phi=cos(elat)*sin(elon-slon)
         if (sin(alpha).ne.0.) then
           ang=phi/sin(alpha)
           if (abs(ang).gt.1.) ang=sign(1.,ang)
           ang=asin(ang)*rad2deg
         else
           if (phi.eq.0.) then
             ang=0.
           elseif (phi.gt.0.) then
             ang=90.
           else 
             ang=-90.
           endif
         endif
         if (elat.lt.slat) ang=180.-ang
         bearing=mod(ang+360.,360.)
c
c-----------------------------------------------------------------------
c  Compute bearing from point 1 to point 2 counterclockwise from
c  true EAST.
c-----------------------------------------------------------------------
c
         if ((0.0.le.bearing).and.(bearing.le.90.)) then
           bearing=90.-bearing
           if (alpha.eq.0.) bearing=0.
         else
           bearing=mod(450.-bearing,360.)
         endif
c
c-----------------------------------------------------------------------
c  Use tangent plane approximation to preserve numerical accuracy for
c  small distances.
c-----------------------------------------------------------------------
c
       else
c
c-----------------------------------------------------------------------
c  Compute tangent plane distances.
c-----------------------------------------------------------------------
c
         dtng    = sqrt(dtng)
         gcircle = dtng*rad2deg
         dist    = radius*dtng
c
c-----------------------------------------------------------------------
c  Compute tangent plane bearing.
c-----------------------------------------------------------------------
c
         if (dtng.gt.small) then
            dtngx   = dtngx/dtng
            dtngy   = dtngy/dtng
            bearing = atan2(dtngy,dtngx)*rad2deg
           else
            bearing = c0
         end if
      end if
c
      return 
      end
