      subroutine get_data (gridfile,tlon,tlat,blon,blat,hybrid,
     &                     kc,optimum,no_lvl,dzt,new_topo,target,
     &                    no_cl,clvl,clmin,clint,block_cl,hno_cl,hclvl,
     &             hclmin,hclint,hblock_cl,plot_grid,cont_top,plot_step)
c
c=======================================================================
c                                                                    ===
c  This subroutine gets the data needed to rotate and size the grid, ===
c  as well as the data needed to plot the results.                   ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  BLOCK_CL      Switch to block the drawing of contour              ===
c                lines through the high/low labels.   (logical)      ===
c                   [T]  block lines.                                ===
c                   [F]  do NOT block.                               ===
c  BLON, BLAT    Longitude & latitude of the lower                   ===
c                right corner of the plot window.     (real; degrees)===
c  CLINT         Contour interval.                    (real; m)      ===
c                (only active if no_cl > 0)                          ===
c  CLMIN         Minimum contour level.               (real; m)      ===
c                (only active if no_cl > 0)                          ===
c  CLVL          Contour levels for topography.       (real; m)      ===
c                (only active if no_cl > 0)                          ===
c  CONT_TOP      Indicates whether or not to plot a                  ===
c                contour of the topography.           (logical)      ===
c                   [T]  plot topography contours                    ===
c                   [F]  do NOT plot                                 ===
c  DZT           Vertical grid spacing.               (real array; m)===
c  GRIDFILE      The name of the output file to                      ===
c                contain the grid information.        (character)    ===
c  HBLOCK_CL     BLOCK_CL for hydrostat. consist. fac.(logical)      ===
c  HCLINT        CLINT for hydrostatic consist. fact. (real)         ===
c  HCLMIN        CLMIN for hydrostatic consist. fact. (real)         ===
c  HCLVL         CLVL for hydrostatic consist. fact.  (real)         ===
c  HNO_CL        NO_CL for hydrostatic consist. fact. (integer)      ===
c  HYBRID        Indicates whether a hybrid or step                  ===
c                 system isused in the vertical.      (logical)      ===
c                   [T]  hybrid vertical system                      ===
c                   [F]  step vertical system                        ===
c  KC            Level at which to interface systems. (integer)      ===
c  NEW_TOPO      Switch to generate a new topography                 ===
c                file at the T grid points.           (logical)      ===
c  NO_CL         Number of contour levels.            (integer)      ===
c  NO_LVL        Number of vertical model levels.     (integer)      ===
c  OPTIMUM       Switch to "optimize" sigma.          (logical)      ===
c                   [T]  optimize sigma system.                      ===
c                   [F]  do NOT optimize.                            ===
c  PLOT_GRID     Switch fo plottomg the grid.         (logical)      ===
c                   [T]  plot the grid                               ===
c                   [F]  do NOT plot                                 ===
c  PLOT_STEP     Switch for auxillary plotting.       (logical)      ===
c                   [T]  plot the lines                              ===
c                   [F]  do NOT plot                                 ===
c  TARGET        Indicates whether making "target"                   ===
c                topography.                          (logical)      ===
c  TLON, TLAT    The longitude & latitude of the upper               ===
c                left corner of the plot window.      (real; degrees)===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /DOM/                                                             ===
c                                                                    ===
c     COORD    Horizontal coordinate type flag.          (integer)   ===
c                   [0]  cartesian grid.                             ===
c                   [1]  geographic spherical grid.                  ===
c                   [2]  rotated spherical grid.                     ===
c     DELX     X-offset between grid & trans. centers.   (real)      ===
c     DX       Grid x spacing.                           (real)      ===
c     DELY     Y-offset between grid & trans. centers.   (real)      ===
c     DY       Grid y spacing.                           (real)      ===
c     LATC     Latitude of transformation center.        (real)      ===
c     LONC     Longitude of transformation center.       (real)      ===
c     NO_X     Number of elements in the x direction.    (integer)   ===
c     NO_Y     Number of elements in the y direction.    (integer)   ===
c     ROTANG   Rotation angle.                           (real)      ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     CST_FILE      Name of coastline netCDF file.          (string) ===
c     TOPO_INFILE   Name of topography archive file.        (string) ===
c                                                                    ===
c  /SEARCH/                                                          ===
c                                                                    ===
c     HRADX   RADX for hydrostatic consistency factor.   (integer)   ===
c     HRADY   RADY for hydrostatic consistency factor.   (integer)   ===
c     RADX    Topography high/low search radius in x.    (integer)   ===
c     RADY    Topography high/low search radius in y.    (integer)   ===
c                                                                    ===
c  /TITLES/                                                          ===
c                                                                    ===
c     CNTR_TTL   Title of the contour plot.           (character)    ===
c     GRD_TTL    Title of the grid plot.              (character)    ===
c     STP_TTL    Title of the step topography plot.   (character)    ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    CHK_CDF,  CHK_TFILE,  CONT_DATA,  DEFCDF,  GET_VERT   ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <gridsio.h>
#include <gridsparm.h>
#include <gpconst.h>
#include <dom.h>
#include <search.h>
#include <titles.h>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      real
     &      blat,blon,clint,clmin,hclint,hclmin,hclvl,tlat,tlon
      real
     &        clvl(max_cl),dzt(mx_lvl)
      integer hno_cl,kc,no_cl,no_lvl
      logical block_cl,cont_top,hblock_cl,hybrid,new_topo,optimum,
     &              plot_grid,plot_step,target
      character*(*) gridfile
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Get grid size & rotation data.
c-----------------------------------------------------------------------
c
      read (in_unit,10) gridfile
      write (out_unit,20) gridfile
      read (in_unit,*) coord
      read (in_unit,*) dx
      read (in_unit,*) dy
      read (in_unit,*) lonc
      read (in_unit,*) latc
      read (in_unit,*) delx
      read (in_unit,*) dely
      read (in_unit,*) rotang
      read (in_unit,*) no_x
      read (in_unit,*) no_y
c
      if (coord.eq.0) then
         write (out_unit,30) coord,dx,dy,lonc,latc,delx,dely,rotang,
     &                       no_x,no_y
        else if (coord.eq.1) then
         write (out_unit,35) coord,dx,dy,lonc,latc,delx,dely,rotang,
     &                       no_x,no_y
        else
         write (out_unit,40) coord,dx,dy,lonc,latc,delx,dely,rotang,
     &                       no_x,no_y
      end if
c
      call get_vert (no_lvl,dzt)
c
      read (in_unit,*) hybrid
      write (out_unit,50) hybrid
      if (hybrid) then
         read (in_unit,*) kc
         read (in_unit,*) optimum
         write (out_unit,60) kc,optimum
      end if
c
c-----------------------------------------------------------------------
c  Get plotting data.
c-----------------------------------------------------------------------
c
      read (in_unit,*) tlon
      read (in_unit,*) tlat
      read (in_unit,*) blon
      read (in_unit,*) blat
      write (out_unit,70) tlon,tlat,blon,blat
c
      read (in_unit,10) cst_file
      write (out_unit,80) cst_file
c
c-----------------------------------------------------------------------
c  Find out whether to generate a new topography file from archived
c  data or to rework an old topography file.
c-----------------------------------------------------------------------
c
      read (in_unit,*) new_topo
      read (in_unit,*) target
      read (in_unit,10) topo_infile
      if (new_topo) then
         write (out_unit,90) new_topo,topo_infile
        else
         write (out_unit,100) new_topo,gridfile
      end if
c
c-----------------------------------------------------------------------
c  Read in information to determine contour levels for plotting
c  topography.
c-----------------------------------------------------------------------
c
      write (out_unit,110)
      call cont_data (max_cl,no_cl,block_cl,radx,rady,clvl,clmin,clint)
c
      if (hybrid) then
         write (out_unit,120)
         call cont_data (max_cl,hno_cl,hblock_cl,hradx,hrady,hclvl,
     &                                                    hclmin,hclint)
      end if
c
c-----------------------------------------------------------------------
c  Determine which plots should actually be done.
c-----------------------------------------------------------------------
c
      read (in_unit,*) plot_grid
      read (in_unit,10) grd_ttl
c
      read (in_unit,*) cont_top
      read (in_unit,10) cntr_ttl
c
      read (in_unit,*) plot_step
      read (in_unit,10) stp_ttl
c
      if (hybrid) then
         write (out_unit,130) plot_grid,cont_top,plot_step
        else
         write (out_unit,140) plot_grid,cont_top,plot_step
      end if
c
c-----------------------------------------------------------------------
c  Either generate new netCDF file or check input cards against old.
c-----------------------------------------------------------------------
c
      if (new_topo) then
         call chk_tfile (gridfile)
         call defcdf (gridfile,coord,no_x,no_y,dx,dy,lonc,latc,delx,
     &                dely,rotang,no_lvl,hybrid)
        else
         call chk_cdf (gridfile,coord,no_x,no_y,dx,dy,lonc,latc,delx,
     &                 dely,rotang,no_lvl,hybrid)
      end if
#ifdef sunflush
      call flush (out_unit)
#endif
c
      return
c
 10   format (a)
 20   format (/'Grid data file will be:  ',a)
 30   format (/'coord = ',i10/'Generating cartesian grid.'/
     &        /'cartesian parameters:'/3x,'dx = ',1pg23.16,' (m)'/3x,
     &       'dy = ',1pg23.16,' (m)'/3x,'lonc = ',1pg23.16,' (degrees)'/
     &       3x,'latc = ',1pg23.16,' (degrees)'/3x,'delx = ',1pg23.16,
     &    ' (m)'/3x,'dely = ',1pg23.16,' (m)'/3x,'rotang',
     &      ' = ',1pg23.16,' (degrees)'/3x,'no_x = ',i4/3x,'no_y = ',i4)
 35   format (/'coord = ',i10/'Generating geographic spherical grid.'/
     &        /'spherical rotation parameters:'/3x,'dx = ',1pg23.16,
     &        ' (degrees)'/3x,'dy = ',1pg23.16,' (degrees)'
     &        /3x,'lonc = ',1pg23.16,' (degrees)'/
     &       3x,'latc = ',1pg23.16,' (degrees)'/3x,'delx = ',1pg23.16,
     &    ' (degrees)'/3x,'dely = ',1pg23.16,' (degrees)'/3x,'rotang',
     &      ' = ',1pg23.16,' (degrees)'/3x,'no_x = ',i4/3x,'no_y = ',i4)
 40   format (/'coord = ',i10/'Generating rotated spherical grid.'/
     &        /'spherical rotation parameters:'/3x,'dx = ',1pg23.16,
     &        ' (degrees)'/3x,'dy = ',1pg23.16,' (degrees)'
     &        /3x,'lonc = ',1pg23.16,' (degrees)'/
     &       3x,'latc = ',1pg23.16,' (degrees)'/3x,'delx = ',1pg23.16,
     &    ' (degrees)'/3x,'dely = ',1pg23.16,' (degrees)'/3x,'rotang',
     &      ' = ',1pg23.16,' (degrees)'/3x,'no_x = ',i4/3x,'no_y = ',i4)
 50   format (/'hybrid flag = ',l1)
 60   format ('kc = ',i20/'optimize sigma system?  ',l1)
 70   format (/'Plot Window:'/26x,'Longitude',5x,'Latitude'/
     &        26x,'(degrees)',5x,'(degrees)'/
     &        '   Upper left corner: ',2(2x,1pg12.5)/'   Lower right',
     &       ' corner:',2(2x,1pg12.5))
 80   format (/'coastline data file:  ',a)
 90   format (/'new_topo = ',l1/'Generating topography at T grid points'
     &        ,' from archived data.'/3x,'topography input file:  ',a)
100   format (/'new_topo = ',l1/'Generating topography at T grid points'
     &        ,' from old file.'/3x,'topography input file:  ',a)
110   format (/'Topography contouring:')
120   format (/'Hydrostatic consistency factor contouring:')
130   format (/'Plot grid?  ',l1/'Contour topography?  ',l1/
     &                   'Contour hydrostatic consistency factor?  ',l1)
140   format (/'Plot grid?  ',l1/'Contour topography?  ',l1/
     &                                     'Plot step topography?  ',l1)
c
      end
