      subroutine get_dom (ncid,crd,nx,ny,lon,lat,delx,dely,dx,dy,theta,
     &                    err)
c
c=======================================================================
c                                                                    ===
c  This routine reads in the coordinate specification parameters     ===
c  from the given netCDF file.                                       ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     NCID....The identifier for the netCDF file.    (integer)       ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     CRD.....Coordinate system type flag.           (integer)       ===
c     NX......Number of grid points in x-direction.  (integer)       ===
c     NY......Number of grid points in y-direction.  (integer)       ===
c     LON.....Transformation centroid longitude.     (real; degrees) ===
c     LAT.....Transformation centroid latitude.      (real; degrees) ===
c     DELX....X-offset domain-trans. centers.        (real; m or deg)===
c     DELY....Y-offset domain-trans. centers.        (real; m or deg)===
c     DX......The grid spacing in x-direction.       (real; m or deg)===
c     DY......The grid spacing in y-direction.       (real; m or deg)===
c     THETA...Domain rotation angle.                 (real; deg)     ===
c     ERR.....Error flag.                            (logical)       ===
c                [.true. ]  an error occurred.                       ===
c                [.false.]  no error occurred.                       ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     netCDF:  NF_GET_VAR_INT,  NF_GET_VAR_REAL,  NF_INQ_VARID       ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <gpconst.h>
#include <netcdf.inc>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer crd,ncid,nx,ny,varid
      real    delx,dely,dx,dy,lat,lon,theta
      logical err,old
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Initialize error flag.
c-----------------------------------------------------------------------
c
      err = .false.
c
c-----------------------------------------------------------------------
c  Read coordinate indicator.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncid,'coord',varid) .eq. NF_NOERR) then
         if (nf_get_var_int (ncid,varid,crd) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'coord'
         end if
        else
         err = .true.
         write (6,20) 'coord'
      end if
c
c-----------------------------------------------------------------------
c  Read number of grid points.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncid,'imt',varid) .eq. NF_NOERR) then
         if (nf_get_var_int (ncid,varid,nx) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'imt'
         end if
        else
         err = .true.
         write (6,20) 'imt'
      end if
c
      if (nf_inq_varid (ncid,'jmt',varid) .eq. NF_NOERR) then
         if (nf_get_var_int (ncid,varid,ny) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'jmt'
         end if
        else
         err = .true.
         write (6,20) 'jmt'
      end if
c
c-----------------------------------------------------------------------
c  Read grid spacings.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncid,'meandx',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,dx) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'meandx'
         end if
        else
         err = .true.
         write (6,20) 'meandx'
      end if
c
      if (nf_inq_varid (ncid,'meandy',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,dy) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'meandy'
         end if
        else
         err = .true.
         write (6,20) 'meandy'
      end if
c
c-----------------------------------------------------------------------
c  Read domain centroid.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncid,'rlngd',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,lon) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'rlngd'
         end if
        else
         err = .true.
         write (6,20) 'rlngd'
      end if
c
      if (nf_inq_varid (ncid,'rlatd',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,lat) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'rlatd'
         end if
        else
         err = .true.
         write (6,20) 'rlatd'
      end if
c
c-----------------------------------------------------------------------
c  Read domain offset.
c-----------------------------------------------------------------------
c
      old = .false.
c
      if (nf_inq_varid (ncid,'delx',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,delx) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'delx'
         end if
        else
         old = .true.
      end if
c
      if (nf_inq_varid (ncid,'dely',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,dely) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'dely'
         end if
        else
         old = .true.
      end if
c
      if (old) then
         if (crd.ne.1) then
            delx = c0
            dely = c0
           else
            delx = lon
            dely = lat
            lon  = c0
            lat  = c0
         end if
      end if
c
c-----------------------------------------------------------------------
c  Read central rotation angle.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncid,'thetad',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncid,varid,theta) .ne. NF_NOERR) then
            err = .true.
            write (6,10) 'thetad'
         end if
        else
         err = .true.
         write (6,20) 'thetad'
      end if
c
      return
c
 10   format (/'***Error:  GET_DOM -- unable to read variable ',a)
 20   format (/'***Error:  GET_DOM -- unable to find variable ',a)
c
      end
