      subroutine hyb_sys (h,no_x,no_y,kc,no_lvl,dzt,top_err,hcf,hcf_max,
     &                               hcf_min,optimum,vrt_err,kmt,pltbad)
c
c=======================================================================
c                                                                    ===
c  This routine computes the maximal hydrostatic consistency factor. ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     DZT       Vertical level thicknesses.          (real array; m) ===
c     H         Topography at T-grid points.         (real array; cm)===
c     KC        Level at which to interface upper                    ===
c               & lower systems.                     (integer)       ===
c     NO_LVL    Total number of vertical levels.     (integer)       ===
c     NO_X      Number of grid points in x dir.      (integer)       ===
c     NO_Y      Number of grid points in y dir.      (integer)       ===
c     OPTIMUM   Indicates whether sigma levels                       ===
c               should be "optimized"                (logical)       ===
c                [.true. ]  "optimize" sigma levels.                 ===
c                [.false.]  do NOT "optimize".                       ===
c     TOP_ERR   Indicates whether or not an error                    ===
c               occurred in computing the topography                 ===
c               and therefore whether or not this                    ===
c               calculation can be done.             (logical)       ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     NCOUTID   Identifier number for netCDF output file.  (integer) ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     HCF       Hydrostatic consistency factor.      (real array)    ===
c     HCF_MAX   Maximum value for hcf.               (real)          ===
c     HCF_MIN   Minimum value for hcf.               (real)          ===
c     KMT       Locations of bad topography points.  (integer array) ===
c     PLTBAD    Indicates whether bad points exist                   ===
c               to be plotted.                       (logical)       ===
c     VRT_ERR   Flag to indicate if an error occurred                ===
c               in computing the model levels.       (logical)       ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     NCSTAT    Status flag for netCDF output file.        (integer) ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    AUTO_IFACE,        GET_SIGMA,        GR_EXIT,         ===
c              HCF_STAT,          MAX_SIGMA,        MAX_SLOPE,       ===
c              RESCALE_THICK,     SET_VGRID,        UV_DEPTH,        ===
c              VAR_SIGMA,         VERT_SIGMA                         ===
c     netCDF:  NF_ENDDEF,         NF_INQ_VARID,     NF_PUT_ATT_REAL, ===
c              NF_PUT_VAR_INT,    NF_PUT_VAR_REAL,  NF_PUT_VAR_TEXT, ===
c              NF_PUT_VARA_REAL,  NF_REDEF                           ===
c                                                                    ===
c                                                                    ===
c  The maximal hydrostatic consistency factor is found from:         ===
c                                                                    ===
c                           __       __                              ===
c                           |  sigma  |      L                       ===
c                           |       k |       ij                     ===
c              hcf   =  max | ------- | ------------                 ===
c                 ij     k  | dsigma  | ( h   - hc )                 ===
c                           |       k |    ij                        ===
c                           --       --                              ===
c                                                                    ===
c     where                                                          ===
c                                                                    ===
c                        __                         __               ===
c                        | | dh   |      | dh   |    |               ===
c                        | |   ij |      |   ij |    |               ===
c              L   = max | | ---- | dx , | ---- | dy |               ===
c               ij       | |  dx  |      |  dy  |    |               ===
c                        --                         --               ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <gridsparm.h>
#include <gridsio.h>
#include <gpconst.h>
#include <netcdf.inc>
#include <optzc1.h>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      integer i,itmax,j,kc,n,nobadf,nobadh,no_lvl,no_x,no_y,
     &        opt_iface,varid
      integer count(3),kmt(imt,jmt),start(3)
      real
     &      c,cm,hij,hmf,hx,hy,fac,favg,fdif,fij,fx,fy,rk,sig_fac,sigij,
     &      spval,tol,ub,zc1
      real
     &        dzt(mx_lvl),f(imt,jmt),h(imt,jmt),hcf(imt,jmt,2),
     &        hcf_max(2),hcf_min(2),sigma(mx_lvl)
      logical     flat,optimum,pltbad,top_err,vrt_err
      character*1 dum
c
      parameter (spval = cm1)
      data nobadf,nobadh /0,0/
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Determine whether or not this calculation can be done.
c-----------------------------------------------------------------------
c
      if (.not. top_err) then
c
c-----------------------------------------------------------------------
c  Compute coordinate interface parameters.
c-----------------------------------------------------------------------
c
         rk = c1 / float(no_lvl-kc)
         call interface (h,imt,no_x,no_y,c,zc1,zc2,alpha,f,rk,flat,cm,
     &                           fac,favg,fdif,href,tol,itmax,opt_iface)
c
c-----------------------------------------------------------------------
c  Compute sigma thicknesses and levels.
c-----------------------------------------------------------------------
c
         call get_sigma (no_lvl,kc,optimum,dzt,c,alpha,flat,sigma,
     &                                                          vrt_err)
c
c-----------------------------------------------------------------------
c  Reset coordinate interface as needed.
c-----------------------------------------------------------------------
c
         call auto_iface (flat,no_lvl,dzt,kc,zc1,favg,fdif,fac,imt,no_x,
     &                                     no_y,f,h,tol,itmax,opt_iface)
c
c-----------------------------------------------------------------------
c  Write interface depths to netCDF file.
c-----------------------------------------------------------------------
c
         if (nf_inq_varid (ncoutid,'interface',varid) .ne.NF_NOERR) then
            write (out_unit,90) 'interface'
            call gr_exit ('HYB_SYS')
         end if
         start(1) = 1
         count(1) = no_x
         count(2) = 1
         do 10 j = 1, no_y
            start(2) = j
            if (nf_put_vara_real (ncoutid,varid,start,count,f(1,j))
     &                                               .ne. NF_NOERR) then
               write (out_unit,100) 'interface',j
               call gr_exit ('HYB_SYS')
            end if
 10      continue
c
c-----------------------------------------------------------------------
c  Write interface at the velocity points.
c-----------------------------------------------------------------------
c
         call set_vgrid (no_x,no_y,f,ncoutid,'viface',out_unit)
c
c-----------------------------------------------------------------------
c  Find the maximum of sigma/dsigma.
c-----------------------------------------------------------------------
c
         call wrt_lvl (optimum,no_lvl,kc,sigma,fac,favg,fdif,href)
c
         call max_sigma (kc,no_lvl,sigma,sig_fac)
c
c-----------------------------------------------------------------------
c  Write vertical level parameters.
c-----------------------------------------------------------------------
c
         if (nf_inq_varid (ncoutid,'kc',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'kc'
            call gr_exit ('HYB_SYS')
         end if
         if (nf_put_var_int (ncoutid,varid,kc) .ne. NF_NOERR) then
            write (out_unit,110) 'kc'
            call gr_exit ('HYB_SYS')
         end if
c
         if (nf_inq_varid (ncoutid,'opt_lvl',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'opt_lvl'
            call gr_exit ('HYB_SYS')
         end if
         write (dum,130) optimum
         if (nf_put_var_text (ncoutid,varid,dum) .ne. NF_NOERR) then
            write (out_unit,110) 'opt_lvl'
            call gr_exit ('HYB_SYS')
         end if
c
c-----------------------------------------------------------------------
c  Write coordinate interface parameters.
c-----------------------------------------------------------------------
c
         if (nf_inq_varid (ncoutid,'zc1',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'zc1'
            call gr_exit ('HYB_SYS')
         end if
         if (nf_put_var_real (ncoutid,varid,zc1). ne. NF_NOERR) then
            write (out_unit,110) 'zc1'
            call gr_exit ('HYB_SYS')
         end if
c
         if (nf_inq_varid (ncoutid,'zc2',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'zc2'
            call gr_exit ('HYB_SYS')
         end if
         if (nf_put_var_real (ncoutid,varid,zc2) .ne. NF_NOERR) then
            write (out_unit,110) 'zc2'
            call gr_exit ('HYB_SYS')
         end if
c
         if (nf_inq_varid (ncoutid,'zslope',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'zslope'
            call gr_exit ('HYB_SYS')
         end if
         if (nf_put_var_real (ncoutid,varid,alpha) .ne. NF_NOERR) then
            write (out_unit,110) 'zslope'
            call gr_exit ('HYB_SYS')
         end if
c
c-----------------------------------------------------------------------
c  At each uv-grid point, compute the maximal hydrostatic consistency
c  factor.  Also compute the upper bound for theory.
c-----------------------------------------------------------------------
c
         ub = c0
c
         do 20 j = 1, no_y
            do 20 i = 1, no_x
c
c-----------------------------------------------------------------------
c  At each grid point, compute maximum slope.  Then compute the depth at
c  the uv-grid point.  Combine everything to find the hydrostatic
c  consistency factor.
c-----------------------------------------------------------------------
c
               call get_slope (i,j,imt,no_x,no_y,h,hx,hy)
               call uv_depth (i,j,imt,no_x,no_y,h,hij)
               call get_slope (i,j,imt,no_x,no_y,f,fx,fy)
               call uv_depth (i,j,imt,no_x,no_y,f,fij)
c
               call var_sigma (kc,no_lvl,dzt,sigma,fx,fy,hx,hy,sigij)
c
               hmf = hij - fij
c
               if (fij .lt. c0) then
                  hcf(i,j,1) = sig_fac * max(fx,fy) / abs(fij)
                 else
                  hcf(i,j,1) = spval
                  if ((fij.gt.c0) .or. ((fij.eq.c0).and.(kc.gt.0))) then
                     nobadf = nobadf + 1
                     kmt(i,j) = 1
                  end if
               end if
c
               if (hmf .lt. c0) then
                  hcf(i,j,2) = sigij / abs(hmf)
                  ub = max ( ub, max(hx,hy)/abs(hmf) )
                 else
                  hcf(i,j,2) = spval
                  nobadh = nobadh + 1
                  kmt(i,j) = 1
               end if
c
 20      continue
c
c-----------------------------------------------------------------------
c  Find maximum, minimum and average values for the hydrostatic
c  consistency factor and report them.  Report theoretical upper bound.
c-----------------------------------------------------------------------
c
         do 30 i = 1,2
            call hcf_stat (hcf(1,1,i),imt,no_x,no_y,out_unit,i,
     &                                            hcf_max(i),hcf_min(i))
 30      continue
c
         write (out_unit,50) cm*ub
c
c-----------------------------------------------------------------------
c  Write hydrostatic consistency factor in data file for exterior
c  handling.  (ie. 3D plots)
c-----------------------------------------------------------------------
c
         if (nf_redef (ncoutid) .eq. NF_NOERR) then
            ncstat = 1
           else
            write (out_unit,80)
            call gr_exit ('HYB_SYS')
         end if
c
         if (nf_inq_varid (ncoutid,'hcf',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'hcf'
            call gr_exit ('HYB_SYS')
         end if
         if (nf_put_att_real (ncoutid,varid,'_FillValue',NF_FLOAT,
     &                                      1,spval) .ne. NF_NOERR) then
            write (out_unit,110) 'hcf:_FillValue'
            call gr_exit ('HYB_SYS')
         end if
c
         if (nf_enddef (ncoutid) .eq. NF_NOERR) then
            ncstat = 2
           else
            write (out_unit,120)
            call gr_exit ('HYB_SYS')
         endif
c
         count(1) = 1
         start(2) = 1
         count(2) = no_x
         count(3) = 1
         do 40 n = 1, 2
            start(1) = n
            do 40 j = 1, no_y
               start(3) = j
               if (nf_put_vara_real (ncoutid,varid,start,count,
     &                                   hcf(1,j,n)) .ne. NF_NOERR) then
                  write (out_unit,100) 'hcf',j
                  call gr_exit ('HYB_SYS')
               end if
  40     continue
c
        else
c
c-----------------------------------------------------------------------
c  Cannot compute hydrostatic consistency factor, report this.
c-----------------------------------------------------------------------
c
         write (out_unit,60)
c
      end if
c
c-----------------------------------------------------------------------
c  Write out 3D thicknesses, if valid.
c-----------------------------------------------------------------------
c
      if (.not. vrt_err) call vert_sigma (no_x,no_y,kc,no_lvl,f,h,dzt)
c
c-----------------------------------------------------------------------
c  Write out level thicknesses, if valid.
c-----------------------------------------------------------------------
c
      if (.not. vrt_err) then
         call rescale_thick (no_lvl,flat,dzt,kc,fac,favg,fdif,href)
         if (nf_inq_varid (ncoutid,'hz',varid) .ne. NF_NOERR) then
            write (out_unit,90) 'hz'
            call gr_exit ('HYB_SYS')
         end if
         if (nf_put_var_real (ncoutid,varid,dzt) .ne. NF_NOERR) then
            write (out_unit,110) 'hz'
            call gr_exit ('HYB_SYS')
         end if
      end if
c
c-----------------------------------------------------------------------
c  Report on any bad points, set plotting flag for such points.
c-----------------------------------------------------------------------
c
      if (nobadf.gt.0) write (out_unit,70) 'interface','surface',nobadf
      if (nobadh.gt.0)
     &   write (out_unit,70) 'topography','interface',nobadh
c
      pltbad = (nobadf.gt.0) .or. (nobadh.gt.0)
#ifdef sunflush
      call flush (out_unit)
#endif
      return
c
 50   format (/'Theoretical upper bound for hydrostatic consistency ',
     &        'factor = ',1pg15.8)
 60   format(/'+++ Warning:  Hydrostatic consistency factor will not be'
     &      ,' computed'/
     &               14x,'due to an error in computing the topography.')
 70   format (/'+++ Warning:  ',a,' crosses ',a,1x,i10,' times.')
 80   format (/'*** Error:  HYB_SYS unable to put netCDF output file ',
     &                                              'into define mode.')
 90   format (/'*** Error:  HYB_SYS unable to find variable ',1h',a,1h')
100   format(/'*** Error:  HYB_SYS unable to write variable ',1h',a,1h',
     &                                                 ' at row j=',i10)
110   format(/'*** Error:  HYB_SYS unable to write variable ',1h',a,1h')
120   format (/'*** Error:  HYB_SYS unable to take netCDF output file ',
     &                                            'out of define mode.')
130   format (l1)
c
      end
