      subroutine interface (h,lda,no_x,no_y,c,zc1,zc2,alpha,f,rk,flat,
     &                      cmax,fac,favg,fdif,href,tol,itmax,opt_iface)
c
c=======================================================================
c                                                                    ===
c  This routine computes the variable coordinate interface depth.    ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     H      The topographic depth.                   (real array; m)===
c     LDA    The leading dimension of the arrays.     (integer)      ===
c     NO_X   Number of gridpoints in the x direction. (integer)      ===
c     NO_Y   Number of gridpoints in the y direction. (integer)      ===
c     RK     Reciprocal of number of lower levels.    (real)         ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     NCOUTID   Identifier number for netCDF output file.  (integer) ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     CMAX        Upper bound for sigma/dsigma.       (real)         ===
c     F           The coordinate interface depth.     (real array; m)===
c     FAC         Scale factor for tanh interface.    (real; 1/m)    ===
c     FAVG        Average of interface bounds.        (real; m)      ===
c     FDIF        Difference of interface bounds.     (real; m)      ===
c     FLAT        Indicates whetherinterface is flat. (logical)      ===
c                    [.true.]   defined flat.                        ===
c                    [.false.]  NOT defined flat.                    ===
c     HREF        Reference depth for tanh interface. (real; m)      ===
c     ITMAX       Max. iterations for zc1.            (integer)      ===
c     OPT_IFACE   Flag for optimizing zc1.            (integer)      ===
c                    [0] leave zc1 as is.                            ===
c                    [1] set zc1 so that                             ===
c                        min. interface depth = zc1                  ===
c                    [2] optimize CFL over min.                      ===
c                        topographic depth.                          ===
c     TOL         Tolerance for correctiong zc1.      (real)         ===
c     ZC1         Shallowest interface depth.         (real; m)      ===
c     ZC2         Deepest interface depth.            (real; m)      ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    GR_EXIT                                               ===
c     netCDF:  NF_INQ_VARID,    NF_PUT_VAR_INT,  NF_PUT_VAR_REAL,    ===
c              NF_PUT_VAR_TEXT                                       ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <gridsio.h>
#include <gpconst.h>
#include <netcdf.inc>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      integer i,itmax,j,lda,no_x,no_y,opt_iface,varid
      real    h(lda,no_y),f(lda,no_y),zc1,zc2,alpha,href,favg,fdif,fac,
     &        c,rk,abs_al,clow,cmax,tol
      logical     flat
      character*1 dum
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Read in additional parameters.  Compute groupings.
c-----------------------------------------------------------------------
c
      read (in_unit,*) zc1
      read (in_unit,*) zc2
      read (in_unit,*) alpha
      read (in_unit,*) href
      read (in_unit,*) c
      read (in_unit,*) tol
      read (in_unit,*) itmax
      read (in_unit,*) opt_iface
c
      abs_al = abs(alpha)
c
      if (abs_al .lt. c1) then
         clow = (c1-abs_al) / (c1-abs_al**rk)
        else if (abs_al .gt. c1) then
         clow = abs_al**rk / (abs_al**rk - c1) * (abs_al - c1)
        else
         clow = c1 / rk
      end if
      flat = (zc1 .eq. zc2) .or. (alpha .eq. c0)
      if (flat) clow = c
      cmax = max (c,clow)
c
      write (out_unit,20) zc1,zc2,alpha,href,c,clow,tol,itmax,opt_iface
c
      favg = p5 * (zc1 + zc2)
      fdif = p5 * (zc1 - zc2)
c
      if (fdif .ne. c0) then
         fac = alpha/fdif
        else
         fac = c0
      end if
c
c-----------------------------------------------------------------------
c  Compute coordinate interface level.
c-----------------------------------------------------------------------
c
      do 10 j = 1, no_y
         do 10 i = 1, no_x
c
            f(i,j) = favg + fdif * tanh(fac*(h(i,j)-href))
c
 10   continue
c
c-----------------------------------------------------------------------
c  Write coordinate interface parameters.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncoutid,'zref',varid) .ne. NF_NOERR) then
         write (out_unit,30) 'zref'
         call gr_exit ('INTERFACE')
      end if
      if (nf_put_var_real (ncoutid,varid,href) .ne. NF_NOERR) then
         write (out_unit,40) 'zref'
         call gr_exit ('INTERFACE')
      end if
c
      if (nf_inq_varid (ncoutid,'sig_bnd',varid) .ne. NF_NOERR) then
         write (out_unit,30) 'sig_bnd'
         call gr_exit ('INTERFACE')
      end if
      if (nf_put_var_real (ncoutid,varid,c) .ne. NF_NOERR) then
         write (out_unit,40) 'sig_bnd'
         call gr_exit ('INTERFACE')
      end if
c
      if (nf_inq_varid (ncoutid,'flat',varid) .ne. NF_NOERR) then
         write (out_unit,30) 'flat'
         call gr_exit ('INTERFACE')
      end if
      write (dum,50) flat
      if (nf_put_var_text (ncoutid,varid,dum) .ne. NF_NOERR) then
         write (out_unit,40) 'flat'
         call gr_exit ('INTERFACE')
      end if
c
      if (nf_inq_varid (ncoutid,'zc1_tol',varid) .ne. NF_NOERR) then
         write (out_unit,30) 'zc1_tol'
         call gr_exit ('INTERFACE')
      end if
      if (nf_put_var_real (ncoutid,varid,tol) .ne. NF_NOERR) then
         write (out_unit,40) 'zc1_tol'
         call gr_exit ('INTERFACE')
      end if
c
      if (nf_inq_varid (ncoutid,'zc1_itmax',varid) .ne. NF_NOERR) then
         write (out_unit,30) 'zc1_itmax'
         call gr_exit ('INTERFACE')
      end if
      if (nf_put_var_int (ncoutid,varid,itmax) .ne. NF_NOERR) then
         write (out_unit,40) 'zc1_itmax'
         call gr_exit ('INTERFACE')
      end if
c
      if (nf_inq_varid (ncoutid,'opt_iface',varid) .ne. NF_NOERR) then
         write (out_unit,30) 'opt_iface'
         call gr_exit ('INTERFACE')
      end if
      if (nf_put_var_int (ncoutid,varid,opt_iface) .ne. NF_NOERR) then
         write (out_unit,40) 'opt_iface'
         call gr_exit ('INTERFACE')
      end if
c
#ifdef sunflush
      call flush (out_unit)
c
#endif
      return
c
 20   format (/'Coordinate interface parameters:'/3x,'zc1 = ',1pg15.8,2x
     &       ,'(m)'/3x,'zc2 = ',1pg15.8,'  (m)'/3x,'alpha = ',1pg15.8/3x
     &       ,'href = ',1pg15.8,'  (m)'/3x,'c (upper) = ',1pg15.8/3x,
     &       'c (lower) = ',1pg15.8/3x,'zc1_tol = ',1pg15.8/3x,
     &       'zc1_itmax = ',i10/3x,'opt_iface = ',i10)
 30   format (/'*** Error:  INTERFACE unable to find variable ',
     &        1h',a,1h')
 40   format (/'*** Error:  INTERFACE unable to write variable ',
     &        1h',a,1h')
 50   format (l1)
c
      end
