      subroutine new_depth (geo_lon,geo_lat,hybrid,topog,top_err)
c
c=======================================================================
c                                                                    ===
c  This routine gets the topography at the provided grid points and  ===
c  writes this information to a file,                                ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     GEO_LAT   Latitudes of T-grid points.      (real array; deg)   ===
c     GEO_LON   Longitudes of T-grid points.     (real array; deg)   ===
c     HYBRID    Flag for step or sigma system.   (logical)           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /DOM/                                                             ===
c                                                                    ===
c     NO_X   Number of elements in x direction.   (integer)          ===
c     NO_Y   Number of elements in y direction.   (integer)          ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     NCOUTID   Identifier number for netCDF output file.  (integer) ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     TOP_ERR   Error flag.                      (logical)           ===
c     TOPOG     Topography at T-grid.            (real array; m)     ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /TPMED/                                                           ===
c                                                                    ===
c     HMED   The median topographic depth.  (real; m)                ===
c                                                                    ===
c  /TPSTT/                                                           ===
c                                                                    ===
c     HMAX   Maximum depth.  (real; m)                               ===
c     HMIN   Minimum depth.  (real; m)                               ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    DEPTH,         GR_EXIT,          SET_VGRID,           ===
c              TOPO_STAT                                             ===
c     netCDF:  NF_INQ_VARID,  NF_PUT_VARA_REAL                       ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <gridsparm.h>
#include <gridsio.h>
#include <gpconst.h>
#include <netcdf.inc>
#include <dom.h>
#include <tpmed.h>
#include <tpstt.h>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      integer i,ier,j,varid,varidr,varidt
      integer count(2),start(2)
      logical hybrid,top_err
      real    depth,hmen,hrms
      real geo_lat(imt,jmt),geo_lon(imt,jmt),topog(imt,jmt)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Compute topography at grid points.
c-----------------------------------------------------------------------
c
      j = 0
      ier = 0
      do 10 while ((j .lt. no_y) .and. (ier .eq. 0))
         j = j + 1
         i = 0
         do 10 while ((i .lt. no_x) .and. (ier .eq. 0))
            i = i + 1
            topog(i,j) = depth(geo_lon(i,j),geo_lat(i,j),ier)
            if (ier .ne. 0) write (out_unit,40) i,j,ier
  10  continue
c
c-----------------------------------------------------------------------
c  Write the topography in the given file (if successful).
c-----------------------------------------------------------------------
c
      top_err = ier .ne. 0
c
      if (.not. top_err) then
c
         call topo_stat (topog,hmen,hmed,hrms)
c
         if (nf_inq_varid (ncoutid,'tbath',varid) .ne. NF_NOERR) then
            write (out_unit,50) 'tbath'
            call gr_exit ('NEW_DEPTH')
         end if
c
         if (hybrid) then
c
            if (nf_inq_varid (ncoutid,'raw_bath',varidr) .ne. NF_NOERR)
     &                                                              then
               write (out_unit,50) 'raw_bath'
               call gr_exit ('NEW_DEPTH')
            end if
c
            if (nf_inq_varid (ncoutid,'trg_bath',varidt) .ne. NF_NOERR)
     &                                                              then
               write (out_unit,50) 'trg_bath'
               call gr_exit ('NEW_DEPTH')
            end if
c
         end if
c
         start(1) = 1
         count(1) = no_x
         count(2) = 1
c
         do 30 j = 1, no_y
c
            start(2) = j
            if (nf_put_vara_real (ncoutid,varid,start,count,topog(1,j))
     &                                               .ne. NF_NOERR) then
               write (out_unit,60) 'tbath',j
               call gr_exit ('NEW_DEPTH')
            end if
c
            if (hybrid) then
c
               if (nf_put_vara_real (ncoutid,varidr,start,count,
     &                                   topog(1,j)) .ne. NF_NOERR) then
                  write (out_unit,60) 'raw_bath',j
                  call gr_exit ('NEW_DEPTH')
               end if
c
               if (nf_put_vara_real (ncoutid,varidt,start,count,
     &                                   topog(1,j)) .ne. NF_NOERR) then
                  write (out_unit,60) 'trg_bath',j
                  call gr_exit ('NEW_DEPTH')
               end if
c
            end if
c
  30     continue
c
      end if
c
c-----------------------------------------------------------------------
c  Write bathymetry at the velocity points.
c-----------------------------------------------------------------------
c
      call set_vgrid (no_x,no_y,topog,ncoutid,'vbath',out_unit)
c
#ifdef sunflush
      call flush (out_unit)
c
#endif
      return
c
  40  format (14x,'Unable to read data for grid point (',i10,',',i10,')'
     &                                           /14x,'error flag=',i10)
  50  format (/'*** Error:  NEW_DEPTH failed find variable ',a)
  60  format (/'*** Error:  NEW_DEPTH failed to write ',a,' at row j='
     &                                                             ,i10)
c
      end
