      subroutine pe_grid (geo_tlon,geo_tlat,geo_lon,geo_lat)
c
c=======================================================================
c                                                                    ===
c  This routine computes the PE grid and associated quantities.      ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  Common Blocks                                                     ===
c                                                                    ===
c  /DOM/                                                             ===
c                                                                    ===
c     COORD    Horizontal coordinate type flag.          (integer)   ===
c                   [0]  cartesian grid.                             ===
c                   [1]  geographic spherical grid.                  ===
c                   [2]  rotated spherical grid.                     ===
c     DELX     X-offset between grid & trans. centers.   (real)      ===
c     DX       Grid x spacing.                           (real)      ===
c     DELY     Y-offset between grid & trans. centers.   (real)      ===
c     DY       Grid y spacing.                           (real)      ===
c     LATC     Latitude of transformation center.        (real)      ===
c     LONC     Longitude of transformation center.       (real)      ===
c     NO_X     Number of elements in the x direction.    (integer)   ===
c     NO_Y     Number of elements in the y direction.    (integer)   ===
c     ROTANG   Rotation angle.                           (real)      ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     GEO_LAT    The geographic latitudes                            ===
c                of the UV grid points.      (real array; degrees)   ===
c     GEO_LON    The geographic longitudes                           ===
c                of the UV grid points.      (real array; degrees)   ===
c     GEO_TLAT   The geographic latitudes                            ===
c                of the T grid points.       (real array; degrees)   ===
c     GEO_TLON   The geographic longitudes                           ===
c                of the T grid points.       (real array; degrees)   ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    GR_EXIT,       ROTANGLE,         XY2LL                ===
c     netCDF:  NF_INQ_VARID,  NF_PUT_VAR_REAL,  NF_PUT_VARA_REAL     ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <gridsparm.h>
#include <gridsio.h>
#include <gpconst.h>
#include <dom.h>
#include <netcdf.inc>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      real
     &      deg2m
c
      parameter (deg2m=rearth*km2m*deg2rad)
c
      real
     &      fx,x,y,yc
      real
     &      cs(jmt),cst(jmt),geo_lat(imt,jmt),geo_lon(imt,jmt),
     &      geo_tlat(imt,jmt),geo_tlon(imt,jmt),tng(jmt),wk(imjm)
      real
     &        rotangle
      integer i,j,varid,varid2
      integer count(3),start(3)
c
      equivalence (cs(1),  wk(1))
      equivalence (cst(1), wk(jmt+1))
      equivalence (tng(1), wk(2*jmt+1))
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Get desired grid spacings on unit intervals, and convert to
c  rotated coordinates.
c-----------------------------------------------------------------------
c
      if (coord.eq.0) then
         fx = c1
       else
         fx = deg2m
      end if
c
c  Longitudinal spacing.
c
      do 10 i = 1, no_x
         wk(i) = dx*fx
  10  continue
c
      if (nf_inq_varid (ncoutid,'dxu',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'dxu'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,wk) .ne. NF_NOERR) then
         write (out_unit,910) 'dxu'
         call gr_exit ('PE_GRID')
      end if
c
      if (nf_inq_varid (ncoutid,'dxt',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'dxt'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,wk) .ne. NF_NOERR) then
         write (out_unit,910) 'dxt'
         call gr_exit ('PE_GRID')
      end if
c
c  Latitudinal spacing.
c
      do 20 i = 1, no_y
         wk(i) = dy*fx
  20  continue
c
      if (nf_inq_varid (ncoutid,'dyu',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'dyu'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,wk) .ne. NF_NOERR) then
         write (out_unit,910) 'dyu'
         call gr_exit ('PE_GRID')
      end if
c
      if (nf_inq_varid (ncoutid,'dyt',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'dyt'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,wk) .ne. NF_NOERR) then
         write (out_unit,910) 'dyt'
         call gr_exit ('PE_GRID')
      end if
c
c-----------------------------------------------------------------------
c  Compute & write the metric factors.
c-----------------------------------------------------------------------
c
      yc = p5*float(no_y+1)
      fx = dy*deg2rad
      do 30 j=1, no_y
         if (coord.eq.0) then
            cst(j) = c1
            cs(j)  = c1
            tng(j) = c0
          elseif ((coord.eq.1).or.(coord.eq.2)) then
            y      = dely*deg2rad+(float(j)-yc)*fx
            cst(j) = cos(y)
            y      = dely*deg2rad+(float(j)-yc+p5)*fx
            cs(j)  = cos(y)
            tng(j) = tan(y)
         end if
  30  continue
c
c  Write cosine factors at velocity points.
c
      if (nf_inq_varid (ncoutid,'cs',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'cs'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,cs) .ne. NF_NOERR) then
         write (out_unit,910) 'cs'
         call gr_exit ('PE_GRID')
      end if
c
c  Write cosine factors at tracer points.
c
      if (nf_inq_varid (ncoutid,'cst',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'cst'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,cst) .ne. NF_NOERR) then
         write (out_unit,910) 'cst'
         call gr_exit ('PE_GRID')
      end if
c
c  Write tangent factors at velocity points.
c
      if (nf_inq_varid (ncoutid,'tng',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'tng'
         call gr_exit ('PE_GRID')
      end if
      if (nf_put_var_real (ncoutid,varid,tng) .ne. NF_NOERR) then
         write (out_unit,910) 'tng'
         call gr_exit ('PE_GRID')
      end if
c
c-----------------------------------------------------------------------
c  Compute the geographic longitudes & latitudes.
c-----------------------------------------------------------------------
c
      if (nf_inq_varid (ncoutid,'vgrid2',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'vgrid2'
         call gr_exit ('PE_GRID')
      end if
c
      if (nf_inq_varid (ncoutid,'tgrid2',varid2) .ne. NF_NOERR) then
         write (out_unit,900) 'tgrid2'
         call gr_exit ('PE_GRID')
      end if
c
      count(1) = 1
      start(2) = 1
      count(2) = no_x
      count(3) = 1
c
      do 50 j = 1, no_y
c
c  Compute positions.
c
         do 40 i = 1, no_x
c
            x = float(i)
            y = float(j)
            call xy2ll (x,y,coord,no_x,no_y,dx,dy,lonc,latc,delx,
     &                  dely,rotang,geo_tlon(i,j),geo_tlat(i,j))
c
            x = float(i)+p5
            y = float(j)+p5
            call xy2ll (x,y,coord,no_x,no_y,dx,dy,lonc,latc,delx,
     &                  dely,rotang,geo_lon(i,j),geo_lat(i,j))
  40     continue
c
c  Write longitudes.
c
         start(1) = 1
         start(3) = j
         if (nf_put_vara_real (ncoutid,varid,start,count,geo_lon(1,j))
     &                                               .ne. NF_NOERR) then
            write (out_unit,920) 'vgrid2 longitude',j
            call gr_exit ('PE_GRID')
         end if
         if (nf_put_vara_real (ncoutid,varid2,start,count,geo_tlon(1,j))
     &                                               .ne. NF_NOERR) then
            write (out_unit,920) 'tgrid2 longitude',j
            call gr_exit ('PE_GRID')
         end if
c
c  Write latitudes.
c
         start(1) = 2
         if (nf_put_vara_real (ncoutid,varid,start,count,geo_lat(1,j))
     &                                               .ne. NF_NOERR) then
            write (out_unit,920) 'vgrid2 latitude',j
            call gr_exit ('PE_GRID')
         end if
         if (nf_put_vara_real (ncoutid,varid2,start,count,geo_tlat(1,j))
     &                                               .ne. NF_NOERR) then
            write (out_unit,920) 'tgrid2 latitude',j
            call gr_exit ('PE_GRID')
         end if
  50  continue
c
c-----------------------------------------------------------------------
c  Compute & write rotation angle at UV points.
c-----------------------------------------------------------------------
c
      do 60 j = 1, no_y
      do 60 i = 1, no_x
         wk(i+(j-1)*no_x) = rotangle (geo_lon(i,j),geo_lat(i,j),
     &                                   coord,lonc,latc,rotang)*rad2deg
  60  continue
c
      if (nf_inq_varid (ncoutid,'lrotang',varid) .ne. NF_NOERR) then
         write (out_unit,900) 'lrotang'
         call gr_exit ('PE_GRID')
      end if
      start(1) = 1
      start(2) = 1
      count(1) = no_x
      count(2) = no_y
      if (nf_put_vara_real (ncoutid,varid,start,count,wk)
     &                                               .ne. NF_NOERR) then
         write (out_unit,910) 'lrotang'
         call gr_exit ('PE_GRID')
      end if
c
c-----------------------------------------------------------------------
c  Write the geographic coordinates of the four corners of the grid.
c-----------------------------------------------------------------------
c
      write (out_unit,70) geo_tlon(1,no_y),geo_tlat(1,no_y),
     &                     geo_tlon(no_x,no_y),geo_tlat(no_x,no_y),
     &     geo_tlon(1,1),geo_tlat(1,1),geo_tlon(no_x,1),geo_tlat(no_x,1)
 70   format (/'The geographic (longitude,latitude) coordinates of the',
     &        ' T-grid corners. (degrees)'//'(',1pg12.5,',',1pg12.5,')',
     &      26('-'),'(',1pg12.5,',',1pg12.5,')'/3(13x,'|',52x,'|'/),'(',
     &      1pg12.5,',',1pg12.5,')',26('-'),'(',1pg12.5,',',1pg12.5,')')
c
      write (out_unit,80) geo_lon(1,no_y),geo_lat(1,no_y),
     &                     geo_lon(no_x,no_y),geo_lat(no_x,no_y),
     &         geo_lon(1,1),geo_lat(1,1),geo_lon(no_x,1),geo_lat(no_x,1)
 80   format (/'The geographic (longitude,latitude) coordinates of the',
     &        ' UV-grid corners. (degrees)'//'(',1pg12.5,',',1pg12.5,')'
     &     ,26('-'),'(',1pg12.5,',',1pg12.5,')'/3(13x,'|',52x,'|'/),'(',
     &      1pg12.5,',',1pg12.5,')',26('-'),'(',1pg12.5,',',1pg12.5,')')
c
#ifdef sunflush
      call flush (out_unit)
c
#endif
      return
c
900   format (/'***Error:  PE_GRID - unable to find variable ',1h",a,1h"
     &        ,'in netCDF output file.')
910   format (/'***Error:  PE_GRID - error while writing variable ',1h",
     &        a,1h",'to netCDF output file.')
920   format (/'***Error:  PE_GRID - error while writing variable ',1h",
     &        a,1h"/10x,'(slab j=',i10,') to netCDF output file.')
c
      end
