      subroutine read_grids (gfile)
c
c=======================================================================
c                                                                    ===
c  This subroutine reads the coarse grid definition from its GRIDS   ===
c  file.                                                             ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  GFILE      Name of previously defined coarse GRIDS                ===
c             file.                                     (character)  ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /C2FIO/                                                           ===
c                                                                    ===
c     STDOUT  Unit number on which to write messages.   (integer)    ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /C2FIO/                                                           ===
c                                                                    ===
c     NCINID...Identifier number for netCDF input file.  (integer)   ===
c     NCINST...Status flag for netCDF input file.        (integer)   ===
c                                                                    ===
c  /CRS_DAT/                                                         ===
c                                                                    ===
c     COORD     Grid type.                            (integer)      ===
c                  [0] cartesian                                     ===
c                  [1] geographic spherical                          ===
c                  [2] rotated spherical                             ===
c     DELX_C    X-offset, grid-trans. centers.        (real; m | deg)===
c     DELY_C    Y-offset, grid-trans. centers.        (real; m | deg)===
c     GRIDX_C   Grid spacing in x-direction.          (real; m | deg)===
c     GRIDY_C   Grid spacing in y-direction.          (real; m | deg)===
c     IMT_C     Number of grid points in x direction. (integer)      ===
c     JMT_C     Number of grid points in y direction. (integer)      ===
c     LATC      Latitude of center of transformation. (real; deg)    ===
c     LONC      Longitude of center of transformation.(real; deg)    ===
c     THETA     Angle grid is rotated WRT East.       (real; deg)    ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    CF_EXIT,      LENGTH                                  ===
c     netCDF:  NF_OPEN,      NF_GET_VAR_INT,  NF_GET_VAR_REAL,       ===
c              NF_INQ_VARID                                          ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <gpconst.h>
#include <c2fio.h>
#include <crs_dat.h>
#include <netcdf.inc>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer sbgn,send,slen,varid
      logical       old
      character*(*) gfile
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Open input GRIDS file.
c-----------------------------------------------------------------------
c
c  Make sure supplied name is valid.
c
      call length (gfile,slen,sbgn,send)
c
      if (slen.lt.1) then
         write (stdout,10)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Open netCDF file.
c
      if (nf_open(gfile(sbgn:send),NF_NOWRITE,ncinid) .eq.NF_NOERR) then
         ncinst = 2
        else
         write (stdout,20) gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c-----------------------------------------------------------------------
c  Read parameters common to both grids.
c-----------------------------------------------------------------------
c
c  Grid type flag.
c
      if (nf_inq_varid (ncinid,'coord',varid) .ne. NF_NOERR) then
         write (stdout,30) 'coord',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_int (ncinid,varid,coord) .ne. NF_NOERR) then
         write (stdout,40) 'coord',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Latitude of center of transformation.
c
      if (nf_inq_varid (ncinid,'rlatd',varid) .ne. NF_NOERR) then
         write (stdout,30) 'rlatd',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_real (ncinid,varid,latc) .ne. NF_NOERR) then
         write (stdout,40) 'rlatd',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Longitude of center of transformation.
c
      if (nf_inq_varid (ncinid,'rlngd',varid) .ne. NF_NOERR) then
         write (stdout,30) 'rlngd',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_real (ncinid,varid,lonc) .ne. NF_NOERR) then
         write (stdout,40) 'rlngd',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Domain rotation angle.
c
      if (nf_inq_varid (ncinid,'thetad',varid) .ne. NF_NOERR) then
         write (stdout,30) 'thetad',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_real (ncinid,varid,theta) .ne. NF_NOERR) then
         write (stdout,40) 'thetad',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c-----------------------------------------------------------------------
c  Read coarse grid parameters.
c-----------------------------------------------------------------------
c
c  Number of grid points in x-direction.
c
      if (nf_inq_varid (ncinid,'imt',varid) .ne. NF_NOERR) then
         write (stdout,30) 'imt',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_int (ncinid,varid,imt_c) .ne. NF_NOERR) then
         write (stdout,40) 'imt',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Number of grid points in y-direction.
c
      if (nf_inq_varid (ncinid,'jmt',varid) .ne. NF_NOERR) then
         write (stdout,30) 'jmt',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_int (ncinid,varid,jmt_c) .ne. NF_NOERR) then
         write (stdout,40) 'jmt',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Grid x-spacing.
c
      if (nf_inq_varid (ncinid,'meandx',varid) .ne. NF_NOERR) then
         write (stdout,30) 'meandx',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_real (ncinid,varid,gridx_c) .ne. NF_NOERR) then
         write (stdout,40) 'meandx',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Grid y-spacing.
c
      if (nf_inq_varid (ncinid,'meandy',varid) .ne. NF_NOERR) then
         write (stdout,30) 'meandy',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
      if (nf_get_var_real (ncinid,varid,gridy_c) .ne. NF_NOERR) then
         write (stdout,40) 'meandy',gfile(sbgn:send)
         call cf_exit ('READ_GRIDS')
      end if
c
c  Prepare to read offsets.
c
      old = .false.
c
c  X-offset between grid & transformation centers.
c
      if (nf_inq_varid (ncinid,'delx',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncinid,varid,delx_c) .ne. NF_NOERR) then
            write (stdout,40) 'delx',gfile(sbgn:send)
            call cf_exit ('READ_GRIDS')
         end if
       else
         old = .true.
      end if
c
c  Y-offset between grid & transformation centers.
c
      if (nf_inq_varid (ncinid,'dely',varid) .eq. NF_NOERR) then
         if (nf_get_var_real (ncinid,varid,dely_c) .ne. NF_NOERR) then
            write (stdout,40) 'dely',gfile(sbgn:send)
            call cf_exit ('READ_GRIDS')
         end if
       else
         old = .true.
      end if
c
c  If dealing with an old GRIDS file (pre 6.1) reset variables to
c  current usage.
c
      if (old) then
         if (coord.ne.1) then
            delx_c = c0
            dely_c = c0
           else
            delx_c = lonc
            dely_c = latc
            lonc   = c0
            latc   = c0
         end if
      end if
c
      return
c
 10   format (/'*** Error:  READ_GRIDS - invalid (empty) file name')
 20   format (/'*** Error:  READ_GRIDS - unable to open file ',
     &        1h",a,1h")
 30   format (/'*** Error:  READ_GRIDS - unable to find variable ',
     &        1h",a,1h",' in file ',1h",a,1h")
 40   format (/'*** Error:  READ_GRIDS - unable to read variable ',
     &        1h",a,1h",' in file ',1h",a,1h")
c
      end
