      subroutine step_sys (no_lvl,dzt,topog,kmt,top_err,vrt_err)
c
c=======================================================================
c                                                                    ===
c  This subroutine computes the information needed for a cartesian   ===
c  coordinate system in the vertical.                                ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     DZT       Vertical level thicknesses.           (real array; m)===
c     NO_LVL    Total number of vertical levels.      (integer)      ===
c     TOP_ERR   Error status from topo computation.   (logical)      ===
c     TOPOG     Topography at T grid points.          (real array; m)===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /DOM/                                                             ===
c                                                                    ===
c     NO_X      Number of elements in x direction.    (integer)      ===
c     NO_Y      Number of elements in y direction.    (integer)      ===
c                                                                    ===
c  /GRIDSIO/                                                         ===
c                                                                    ===
c     NCOUTID   NetCDF identifier for output file.    (integer)      ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     KMT       Number of vertical levels at a                       ===
c               given horizontal grid point.          (integer array)===
c     VRT_ERR   Error status from  vertical                          ===
c               thicknesses computation.              (logical)      ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    CHK_LEV,       FILT_STEP,        GR_EXIT,             ===
c              VERT_FLAT                                             ===
c     netCDF:  NF_INQ_VARID,  NF_PUT_VAR_REAL,  NF_PUT_VARA_REAL     ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <gridsparm.h>
#include <gridsio.h>
#include <gpconst.h>
#include <netcdf.inc>
#include <dom.h>
c
c-----------------------------------------------------------------------
c  Define local data
c-----------------------------------------------------------------------
c
      integer i,j,k,no_lvl,varid
      integer count(2),kmt(imt,jmt),start(2)
      logical top_err,vrt_err
      real    dzt(mx_lvl),h(mx_lvl),topog(imt,jmt)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Check to see that model levels are valid.
c-----------------------------------------------------------------------
c
      call chk_lev (no_lvl,dzt,1,no_lvl,vrt_err)
c
      if (.not. vrt_err) then
c
c-----------------------------------------------------------------------
c  Compute level depths in centimeters.
c-----------------------------------------------------------------------
c
         h(1) = cm1*dzt(1)
         do 10 k = 2, no_lvl
            h(k) = h(k-1) - dzt(k)
 10      continue
c
         if (.not. top_err) then
c
c-----------------------------------------------------------------------
c  Compute the "step topography" such that the bottom vertical level is
c  the one closest to the actual topography.
c-----------------------------------------------------------------------
c
            do 30 j = 1, no_y
               do 30 i = 1, no_x
c
                  k = 1
c
                  do 20 while ( (h(k).gt.topog(i,j)).and.(k.lt.no_lvl) )
                     k = k + 1
 20               continue
c
                  if (k .gt. 1) then
c
                     if ((topog(i,j)-h(k)).lt.(h(k-1)-topog(i,j))) then
                        kmt(i,j) = k
                       else
                        kmt(i,j) = k - 1
                     end if
c
                    else
c
                     if ((topog(i,j)-h(1)).lt.(c0-topog(i,j))) then
                        kmt(i,j) = 1
                       else
                        kmt(i,j) = 0
                     end if
c
                  end if
 30         continue
c
c-----------------------------------------------------------------------
c  Filter the "step topography".
c-----------------------------------------------------------------------
c
            call filt_step (kmt)
c
c-----------------------------------------------------------------------
c  Write out "step topography" data.
c-----------------------------------------------------------------------
c
            if (nf_inq_varid (ncoutid,'hz',varid) .ne. NF_NOERR) then
               write (out_unit,50) 'hz'
               call gr_exit ('STEP_SYS')
            end if
            if (nf_put_var_real (ncoutid,varid,dzt) .ne. NF_NOERR) then
               write (out_unit,60) 'hz'
               call gr_exit ('STEP_SYS')
            end if
c
            if (nf_inq_varid (ncoutid,'kmt',varid) .ne. NF_NOERR) then
               write (out_unit,50) 'kmt'
               call gr_exit ('STEP_SYS')
            end if
            start(1) = 1
            count(1) = no_x
            count(2) = 1
            do 40 j = 1, no_y
               start(2) = j
               if (nf_put_vara_real (ncoutid,varid,start,count,kmt(1,j))
     &                                               .ne. NF_NOERR) then
                  write (out_unit,70) 'kmt',j
                  call gr_exit ('STEP_SYS')
               end if
 40         continue
c
         end if
      end if
c
c-----------------------------------------------------------------------
c  Write out 3D thicknesses.
c-----------------------------------------------------------------------
c
      call vert_flat (no_x,no_y,no_lvl,dzt)
c
      return
c
 50   format(/'*** Error:  STEP_SYS unable to find variable ',1h',a,1h')
 60   format (/'*** Error:  STEP_SYS unable to write variable ',
     &        1h',a,1h')
 70   format (/'*** Error:  STEP_SYS unable to write variable ',
     &        1h',a,1h',' at row j=',i10)
c
      end
