      subroutine sub_sam_mask (ldfb,f_big,ldfs,f_small)
c
c=======================================================================
c                                                                    ===
c  This routine sub-samples the big land mask onto the small land    ===
c  mask.                                                             ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     f_big.....Big 2D field from which to sample.   (integer array) ===
c     ldfb......Leading dimension of f_big.          (integer)       ===
c     ldfs......Leading dimension of f_small.        (integer)       ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /BGDOMDAT/                                                        ===
c                                                                    ===
c     BCRD     coordinate type                       (integer)       ===
c     BDLX     x-offset grid-trans. centers          (real; m | deg) ===
c     BDLY     y-offset grid-trans. centers          (real; m | deg) ===
c     BDX      grid spacing in the x-direction       (real; m | deg) ===
c     BDY      grid spacing in the y-direction       (real; m | deg) ===
c     BLAT     transformation centroid latitude      (real; degrees) ===
c     BLON     transformation centroid longitude     (real; degrees) ===
c     BNX      number of gridpoints in x-direction   (integer)       ===
c     BNY      number of gridpoints in y-direction   (integer)       ===
c     BTHETA   domain rotation angle                 (real; degrees) ===
c                                                                    ===
c  /SMDOMDAT/                                                        ===
c                                                                    ===
c     SCRD     coordinate type                       (integer)       ===
c     SDLX     x-offset grid-trans. centers          (real; m | deg) ===
c     SDLY     y-offset grid-trans. centers          (real; m | deg) ===
c     SDX      grid spacing in the x-direction       (real; m | deg) ===
c     SDY      grid spacing in the y-direction       (real; m | deg) ===
c     SLAT     transformation centroid latitude      (real; degrees) ===
c     SLON     transformation centroid longitude     (real; degrees) ===
c     SNX      number of gridpoints in x-direction   (integer)       ===
c     SNY      number of gridpoints in y-direction   (integer)       ===
c     STHETA   domain rotation angle                 (real; degrees) ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     f_small...Small 2D field to sample.            (integer array) ===
c                                                                    ===
c  Calls:  LL2XY,  XY2LL                                             ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <bgdomdat.h>
#include <smdomdat.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,ib,ix,j,jb,jy,ldfb,ldfs,nxbm1,nybm1,nysm1
      integer f_big(ldfb,bny),f_small(ldfs,sny)
      real    lat,lon,nbhdx,nbhdy,p5,x,xmib,y,ymjb
c
      parameter (p5=0.5)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Compute the penultimate grid points for the big grid (used to
c  prevent out of bounds errors).
c-----------------------------------------------------------------------
c
      nxbm1 = bnx - 1
      nybm1 = bny - 1
      nysm1 = sny - 1
      nbhdx = p5*sdx/bdx
      nbhdy = p5*sdy/bdy
c
c-----------------------------------------------------------------------
c  Loop for every point in the small array, f_small.
c-----------------------------------------------------------------------
c
      do 10 j = 1, nysm1
         do 10 i = 1, snx
c
c-----------------------------------------------------------------------
c        Convert each gridpoint in the small grid to an (x,y) in the 
c        big grid units.
c-----------------------------------------------------------------------
c
         call xy2ll (float(i),float(j),scrd,snx,sny,sdx,sdy,slon,slat,
     &                                         sdlx,sdly,stheta,lon,lat)
         call ll2xy (lon,lat,bcrd,bnx,bny,bdx,bdy,blon,blat,bdlx,bdly,
     &                                                       btheta,x,y)
c
c-----------------------------------------------------------------------
c        Extract a land mask designed for 2-way nesting.
c-----------------------------------------------------------------------
c
         ix = min( max( int(x), 1), nxbm1 )
         ib = nint(x)
         xmib = abs(x - float(ib))
c
         jy = min( max( int(y), 1), nybm1 )
         jb = nint(y)
         ymjb = abs(y - float(jb))
c
         if ( (xmib.ge.nbhdx) .and. (ymjb.ge.nbhdy) ) then
c
            f_small(i,j) = min( f_big(ix,jy),   f_big(ix+1,jy),
     &                          f_big(ix,jy+1), f_big(ix+1,jy+1) )
c
           else if (xmib.ge.nbhdx) then
c
            f_small(i,j) = min( f_big(ix,jb), f_big(ix+1,jb) )
c
           else if (ymjb.ge.nbhdy) then
c
            f_small(i,j) = min( f_big(ib,jy), f_big(ib,jy+1) )
c
           else
c
            f_small(i,j) = f_big(ib,jb)
c
         end if
c
 10   continue
c
c-----------------------------------------------------------------------
c  Last slab is wasted, merely copy penultimate slab.
c-----------------------------------------------------------------------
c
      do 20 i = 1, snx
         f_small(i,sny) = f_small(i,nysm1)
 20   continue
c
      return
      end
