      subroutine xg_param (bfile,bncid,sfile,sncid,xgdir,xgvar,error)
c
c=======================================================================
c                                                                    ===
c  This routine reads in the run parameters governing a GRIDS        ===
c  extraction.                                                       ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  BFILE...Name of large domain GRIDS netCDF file.        (character)===
c  BNCID...netCDF identifier for large domain GRIDS file. (integer)  ===
c  ERROR...Flag to indicate read error.                   (logical)  ===
c  SFILE...Name of small domain GRIDS netCDF file.        (character)===
c  SNCID...netCDF identifier for small domain GRIDS file. (integer)  ===
c  XGDIR..."Direction" of extraction.                     (integer)  ===
c             [0]  large to small.                                   ===
c             [1]  small to large.                                   ===
c             [2]  large to boundary of small.                       ===
c  XGVAR...Which variables to extract.                    (integer)  ===
c             [0]  topography                                        ===
c             [1]  land mask                                         ===
c             [2]  both                                              ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /BGDOMDAT/                                                        ===
c                                                                    ===
c     BCRD     coordinate type                       (integer)       ===
c     BDLX     x-offset grid-trans. centers          (real; m | deg) ===
c     BDLY     y-offset grid-trans. centers          (real; m | deg) ===
c     BDX      grid spacing in the x-direction       (real; m | deg) ===
c     BDY      grid spacing in the y-direction       (real; m | deg) ===
c     BLAT     transformation centroid latitude      (real; degrees) ===
c     BLON     transformation centroid longitude     (real; degrees) ===
c     BNX      number of gridpoints in x-direction   (integer)       ===
c     BNY      number of gridpoints in y-direction   (integer)       ===
c     BTHETA   domain rotation angle                 (real; degrees) ===
c                                                                    ===
c  /SMDOMDAT/                                                        ===
c                                                                    ===
c     SCRD     coordinate type                       (integer)       ===
c     SDLX     x-offset grid-trans. centers          (real; m | deg) ===
c     SDLY     y-offset grid-trans. centers          (real; m | deg) ===
c     SDX      grid spacing in the x-direction       (real; m | deg) ===
c     SDY      grid spacing in the y-direction       (real; m | deg) ===
c     SLAT     transformation centroid latitude      (real; degrees) ===
c     SLON     transformation centroid longitude     (real; degrees) ===
c     SNX      number of gridpoints in x-direction   (integer)       ===
c     SNY      number of gridpoints in y-direction   (integer)       ===
c     STHETA   domain rotation angle                 (real; degrees) ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     HOPS:    GET_DOM                                               ===
c     netCDF:  NF_OPEN                                               ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <bgdomdat.h>
#include <netcdf.inc>
#include <smdomdat.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer bncid,ncmode,sbgn,send,slen,sncid,xgdir,xgvar
      logical       error,errs
      character*(*) bfile,sfile
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Read in job control parameters.
c-----------------------------------------------------------------------
c
      write (6,10) 'large'
      read (5,20) bfile
      write (6,10) 'small'
      read (5,20) sfile
c
      write (6,30)
      read (5,*) xgdir
c
      write (6,40)
      read (5,*) xgvar
c
      if ((xgdir.lt.0) .or. (xgdir.gt.2)) then
         error = .true.
         write (6,50) xgdir
      end if
c
      if ((xgvar.lt.0) .or. (xgvar.gt.2)) then
         error = .true.
         write (6,60) xgvar
      end if
c
c-----------------------------------------------------------------------
c  Open large domain file.  Read domain definition parameters.
c-----------------------------------------------------------------------
c
c  Make sure supplied name is valid.
c
      call length (bfile,slen,sbgn,send)
c
      if (slen.lt.1) then
         write (6,70) '(large grid)'
         error = .true.
      end if
c
c  Open netCDF file.
c
      if (.not.error) then
         if (xgdir.ne.1) then
            ncmode = NF_NOWRITE
          else
            ncmode = NF_WRITE
         end if
         if (nf_open (bfile(sbgn:send),ncmode,bncid) .ne. NF_NOERR) then
            write (6,80) bfile(sbgn:send)
            error = .true.
         end if
      end if
c
c  Read domain parameters.
c
      if (.not.error) then
         call get_dom (bncid,bcrd,bnx,bny,blon,blat,bdlx,bdly,bdx,bdy,
     &                                                     btheta,error)
         if (error) write (6,90) bfile(sbgn:send)
      end if
c
c-----------------------------------------------------------------------
c  Open small domain file.  Read domain definition parameters.
c-----------------------------------------------------------------------
c
c  Make sure supplied name is valid.
c
      call length (sfile,slen,sbgn,send)
c
      if (slen.lt.1) then
         write (6,70) '(small grid)'
         errs = .true.
      end if
c
c  Open netCDF file.
c
      if (.not.errs) then
         if (xgdir.ne.1) then
            ncmode = NF_WRITE
          else
            ncmode = NF_NOWRITE
         end if
         if (nf_open (sfile(sbgn:send),ncmode,sncid) .ne. NF_NOERR) then
            write (6,80) sfile(sbgn:send)
            errs = .true.
         end if
      end if
c
c  Read domain parameters.
c
      if (.not.errs) then
         call get_dom (sncid,scrd,snx,sny,slon,slat,sdlx,sdly,sdx,sdy,
     &                                                     stheta,errs)
         if (errs) write (6,90) sfile(sbgn:send)
      end if
c
c  Update global error flag.
c
      error = error .or. errs
c
      return
c
 10   format (/'Enter name of ',a,' GRIDS file:')
 20   format (a)
 30   format (/'Choose the direction of the extraction:'/3x,'[0]  from',
     &        ' large to small'/3x,'[1]  from small to large'/3x,
     &        '[2]  from large to boundary of small')
 40   format (/'Choose the variables to extract:'/3x,'[0]  topography'/
     &        3x,'[1]  land mask'/3x,'[2]  both')
 50   format (/'***Error:  XG_PARAM -- invalid value for parameter '
     &        'XGDIR:  ',i10/11x,'Must be in the range [0, 2]'/11x,
     &        'No extraction will be done.')
 60   format (/'***Error:  XG_PARAM -- invalid value for parameter '
     &        'XGVAR:  ',i10/11x,'Must be in the range [0, 2]'/11x,
     &        'No extraction will be done.')
 70   format (/'*** Error:  XG_PARAM - invalid (empty) file name ',a)
 80   format (/'***Error:  XG_PARAM -- unable to open file:'
     &                 /11x,1h",a,1h"/11x,'No extraction will be done.')
 90   format (/'***Error:  XG_PARAM -- unable to read file:'
     &                 /11x,1h",a,1h"/11x,'No extraction will be done.')
c
      end
