       subroutine getobs (iinp,iprt)
c
c=======================================================================
c                                                                    ===
c  This routine reads in hydrographic data and store it in common    ===
c   block HYDRO.                                                     ===
c                                                                    ===
c  On input:                                                         ===
c                                                                    ===
c     IINP     input logical unit (integer)                          ===
c     IPRT     output logical unit (integer)                         ===
c                                                                    ===
c  Common blocks: (only relevant variables documented)               ===
c                                                                    ===
c     /HYDOBS/                                                       ===
c                                                                    ===
c     HDATA    hydrographic data (real 3-D array; output)            ===
c     HVARS    number and type of hydrographic variables available   ===
c              per cast (integer 2-D array; output)                  ===
c     IHFRMT   input format type which determines type and           ===
c              number and order of variables to read (integer;       ===
c              input):                                               ===
c              [0] unformatted: depth, temperature, salinity, and    ===
c                  dynamic height                                    ===
c              [1] unformatted: depth, temperature, and salinity     ===
c              [2] unformatted: depth and temperature                ===
c              [3] unformatted: depth and dynamic height             ===
c              [4] unformatted: depth and any oceanic field          ===
c              [5] unformatted: any field (a single value)           ===
c              [6] MODS hydrography ASCII format                     ===
c     NLEV     number of depth levels to OA (integer; input)         ===
c     NHYDOBS  number of hydrographic observations (integer; output) ===
c              output)                                               ===
c     SPVAL    special value to flag no data (real; input)           ===
c     T        hydrography time of observation (real 1-D array;      ===
c              output)                                               ===
c     X        hydrography zonal position (real 1-D array; output)   ===
c     Y        hydrography meridional position (real 1-D array;      ===
c              output)                                               ===
c     Z        depth level(s) in meters to OA (real 1-D array;       ===
c              input)                                                ===
c                                                                    ===
c  Calls:  EXITUS, GETDYNHT, LINTRP, LL2XY, READ_HEADER, READ_HYDRO  ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
#include <domdat.h>
#include <hydobs.h>
#include <hydro.h>
#include <oagrid.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iinp,iprt,j,k,n
      real c24,c100,c1440,dfrac,gdelx,gdely,gridx,gridy,hour,km2cm,p5,
     &     val,xpos,ypos
      real dh(mlev),wk1(mvar,mpts)
      parameter (c24=24.0,c100=100.0,c1440=1440.0,km2cm=100000.0,p5=0.5)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Set special value.
c
      spval=-9999.
c
c  Set number of variables to read according to input format type.
c
      if (ihfrmt.eq.0) then
        nhvar=4
        write(iprt,10) 'Depth, Temperature, Salinity and dynamic height'
      elseif (ihfrmt.eq.1) then
        nhvar=3
        write(iprt,10) 'Depth, Temperature and Salinity'
      elseif (ihfrmt.eq.2) then
        nhvar=2
        write(iprt,10) 'Depth and Temperature'
      elseif (ihfrmt.eq.3) then
        nhvar=2
        write(iprt,10) 'Depth and Dynamic Height'
      elseif (ihfrmt.eq.4) then
        nhvar=2
        write(iprt,10) 'Depth and any Oceanic Field'
      elseif (ihfrmt.eq.5) then
        nhvar=1
        write(iprt,10) 'a single field value'
      elseif (ihfrmt.eq.6) then
        write(iprt,10) 'MODS hydrography ASCII format'
      endif
  10  format(//' GETOBS : Reading ',a)
      if (idhopt.ne.0) then
        write(iprt,20)
  20    format(' GETOBS : Computing Dynamic Height from (S,T,P)')
      endif
c
c  Start counter for number of observations
c
      nhydobs=0
      ictd=0
      ixbt=0
      ixbts=0
      ixctd=0
c
c=======================================================================
c  Read in hydrography data.
c=======================================================================
c
      eof=.false.
c
c  Read MODS hydrography format header.
c
      if (ihfrmt.eq.6) call read_header (iinp)
c
      do 110 while (.not.eof)
        ctd=.false.
        xbt=.false.
        xbts=.false.
        xctd=.false.
c
c-----------------------------------------------------------------------
c  Read unformatted data.
c-----------------------------------------------------------------------
c
        if ((0.le.ihfrmt).and.(ihfrmt.le.5)) then
c
c  Read in profile header.
c
          read(iinp,end=120) castid,hlat,hlng,jday,ihour
          hour=float(ihour)
          dfrac=float(ihour/100)/c24+mod(hour,c100)/c1440
          htime=float(jday)+dfrac
          hflds(1)='z'
          hflds(2)='t'
          hflds(3)='s'
          hflds(4)='dh'
          if ((1000.lt.castid).and.(castid.lt.2000)) then
            ctd=.true.
          elseif ((2000.lt.castid).and.(castid.lt.3000)) then
            xbt=.true.
          elseif ((3000.lt.castid).and.(castid.lt.4000)) then
            xctd=.true.
          elseif ((4000.lt.castid).and.(castid.lt.5000)) then
            xbts=.true.
          endif
c
c  Read in hydrographic profile(s).
c
          read(iinp,end=120) nhpts,((wk1(i,j),i=1,nhvar),j=1,nhpts)
c
c  Vectorize hydrographic profile(s) to facilitate interpolation.
c
          do 40 j=1,nhpts
            do 30 i=1,nhvar
              hdat(j,i)=wk1(i,j)
  30        continue
  40      continue
c
c-----------------------------------------------------------------------
c  Read formatted data.
c-----------------------------------------------------------------------
c
        elseif (ihfrmt.eq.6) then
          call read_hydro (iinp)
          if (eof) goto 120
        endif
c
c  Use the appropriate coordinate system (Cartesian or spherical).
c
        if (coord.eq.0) then
          gridx=dx*km2cm
          gridy=dy*km2cm
          gdelx=delx*km2cm
          gdely=dely*km2cm
          call ll2xy (hlng,hlat,coord,im,jm,gridx,gridy,clngd,clatd,
     &                gdelx,gdely,thetad,xpos,ypos)
          xpos=xpos*dx
          ypos=ypos*dy
        elseif ((coord.eq.1).or.(coord.eq.2)) then
          xpos=hlng
          ypos=hlat
        endif
c
c-----------------------------------------------------------------------
c  Interpolate hydrographic data to the desired OA depth levels.
c  If applicable, flag out salinity when dealing with observations
c  taken with XBT (it is assumed that observations of XBT origin
c  does not have salinity).
c-----------------------------------------------------------------------
c
c  Set data type counters and flags.
c
        nhydobs=nhydobs+1
        if(nhydobs.gt.mobs) then
           write(iprt,50) nhydobs,mobs
  50       format(/' GETOBS: number of observations exceeded,'/
     &              10x,'NHYDOBS = ',i6/13x,'MOBS = ',i6/
     &              10x,'change  param.h  and recompile.')
           call exitus ('GETOBS')
        endif
        if (ctd) ictd=ictd+1
        if (xbt) ixbt=ixbt+1
        if (xbts) ixbts=ixbts+1
        if (xctd) ixctd=ixctd+1
        hvars(nhydobs,0)=nhvar
        do 60 n=1,nhvar
          if (hflds(n).eq.'z') then
            hvars(nhydobs,n)=dpthid
          elseif (hflds(n).eq.'t') then
            hvars(nhydobs,n)=tempid
          elseif (hflds(n).eq.'s') then
            hvars(nhydobs,n)=saltid
          elseif (hflds(n).eq.'dh') then
            hvars(nhydobs,n)=dynhid
          endif
  60    continue
c
c  Interpolate.
c
        if (ihfrmt.ne.5) then
          do 80 n=2,nhvar
            do 70 k=1,nlev
              if (xbt.and.n.eq.3) then
                val=spval
              else
                call lintrp (nhpts,hdat(1,1),hdat(1,n),1,z(k),val,spval)
              endif
              hdata(nhydobs,k,1)=z(k)
              hdata(nhydobs,k,n)=val
              t(nhydobs)=htime
              x(nhydobs)=xpos
              y(nhydobs)=ypos
  70        continue
  80      continue
        else
          hdata(nhydobs,1,1)=hdat(1,1)
          t(nhydobs)=htime
          x(nhydobs)=xpos
          y(nhydobs)=ypos
        endif
c
c  If requested, compute dynamic height.
c
        if (idhopt.ne.0) then
          hvars(nhydobs,0)=nhvar+1
          hvars(nhydobs,nhvar+1)=dynhid
          if (ctd.or.xctd.or.xbts) then
            call getdynht (dh,z,nlev,zref,idhopt,hdat,nhpts,spval)
            do 90 k=1,nlev
              hdata(nhydobs,k,nhvar+1)=dh(k)
  90        continue
          else
            do 100 k=1,nlev
              hdata(nhydobs,k,nhvar+1)=spval
 100        continue
          endif
        endif
c
c  Read next hydrographic profile(s).
c
 110  continue
 120  close(iinp)
      write(iprt,130) ictd,ixbt,ixbts,ixctd,nhydobs
 130  format(10x,'number of CTD   stations = ',i6,/,
     &       10x,'number of XBT   stations = ',i6,/,
     &       10x,'number of XBTS  stations = ',i6,/,
     &       10x,'number of XCTD  stations = ',i6,/,
     &       10x,'       Hydrography TOTAL = ',i6,/)
      return
      end
