       subroutine oamean (fmean,obsmean,fobs,xobs,yobs,tobs,nobs,fclm,
     &                    xclm,yclm,tclm,nclm,iclima,tmap,nnce,rnce,
     &                    tnce,iprt,found)
c
c=======================================================================
c                                                                    ===
c  This routine computes the mean FMEAN at each OA grid point and    ===
c  the mean  OBSMEAN  at each influential observation point.  The    ===
c  climatology  data is  used in the  computation of  these means    ===
c  when the switch ICLIMA is activated.                              ===
c                                                                    ===
c  On Input:                                                         ===
c                                                                    ===
c     FOBS     observation field (real array)                        ===
c     XOBS     X position of observation (real array)                ===
c     YOBS     Y position of observation (real array)                ===
c     TOBS     time of observation (real array)                      ===
c     NOBS     number of observations (integer)                      ===
c     FCLM     climatological field (real array)                     ===
c     XCLM     X position of climatology (real array)                ===
c     YCLM     Y position of climatology (real array)                ===
c     TCLM     time of climatology (real array)                      ===
c     NCLM     number of observations (integer)                      ===
c     ICLIMA   weighted mean computation option:                     ===
c              [0] use observations only                             ===
c              [1] use climatology only                              ===
c              [2] use both observations and climatology             ===
c              [3] use global observation mean (a single value)      ===
c     TMAP     time of interpolation (real)                          ===
c     NNCE     maximum number of infuential points to use (integer)  ===
c     RNCE     spatial radius of influence (real)                    ===
c     TNCE     temporal radius of influence (real)                   ===
c     IPRT     output logical unit                                   ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     FMEAN    mean at each OA grid point (real array)               ===
c     OBSMEAN  mean at each observation point (real array)           ===
c     FOUND    flag to indicate success of finding mean.  (logical)  ===
c                                                                    ===
c  Required common blocks:                                           ===
c                                                                    ===
c     CORSCL, DOMDAT, OAGRID                                         ===
c                                                                    ===
c  Calls:  EXITUS, OBJAN, SELECT                                     ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
#include <corscl.h>
#include <domdat.h>
#include <oagrid.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
       integer i,iclima,index(mobs),ip,iprt,is,nclm,nnce,nobs,ntot,nsel
       logical found
       real coss,favg,emap,rnce,sins,sum,tdcays,tmap,tnce,uphs,vphs,
     &      xdcays,xzeros,ydcays,yzeros
       real fcor(mobs),fmean(*),fobs(*),fclm(nclm),obsmean(*),
     &      tclm(nclm),tobs(*),xclm(nclm),xobs(*),yclm(nclm),yobs(*)
c
      save xzeros,yzeros,xdcays,ydcays,tdcays,uphs,vphs,coss,sins
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  If using climatology, put the climatology values at the end of the
c  observations arrays.  Notice that this step will not affect the use
c  of these arrays outside of this routine.
c-----------------------------------------------------------------------
c
       if ((iclima.eq.1).or.(iclima.eq.2)) then
         if ((nobs+nclm).gt.mobs) then
           write(iprt,10) mobs,nobs+nclm
 10        format(' OAMEAN: underdimension arrays, MOBS, NTOT = ',2i6)
           call exitus ('OAMEAN')
         endif
         do 20 i=nobs+1,nobs+nclm
           ip=i-nobs
           fobs(i)=fclm(ip)
           tobs(i)=tmap
           xobs(i)=xclm(ip)
           yobs(i)=yclm(ip)
 20      continue
       endif
c
c  Determine arrays starting address and number of points.
c
       if (iclima.eq.0) then
         is=1
         ntot=nobs
       elseif (iclima.eq.1) then
         is=nobs+1
         ntot=nclm
       elseif (iclima.eq.2) then
         is=1
         ntot=nobs+nclm
       elseif (iclima.eq.3) then
         is=1
         ntot=nobs
       endif
c
       found = ntot .gt. 0
c
c-----------------------------------------------------------------------
c  Compute data (observations plus climatology) average, FAVG.
c-----------------------------------------------------------------------
c
       sum=0.
       do 30 i=1,ntot
         ip=is+i-1
         sum=sum+fobs(ip)
 30    continue
       if (ntot.gt.0) then
         favg=sum/float(ntot)
       else
         write(iprt,40) ntot
 40      format(' OAMEAN: no observations available, NTOT = ',i6)
         return
       endif
c
c  As a first step, set the mean at each observation point to FAVG.
c
       do 50 i=1,nobs+nclm
         obsmean(i)=favg
 50    continue
c
c  Set mean to global average.
c
       if (iclima.eq.3) then
         do 55 i=1,imjm
           fmean(i)=favg
 55      continue
         return
       endif
c
c-----------------------------------------------------------------------
c  Compute the mean field at each grid point.
c-----------------------------------------------------------------------
c
       do 60 i=1,imjm
c
c-----------------------------------------------------------------------
c  Determine the most influential observation indices.
c-----------------------------------------------------------------------
c
         call select (fcor,index,nsel,fobs(is),xobs(is),yobs(is),
     &                tobs(is),ntot,xmap(i),ymap(i),tmap,rnce,tnce,iprt)
c
c-----------------------------------------------------------------------
c  Perform an OA to determine the mean at each grid point.
c-----------------------------------------------------------------------
c
         call objan (fcor,index,nsel,nnce,fobs(is),xobs(is),yobs(is),
     &               tobs(is),obsmean,ntot,iclima,nobs,xmap(i),ymap(i),
     &               tmap,favg,fmean(i),emap,iprt)
 60    continue
c
c-----------------------------------------------------------------------
c  If the observation point is outside of the OA grid, perform an OA
c  to get the observation mean OBSMEAN at those points.
c  If the observation point is inside the OA domain, interpolate the
c  observation mean OBSMEAN using cubic Bessel interpolation from OA
c  grid mean values FMEAN.
c-----------------------------------------------------------------------
c
       do 70 i=1,nobs
          call select (fcor,index,nsel,fobs(is),xobs(is),yobs(is),
     &                 tobs(is),ntot,xobs(i),yobs(i),tmap,rnce,tnce,
     &                 iprt)
          call objan (fcor,index,nsel,nnce,fobs(is),xobs(is),yobs(is),
     &                tobs(is),obsmean,ntot,iclima,nobs,xobs(i),
     &                yobs(i),tmap,favg,obsmean(i),emap,iprt)
 70    continue
c
       return
c
c-----------------------------------------------------------------------
c  Activate correlation scales for computing weighted mean.  Store
c  OA mapping correlation scales.
c-----------------------------------------------------------------------
c
      entry large_scales
c
      xzeros=xzero
      yzeros=yzero
      xdcays=xdcay
      ydcays=ydcay
      tdcays=tdcay
      uphs=uphse
      vphs=vphse
      coss=cosang
      sins=sinang
c
      xzero=xzerom
      yzero=yzerom
      xdcay=xdcaym
      ydcay=ydcaym
      tdcay=tdcaym
      uphse=uphsem
      vphse=vphsem
      cosang=cosangm
      sinang=sinangm
c
      return
c
c-----------------------------------------------------------------------
c  Restore saved correlation scales.
c-----------------------------------------------------------------------
c
      entry synop_scales
c
      xzero=xzeros
      yzero=yzeros
      xdcay=xdcays
      ydcay=ydcays
      tdcay=tdcays
      uphse=uphs
      vphse=vphs
      cosang=coss
      sinang=sins
c
      return
      end
