       subroutine outlier (fobs,xobs,yobs,tobs,nobs,rout,iprt)
c
c=======================================================================
c                                                                    ===
c  This routine removes data that there is far from the domain       ===
c  of interest.  Any data,  whose  east-west  and  north-south       ===
c  distance is greater than ROUT is removed now. This saves on       ===
c  computations later.                                               ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     FOBS     observation field.           (real array)             ===
c     XOBS     X position of observation.   (real array)             ===
c     YOBS     Y position of observation.   (real array)             ===
c     TOBS     time of observation.         (real array)             ===
c     NOBS     number of observations.      (integer)                ===
c     ROUT     outlier radius.              (real)                   ===
c     IPRT     output logical unit.         (integer)                ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /DEBUG/                                                           ===
c                                                                    ===
c     IWRT   flag to write out verbose output:      (integer)        ===
c            [0] no, [1] yes                                         ===
c                                                                    ===
c  /DOMDAT/                                                          ===
c                                                                    ===
c     CLATD    domain centroid latitude.            (real; degrees)  ===
c     CLNGD    domain centroid longitude.           (real; degrees)  ===
c     DX       domain grid spacing in x-direction.  (real; km | deg) ===
c     DY       domain grid spacing in y-direction.  (real; km | deg) ===
c     THETAD   domain rotation angle.               (real; degrees)  ===
c                                                                    ===
c  /OAGRID/                                                          ===
c                                                                    ===
c     COORD  objective analysis coordinates option. (integer)        ===
c            [0] Cartesian                                           ===
c            [1] unrotated spherical                                 ===
c            [2] rotated spherical                                   ===
c     DELX,                                                          ===
c     DELY   Offset grid-trans. centers.            (real; km | deg) ===
c     IM     number of points in the x-direction.   (integer)        ===
c     JM     number of points in the y-direction.   (integer)        ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     FOBS     observation field.           (real array)             ===
c     XOBS     X position of observation.   (real array)             ===
c     YOBS     Y position of observation.   (real array)             ===
c     TOBS     time of observation.         (real array)             ===
c     NOBS     number of observations.      (integer)                ===
c                                                                    ===
c  Calls:  GCIRCLE, XY2LL                                            ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
#include <debug.h>
#include <domdat.h>
#include <oagrid.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
       integer i,iprt,nacc,nobs,nrej
       real dang,dist,fsav,gcircle,p5,phi,rout,tsav,xc,xcen,xsav,yc,
     &      ycen,ysav
       real fobs(nobs),tobs(nobs),xobs(nobs),yobs(nobs)
       parameter (p5=0.5)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Start counters for accepted and rejected observations.
c
       if (coord.eq.0) then
        xcen=p5*float(im+1)
        ycen=p5*float(jm+1)
       elseif ((coord.eq.1).or.(coord.eq.2)) then
         xc = p5*float(im+1)
         yc = p5*float(jm+1)
         call xy2ll (xc,yc,coord,im,jm,dx,dy,clngd,clatd,
     *                 delx,dely,thetad,xcen,ycen)
       endif
       nacc=0
       nrej=0
c
c  Remove outliers.
c
       do 20 i=1,nobs
         fsav=fobs(i)
         xsav=xobs(i)
         ysav=yobs(i)
         tsav=tobs(i)
         if (coord.eq.0) then
           dist=sqrt((xsav-xcen)**2+(ysav-ycen)**2)
         elseif ((coord.eq.1).or.(coord.eq.2)) then
           dang=gcircle(xcen,ycen,xsav,ysav,dist,phi)
         endif
         if (dist.lt.rout) then
           nacc=nacc+1
           fobs(nacc)=fsav
           xobs(nacc)=xsav
           yobs(nacc)=ysav
           tobs(nacc)=tsav
         else
           nrej=nrej+1
           if (iwrt.ne.0) write(iprt,10) xsav,ysav,tsav,dist
 10        format(' OUTLIER : Observation Removed, (X,Y,T) = ',3f12.4,/,
     &            34x,'Dist=',f12.4)
         endif
 20    continue
c
c  Set new number of observations.
c
       if (nrej.gt.0) write(iprt,30) nrej,nobs
 30    format(' OUTLIER : ',i5,' outliers removed out of ',i5,
     &        ' observations',/)
       nobs=nacc
c
#ifdef sunflush
c-----------------------------------------------------------------------
c  Flush output buffers.
c-----------------------------------------------------------------------
c
       call flush(iprt)
c
#endif
       return
       end
