      function brent(ax,bx,cx,f,amat,bmat,pc,xic,wk,n,tol,xmin)
c
c=======================================================================
c                                                                    ===
c  Given a function F,  and given a bracketing triplet of abscissas  ===
c  AX, BX, CX (such that BX is between AX and CX, and F(BX) is less  ===
c  than both F(AX) and F(CX)), this routine isolates the minimum to  ===
c  a fractional precision of about  TOL  using Brent  method.   The  ===
c  abscissa of the  minimum is returned as  XMIN,   and the minimum  ===
c  function value is returned as BRENT, the retuned function value.  ===
c                                                                    ===
c  On Input:                                                         ===
c                                                                    ===
c     AX     bracketing abscissa (real).                             ===
c     BX     bracketing abscissa (real).                             ===
c     CX     bracketing abscissa (real).                             ===
c     AMAT   symmetric, positive-definite, NxN matrix to invert      ===
c            using conjugate gradient method (real array).           ===
c     BMAT   right-hand side vector (real array).                    ===
c     PC     initial position (real vector)                          ===
c     XIC    initial direction(real vector)                          ===
c     WK     working vector   (real vector)                          ===
c     N      length of vector (integer).                             ===
c     TOL    fractional precision (real).                            ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     BRENT  minimum value of cost function (real).                  ===
c     XMIN   abscissa at the minimum (real).                         ===
c                                                                    ===
c  Calls:    EXITUS, F                                               ===
c                                                                    ===
c  Reference:  Press, W.H. et al., 1989: Numerical Recipes, The Art  ===
c              of Scientific Computing,  pp 284-286.                 ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdescent.h>
#include <inppar.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer its,n
      real a,ax,b,bx,c0,c2,cgold,cx,d,e,etemp,fu,fv,fw,fx,p,p5,q,r,tol,
     *     tol1,tol2,u,v,w,x,xm,xmin,zeps
      real amat(n,n),bmat(n),pc(n),xic(n),wk(n)
      real brent,f
      parameter (c0=0.0,c2=2.0,cgold=.3819660,p5=0.5,zeps=1.0e-10)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
      a=min(ax,cx)
      b=max(ax,cx)
      v=bx
      w=v
      x=v
      e=c0
      fx=f(amat,x,bmat,pc,xic,wk,n)
      fv=fx
      fw=fx
      do 30 its=1,itmax
        biter=its
        xm=p5*(a+b)
        tol1=tol*abs(x)+zeps
        tol2=c2*tol1
        if(abs(x-xm).le.(tol2-p5*(b-a))) goto 40
        if(abs(e).gt.tol1) then
          r=(x-w)*(fx-fv)
          q=(x-v)*(fx-fw)
          p=(x-v)*q-(x-w)*r
          q=c2*(q-r)
          if(q.gt.c0) p=-p
          q=abs(q)
          etemp=e
          e=d
          if((abs(p).ge.abs(p5*q*etemp)).or.(p.le.q*(a-x)).or.
     *       (p.ge.q*(b-x))) goto 10
          d=p/q
          u=x+d
          if((u-a.lt.tol2).or.(b-u.lt.tol2)) d=sign(tol1,xm-x)
          goto 20
        endif
  10    if(x.ge.xm) then
          e=a-x
        else
          e=b-x
        endif
        d=cgold*e
  20    if(abs(d).ge.tol1) then
          u=x+d
        else
          u=x+sign(tol1,d)
        endif
        fu=f(amat,u,bmat,pc,xic,wk,n)
        if(fu.le.fx) then
          if(u.ge.x) then
            a=x
          else
            b=x
          endif
          v=w
          fv=fw
          w=x
          fw=fx
          x=u
          fx=fu
        else
          if(u.lt.x) then
            a=u
          else
            b=u
          endif
          if((fu.le.fw).or.(w.eq.x)) then
            v=w
            fv=fw
            w=u
            fw=fu
          elseif((fu.le.fv).or.(v.eq.x).or.(v.eq.w)) then
            v=u
            fv=fu
          endif
        endif
  30  continue
      write (iprt,'(/a)') 'BRENT:  exceed maximum number of iterations.'
      call exitus ('BRENT')
  40  xmin=x
      brent=fx
      return
      end
