      subroutine descent(a,x,b,g,h,xi,wk,n,ier)
c
c=======================================================================
c                                                                    ===
c  This routine solves the linear system of equations                ===
c                             A x = b                                ===
c  for the vector  X  of lenght N using the fletcher-Reeves-Polak-   ===
c  Ribiere minimization method. The cost function and its gradient   ===
c  is evaluated in routines COSTFUN and DCOSTFUN, respectively.      ===
c                                                                    ===
c  On Input:                                                         ===
c                                                                    ===
c     A      symmetric, positive-definite, NxN matrix to invert      ===
c            using conjugate gradient method (real array).           ===
c     X      first guest solution vector (real array).               ===
c     B      right-hand side vector (real array).                    ===
c     G      auxiliary vector to store the evaluation of the         ===
c            gradient (real array).                                  ===
c     H      auxiliary vector to store the sequence of directions    ===
c            (real array).                                           ===
c     XI     auxiliary vector (real array).                          ===
c     WK     working vector   (real vector)                          ===
c     N      length of vector (integer).                             ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     X      solution vector to the linear system (real array).      ===
c     IER    error flag (integer):                                   ===
c            IER=0 successful convergence.                           ===
c            IER=1 no convergence.                                   ===
c                                                                    ===
c  Calls:    COSTFUN, DCOSTFUN, F1DIM                                ===
c                                                                    ===
c  Reference:  Press, W.H. et al., 1989: Numerical Recipes, The Art  ===
c              of Scientific Computing,  pp 301-307.   Adapted from  ===
c              routine FRPRMN and simplified 1D search.              ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdescent.h>
#include <inppar.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer ier,j,n
      integer its
      real c0,c2,dgg,fx,gam,gg
      real a(n,n),b(n),g(n),h(n),x(n),xi(n)
      real wk(n)
      real costfun
      real xlim,a1,gold,f1dim,c1,small
      parameter (gold=1.618034,c1=1.0,small=1.0e-33)
      parameter (c0=0.0,c2=2.0)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Initializations.
c
      ier=0
      fx=costfun(a,x,b,n,1)
      call dcostfun(a,x,b,xi,n)
      gg=c0
      do 10 j=1,n
        g(j)=-xi(j)
        gg=gg+g(j)**2
        h(j)=g(j)
        xi(j)=h(j)
  10  continue
      if(gg.eq.c0) return      
c
c Set max iterations to the matrix size
c
      itmax=n

      do 60 its=1,itmax
c
c  Set 1D point
c
        xlim=c2*costfun(a,xi,b,n,0)
        if(abs(xlim).ge.small) then
           call dcostfun(a,x,b,wk,n)
           a1=c0
           do 20 j=1,n
              a1=a1+wk(j)*xi(j)
 20        continue
           xlim=-a1/xlim
           fret=f1dim(a,xlim,b,x,xi,wk,n)
        else
           ier=2
        endif
c
c Ensure NOT ascent
c
        if(fret .le. fx ) then
c
c  Construct direction xi and update of x
c
           do 30 j=1,n
              xi(j)=xlim*xi(j)
              x(j)=x(j)+xi(j)
 30        continue
        else
           ier=1
        endif
        if(ier.ge.1) goto 80
        if(c2*abs(fret-fx).le.ftol*(abs(fret)+abs(fx)+eps)) return
        fx=costfun(a,x,b,n,1)
        call dcostfun(a,x,b,xi,n)
        gg=c0
        dgg=c0
        do 40 j=1,n
          gg=gg+g(j)**2
c         dgg=dgg+xi(j)**2
          dgg=dgg+(xi(j)+g(j))*xi(j)
  40    continue
        if(gg.eq.c0) return
        gam=dgg/gg
        do 50 j=1,n
          g(j)=-xi(j)
          h(j)=g(j)+gam*h(j)
          xi(j)=h(j)
  50    continue
  60  continue
c
c Exceeded max iterations
c
      ier=1
      write(iprt,1000)itmax
      goto 100
c
c Limited search found no descent
c
 80   continue
      write(iprt,1500)ier
 100  continue
 1000 format(/' DESCENT: max number of iterations ',
     &        i10,' were exceeded.')
 1500 format(/' DESCENT: direction of descent was not found. '/
     &        '            IER = ',i2)
      return
      end
