       subroutine dynht (dh,zlev,m,zref,s,t,z,n,idhopt,spval)
c
c=======================================================================
c                                                                    ===
c  Given the vertical profiles of temperature T(z) and salinity S(z) ===
c  of length N,  this subroutine computes the dynamic height anomaly ===
c  DH(z)  at the  provided  M  depth  levels (in meters),  ZLEV.  It ===
c  integrates  the  specific  volume  anomaly  from the  level of no ===
c  motion ZREF (in meters) to the surface  (IDHOPT=1),  or from ZREF ===
c  the bottom (IDHOPT=2),  or from ZREF to the surface and then from ===
c  ZREF to the bottom (IDHOPT=3).                                    ===
c                                                                    ===
c                                                 z2                 ===
c                   dh(z) = integral [ sva(z) dz ]                   ===
c                                                 z1                 ===
c  On Input:                                                         ===
c                                                                    ===
c     ZLEV     depths where to compute dynamic height (real array;   ===
c              meters)                                               ===
c     M        number of depth levels (integer)                      ===
c     ZREF     depth level of no motion (real; meters)               ===
c     S        salinity profile (real array; PSS)                    ===
c     T        temperature profile (real array; deg C)               ===
c     Z        depths of profile (real array; meters)                ===
c     N        number of point in profile (integer)                  ===
c     IDHOPT   integration direction (integer)                       ===
c              IDHOPT = 1  from ZREF to the surface                  ===
c              IDHOPT = 2  from ZREF to the bottom                   ===
c              IDHOPT = 3  both above directions from ZREF           ===
c     SPVAL    special value to flag no data (real)                  ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     DH       dynamic height (real array; dynamic meters)           ===
c                                                                    ===
c  Required common blocks:                                           ===
c                                                                    ===
c     TRAPZ                                                          ===
c                                                                    ===
c  Calls:  EXITUS, QTRAP, SVAN, SVAFUNC                              ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
#include <inppar.h>
#include <trapz.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
       integer i,idhopt,k,kref,m,n
       real a,alpha,b,zref,sigma,sigmastn,sum,svascl,spval
       real dh(m),dhl(0:nstp),s(n),stnvol(nstp),svol(nstp),t(n),z(n),
     &      zlev(m),zbot
       real svan
       parameter (svascl=1.0e-5)
       external svafunc
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Check size of working arrays
c
       if (n.gt.nstp) then
         write (iprt,10) nstp, n
 10      format(/' DYNHT: underdimensioned arrays, NSTP, N = ',2i5,/,
     &          '        change NSTP and recompile',/)
         call exitus ('DYNHT')
       endif
       if ((idhopt.lt.1).or.(idhopt.gt.3)) then
         write (iprt,20) idhopt
 20      format(/' DYNHT: illegal flag, IDHOPT = ',i5,/)
         call exitus ('DYNHT')
       endif
c
c  Set dynamic height array to spval.
c
       do 30 k=1,m
           dh(k)=spval
 30    continue
c
c-----------------------------------------------------------------------       c
c  Fill common block to compute function svafunc
c-----------------------------------------------------------------------
c
       np=n
       do 40 i=1,n
         stnvol(i)=svan(35.0,0.0,z(i),sigmastn)
         svol(i)=svan(s(i),t(i),z(i),sigma)
         sva(i)=(svol(i)-stnvol(i))*svascl
         h(i)=-abs(z(i))
 40    continue
       zbot=abs(z(n))
c
c------------------------------------------------------------------------
c  Calculate dynamic height anomaly by integrating the specific volume
c  anomaly using refined trapezoidal rule:
c
c                                          b
c                    integral [ sva(z) dz ]
c                                          a
c------------------------------------------------------------------------
c
c  Integrate from ZREF to the surface.
c
       if ((idhopt.eq.1).or.(idhopt.eq.3)) then
c
c Find range of integration
c
c
         kref = 0
         do 41 k=1,m
           if (abs(zlev(k)).lt.abs(zref)) kref=k
 41      continue
         dhl(kref+1)=-abs(zref)
         do 42 k=kref,1,-1
           dhl(k)=-abs(zlev(k))
 42      continue
c
         if(kref.lt.m) then
            if(abs(zref).eq.abs(zlev(kref+1))) then
               dh(kref+1)=0.0
            endif
         endif
c
         sum=0.
         do 50 k=kref,1,-1
           a=dhl(k+1)
           b=dhl(k)
           call qtrap (svafunc,a,b,alpha)
           sum=sum+alpha
           dh(k)=sum
 50      continue
       endif
c
c  Integrate from ZREF to the bottom.
c
       if (idhopt.eq.2.or.idhopt.eq.3) then
c
c Find range of integration
c
         kref=m+1
         do 51 k=m,1,-1
           if (abs(zlev(k)).gt.abs(zref)) kref=k
 51      continue
         dhl(kref-1)=-abs(zref)
         do 52 k=kref,m
           dhl(k)=-abs(zlev(k))
 52      continue
c
         if(kref.gt.1) then
            if(abs(zref).eq.abs(zlev(kref-1))) then
               dh(kref-1)=0.0
            endif
         endif
c
         sum=0.
         do 60 k=kref,m
           a=dhl(k-1)
           b=dhl(k)
           if(abs(b).le.zbot) then
              call qtrap (svafunc,a,b,alpha)
              sum=sum+alpha
              dh(k)=sum
           endif
 60      continue
       endif
c
       return
       end
