       subroutine getclima (iclm,iprt,rout)
c
c=======================================================================
c                                                                    ===
c  This routine reads in climatological data and store it in common  ===
c  block CLIMA.                                                      ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     ICLM     input logical unit.     (integer)                     ===
c     IPRT     output logical unit.    (integer)                     ===
c     ROUT     outlier radius.         (real; km | deg)              ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /CLIMA/                                                           ===
c                                                                    ===
c     ICFRMT   input format type which determines type               ===
c              and number and order of variables to read.   (integer)===
c              [0] depth (pressure), temperature,                    ===
c                  salinity, and dynamic height                      ===
c              [1] depth (pressure), temperature, and salinity       ===
c              [2] depth (pressure) and temperature                  ===
c              [3] depth (pressure) and dynamic height               ===
c              [4] depth (pressure) and any oceanic field            ===
c              [5] any field (a single value)                        ===
c              [6] MODS hydrography ASCII format                     ===
c                                                                    ===
c  /DOMDAT/                                                          ===
c                                                                    ===
c     CLATD    domain centroid latitude.            (real; degrees)  ===
c     CLNGD    domain centroid longitude.           (real; degrees)  ===
c     DX       domain grid spacing in x-direction.  (real; km | deg) ===
c     DY       domain grid spacing in y-direction.  (real; km | deg) ===
c     THETAD   domain rotation angle.               (real; degrees)  ===
c                                                                    ===
c  /HYDOBS/                                                          ===
c                                                                    ===
c     NLEV     number of depth levels to OA.     (integer)           ===
c     SPVAL    special value to flag no data.    (real)              ===
c     Z        depth level(s) in meters to OA.   (real vector)       ===
c                                                                    ===
c  /OAGRID/                                                          ===
c                                                                    ===
c     COORD  objective analysis coordinates option. (integer)        ===
c            [0] Cartesian                                           ===
c            [1] unrotated spherical                                 ===
c            [2] rotated spherical                                   ===
c     DELX,                                                          ===
c     DELY   Offset grid-trans. centers.            (real; km | deg) ===
c     IM     number of points in the x-direction.   (integer)        ===
c     JM     number of points in the y-direction.   (integer)        ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /CLIMA/                                                           ===
c                                                                    ===
c     CDATA    climatological data.                   (real array)   ===
c     CVARS    number and type of climatology                        ===
c              variables availabl per cast.           (integer array)===
c     NCLMOBS  number of climatological observations. (integer)      ===
c     TC       climatology time of observation.       (real vector)  ===
c     XC       climatology zonal position             (real vector)  ===
c     YC       climatology meridional position        (real vector)  ===
c                                                                    ===
c  Calls:  EXITUS, GETDYNHT, LINTRP, LL2XY, READ_HEADER, READ_HYDRO, ===
c          XY2LL                                                     ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
#include <clima.h>
#include <domdat.h>
#include <hydobs.h>
#include <hydro.h>
#include <oagrid.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iclm,ioutsta,iprt,j,k,n
      real c24,c100,c1440,dang,dfrac,dist,gdelx,gdely,gridx,gridy,hour,
     &     km2cm,m2km,p5,phi,rout,val,xcen,xpos,xwk,ycen,ypos,ywk
      real dh(mlev),wk1(mvar,mpts)
      real gcircle
      parameter (c24=24.0,c100=100.0,c1440=1440.0,km2cm=100000.0,
     &           m2km=0.001,p5=0.5)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Set centroid.
c
      if (coord.eq.0) then
        xcen=p5*float(im+1)*dx
        ycen=p5*float(jm+1)*dx
      elseif ((coord.eq.1).or.(coord.eq.2)) then
        xwk = p5*float(im+1)
        ywk = p5*float(jm+1)
        call xy2ll (xwk,ywk,coord,im,jm,dx,dy,clngd,clatd,
     *                 delx,dely,thetad,xcen,ycen)
      endif
c
c  Set number of variables to read according to input format type.
c
      if (icfrmt.eq.0) then
        nhvar=4
        write(iprt,10) 'Depth, Temperature, Salinity and dynamic height'
      elseif (icfrmt.eq.1) then
        nhvar=3
        write(iprt,10) 'Depth, Temperature and Salinity'
      elseif (icfrmt.eq.2) then
        nhvar=2
        write(iprt,10) 'Depth and Temperature'
      elseif (icfrmt.eq.3) then
        nhvar=2
        write(iprt,10) 'Depth and Dynamic Height'
      elseif (icfrmt.eq.4) then
        nhvar=2
        write(iprt,10) 'Depth and any Oceanic Field'
      elseif (icfrmt.eq.5) then
        nhvar=1
        write(iprt,10) 'a single field value'
      elseif (icfrmt.eq.6) then
        write(iprt,10) 'MODS hydrography ASCII format'
      endif
  10  format(//' GETCLIMA : Reading ',a)
      if (idhopt.ne.0) then
        write(iprt,20)
  20    format(' GETCLIMA : Computing Dynamic Height from (S,T,P)')
      endif
c
c  Start counter for number of observations.
c
      nclmobs=0
      ictd=0
      ixbt=0
      ixbts=0
      ixctd=0
      ioutsta=0
c
c=======================================================================
c  Read in climatology data.
c=======================================================================
c
      eof=.false.
c
c  Read MODS hydrography format header.
c
      if (icfrmt.eq.6) call read_header (iclm)
c
c
c-----------------------------------------------------------------------
c  Read in climatology data.
c-----------------------------------------------------------------------
c
      do 110 while (.not.eof)
c
c  Read in profile header.
c
        ctd=.false.
        xbt=.false.
        xbts=.false.
        xctd=.false.
c
c-----------------------------------------------------------------------
c  Read unformatted data.
c-----------------------------------------------------------------------
c
        if ((0.le.icfrmt).and.(icfrmt.le.5)) then
c
c  Read in profile header.
c
          read(iclm,end=120) castid,hlat,hlng,jday,ihour
          hour=float(ihour)
          dfrac=float(ihour/100)/c24+mod(hour,c100)/c1440
          htime=float(jday)+dfrac
          hflds(1)='z'
          hflds(2)='t'
          hflds(3)='s'
          hflds(4)='dh'
          if ((1000.lt.castid).and.(castid.lt.2000)) then
            ctd=.true.
          elseif ((2000.lt.castid).and.(castid.lt.3000)) then
            xbt=.true.
          elseif ((3000.lt.castid).and.(castid.lt.4000)) then
            xctd=.true.
          elseif ((4000.lt.castid).and.(castid.lt.5000)) then
            xbts=.true.
          endif
c
c  Read in climatology profile(s).
c
          read(iclm,end=120) nhpts,((wk1(i,j),i=1,nhvar),j=1,nhpts)
c
c  Vectorize climatology profile(s) to facilitate interpolation.
c
          do 40 j=1,nhpts
            do 30 i=1,nhvar
              hdat(j,i)=wk1(i,j)
  30        continue
  40      continue
c
c-----------------------------------------------------------------------
c  Read formatted data.
c-----------------------------------------------------------------------
c
        elseif (icfrmt.eq.6) then
          call read_hydro (iclm)
          if (eof) goto 120
        endif
c
c  Use the appropriate coordinate system (Cartesian or spherical).
c
        if (coord.eq.0) then
          gridx=dx*km2cm
          gridy=dy*km2cm
          gdelx=delx*km2cm
          gdely=dely*km2cm
          call ll2xy (hlng,hlat,coord,im,jm,gridx,gridy,clngd,clatd,
     &                gdelx,gdely,thetad,xpos,ypos)
          xpos=xpos*dx
          ypos=ypos*dy
          dist=sqrt((xpos-xcen)**2+(ypos-ycen)**2)
        elseif ((coord.eq.1).or.(coord.eq.2)) then
          xpos=hlng
          ypos=hlat
          dang=gcircle(xcen,ycen,xpos,ypos,dist,phi)
        endif
c
c  Interpolate climatological data to the desired OA depth levels.
c  If applicable, flag out salinity when dealing with observations
c  taken with XBT (it is assumed that observations of XBT origin
c  does not have salinity).
c
        if (dist.lt.rout) then
c
c  Set data type counters and flags.
c
          nclmobs=nclmobs+1
          if(nclmobs.gt.mobs) then
           write(iprt,50) nclmobs,mobs
  50       format(/' GETCLIMA: number of observations exceeded,'/
     &              12x,'NCLMOBS = ',i6/15x,'MOBS = ',i6/
     &              12x,'change  param.h  and recompile.')
           call exitus ('GETCLIMA')
          endif
          if (ctd) ictd=ictd+1
          if (xbt) ixbt=ixbt+1
          if (xbts) ixbts=ixbts+1
          if (xctd) ixctd=ixctd+1
          cvars(nclmobs,0)=nhvar
          do 60 n=1,nhvar
            if (hflds(n).eq.'z') then
              cvars(nclmobs,n)=dpthid
            elseif (hflds(n).eq.'t') then
              cvars(nclmobs,n)=tempid
            elseif (hflds(n).eq.'s') then
              cvars(nclmobs,n)=saltid
            elseif (hflds(n).eq.'dh') then
              cvars(nclmobs,n)=dynhid
            endif
  60      continue
c
c  Interpolate.
c
          if (icfrmt.ne.5) then
            do 80 n=2,nhvar
              do 70 k=1,nlev
                if (xbt.and.n.eq.3) then
                  val=spval
                else
                  call lintrp (nhpts,hdat(1,1),hdat(1,n),1,z(k),val,
     &                         spval)
                endif
                cdata(nclmobs,k,1)=z(k)
                cdata(nclmobs,k,n)=val
                tc(nclmobs)=htime
                xc(nclmobs)=xpos
                yc(nclmobs)=ypos
  70          continue
  80        continue
          else
            cdata(nclmobs,1,1)=hdat(1,1)
            tc(nclmobs)=htime
            xc(nclmobs)=xpos
            yc(nclmobs)=ypos
          endif
c
c  If requested, compute dynamic height.
c
          if (idhopt.ne.0) then
            cvars(nclmobs,0)=nhvar+1
            cvars(nclmobs,nhvar+1)=dynhid
            if (ctd.or.xctd.or.xbts) then
              call getdynht (dh,z,nlev,zref,idhopt,hdat,nhpts,spval)
              do 90 k=1,nlev
                cdata(nclmobs,k,nhvar+1)=dh(k)
  90          continue
            else
              do 100 k=1,nlev
                cdata(nclmobs,k,nhvar+1)=spval
 100          continue
            endif
          endif
        else
          ioutsta=ioutsta+1
        endif
c
c  Read next climatological profile(s).
c
 110  continue
 120  close(iclm)
      write(iprt,130) ioutsta,ictd,ixbt,ixbts,ixctd,nclmobs
 130  format(12x,'number of OUTLIER stations = ',i6,/,
     &       12x,'number of CTD     stations = ',i6,/,
     &       12x,'number of XBT     stations = ',i6,/,
     &       12x,'number of XBTS    stations = ',i6,/,
     &       12x,'number of XCTD    stations = ',i6,/,
     &       12x,'         Climatology TOTAL = ',i6,/)
       return
       end
