      subroutine mnbrak(ax,bx,cx,fa,fb,fc,func,amat,bmat,pc,xic,wk,n)
c
c=======================================================================
c                                                                    ===
c  Given a function FUNC,  and given distinct initial points AX and  ===
c  BX,  this routine searches in the downhill direction (defined by  ===
c  the function as evaluated at the initial points) and returns new  ===
c  points AX, BX, CX which bracket a minimum of the function.  Also  ===
c  returned are the function values a the three points  FA, FB, and  ===
c  FC.                                                               ===
c                                                                    ===
c  On Input:                                                         ===
c                                                                    ===
c     AX     initial search point A (real).                          ===
c     BX     initial search point B (real).                          ===
c     AMAT   symmetric, positive-definite, NxN matrix to invert      ===
c            using conjugate gradient method (real array).           ===
c     BMAT   right-hand side vector (real array).                    ===
c     PC     initial position (real vector)                          ===
c     XIC    initial direction(real vector)                          ===
c     WK     working vector   (real vector)                          ===
c     N      length of vector (integer).                             ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     AX     bracketing abscissa A (real).                           ===
c     BX     bracketing abscissa B (real).                           ===
c     CX     bracketing abscissa C (real).                           ===
c     FA     function value at A (real).                             ===
c     FB     function value at B (real).                             ===
c     FC     function value at C (real).                             ===
c                                                                    ===
c  Calls:    FUNC                                                    ===
c                                                                    ===
c  Reference:  Press, W.H. et al., 1989: Numerical Recipes, The Art  ===
c              of Scientific Computing,  pp 281-282.                 ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
       integer n,idone
       real ax,bx,c0,cx,dum,fa,fb,fc,fu,glimit,gold,q,r,tiny,u,ulim
       real amat(n,n),bmat(n),pc(n),xic(n),wk(n)
       real func
       parameter (c0=0.0,glimit=100.0,gold=1.618034,tiny=1.e-20)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
      fa=func(amat,ax,bmat,pc,xic,wk,n)
      fb=func(amat,bx,bmat,pc,xic,wk,n)
c
c  Switch roles of A and B so that we can go downhill in the direction
c  from A to B.
c
      if(fb.gt.fa)then
        dum=ax
        ax=bx
        bx=dum
        dum=fb
        fb=fa
        fa=dum
      endif
c
c  First guess for C.
c
      cx=bx+gold*(bx-ax)
      fc=func(amat,cx,bmat,pc,xic,wk,n)
c
c  Keep returning here until we bracket.
c
      idone=0
 10   continue
      if(fb.ge.fc) then
c
c  Compute U by parabolic extrapolation from A, B, C.  TINY is used to
c  prevent any possible division by zero.
c
        r=(bx-ax)*(fb-fc)
        q=(bx-cx)*(fb-fa)
        u=bx-((bx-cx)*q-(bx-ax)*r)/(2.*sign(max(abs(q-r),tiny),q-r))
        ulim=bx+glimit*(cx-bx)
c
c  We will not go further than this.  Now test various possibilities:
c
        if((bx-u)*(u-cx).gt.c0) then
c
c  Parabolic U is between B and C: try it.
c
           fu=func(amat,u,bmat,pc,xic,wk,n)
c
c  Got minimum between B and C.
c
           if(fu.lt.fc) then
              ax=bx
              fa=fb
              bx=u
              fb=fu
              idone=1
c
c  Got minimum between A and U.
c
           elseif(fu.gt.fb) then
              cx=u
              fc=fu
              idone=1
           endif
           if (idone.eq.0) then
c
c  Parabolic fit was no use.  Use default magnification.
c
              u=cx+gold*(cx-bx)
              fu=func(amat,u,bmat,pc,xic,wk,n)
           endif
c
c  Parabolic fit is between C and its allowed limit.
c
        elseif((cx-u)*(u-ulim).gt.c0) then
           fu=func(amat,u,bmat,pc,xic,wk,n)
           if(fu.lt.fc) then
              bx=cx
              cx=u
              u=cx+gold*(cx-bx)
              fb=fc
              fc=fu
              fu=func(amat,u,bmat,pc,xic,wk,n)
           endif
c
c  Limit parabolic U to maximum allowed value.
c
        elseif((u-ulim)*(ulim-cx).ge.c0) then
           u=ulim
           fu=func(amat,u,bmat,pc,xic,wk,n)
c
c  Reject parabolic U, use default magnification.
c
        else
           u=cx+gold*(cx-bx)
           fu=func(amat,u,bmat,pc,xic,wk,n)
        endif
c
c  Eliminate oldest point and continue.
c
        if(idone.eq.0) then
           ax=bx
           bx=cx
           cx=u
           fa=fb
           fb=fc
           fc=fu
        endif
      endif
      if(idone.eq.0) goto 10
      return
      end
