      subroutine gradconv (ncmpct,pcmpct,atol,rtol,isconv)
c
c=======================================================================
c                                                                    ===
c  This routine examines the current pressure tendency to see if its ===
c  gradients have converged to within the requested tolerences.      ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c     NCMPCT   Number of active pressure points.          (integer)  ===
c     PCMPCT   Surface pressure change, compact storage.  (dble prec)===
c     ATOL     Convergence absolute tolerence.            (dble prec)===
c     RTOL     Convergence relative tolerence.            (dble prec)===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
#ifdef coast
c  /CSTSEG/                                                          ===
c                                                                    ===
c     IEP   Ending indices for surface pressure.     (integer array) ===
c     IEQ   Ending indices for velocity points.      (integer array) ===
c     ISP   Starting indices for surface pressure.   (integer array) ===
c     ISQ   Starting indices for velocity points.    (integer array) ===
c                                                                    ===
#endif
c  /GRDDAT/                                                          ===
c                                                                    ===
c     VMETX   x-metric coefficients; v-grid.   (real array; cm)      ===
c     VMETY   y-metric coefficients; v-grid.   (real array; cm)      ===
c                                                                    ===
c  /MODDAT/                                                          ===
c                                                                    ===
c     COORD   flag for grid coordinate type:          (integer)      ===
c             [0] Cartesian.                                         ===
c             [1] unrotated spherical.                               ===
c             [2] rotated spherical.                                 ===
c     DX      mean grid spacing in the x-direction.   (real; cm|deg) ===
c     DY      mean grid spacing in the y-direction.   (real; cm|deg) ===
c                                                                    ===
c  /NDIMEN/                                                          ===
c                                                                    ===
c     IM   number of points in the x-direction.   (integer)          ===
c     JM   number of points in the y-direction.   (integer)          ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c     ISCONV   Flag to indicate convergence.               (logical) ===
c              [true]   gradients have converged.                    ===
c              [false]  gradients have not yet converged.            ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#ifdef coast
# include <cstseg.h>
#endif
#include <grddat.h>
#include <moddat.h>
#include <ndimen.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,iend,ip,ippi,ippij,ippj,istr,j,jp,knew,kold,n,ncmpct
#ifdef coast
      integer l
#endif
      logical first,isconv
      real
     &     deg2cm
      double precision 
     &      atol,dxx,dyy,rdxx,rdyy,rtol,testval
      double precision 
     &      ddmx2r(np),ddmy2r(np),dpsdt(np),
     &      d2psdxdt(np,0:1),d2psdydt(np,0:1),pcmpct(ncmpct)
c
      save first,knew,kold,ddmx2r,ddmy2r,d2psdxdt,d2psdydt
c
      parameter (deg2cm=deg2rad*re)
c
      data knew,kold /0,1/
      data first /.true./
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Transfer change in surface pressure from compact storage to 2D.
c-----------------------------------------------------------------------
c
      n    = 0
      istr = 2
      iend = im-1
c
      do 30 j = 2, jm-1
# ifdef coast
         do 20 l = 1, lseg
            jp = j + (l-1)*jm
            istr = isp(jp)
            if(istr.eq.0) go to 30
            iend = iep(jp)
# endif
            do 10 i = istr, iend
               n = n+1
               ip = i + (j-1)*im
               dpsdt(ip)=pcmpct(n)
  10        continue
# ifdef coast
  20     continue
# endif
  30  continue
c
c-----------------------------------------------------------------------
c  Examine change in gradients of surface pressure tendency.
c-----------------------------------------------------------------------
c
c  Rotate storage pointers.
c
      knew = 1 - knew
      kold = 1 - knew
      testval = c0
      istr = 2
      iend = im-2
c
c  Loop over all active velocity points, examining gradient.
c
      do 60 j = 2, jm-2
#ifdef coast
         do 50 l=1,lseg
            jp = j + (l-1)*jm
            istr=isq(jp)
            if(istr.eq.0) go to 60
            iend=ieq(jp)
#endif
            do 40 i = istr, iend
c
               ip    = i + (j-1)*im
               ippi  = ip + 1
               ippj  = ip + im
               ippij  = ippj + 1
c
               d2psdxdt(ip,knew) = ((dpsdt(ippij)-dpsdt(ippj)) +
     &                              (dpsdt(ippi)-dpsdt(ip)))*ddmx2r(ip)
c
               d2psdydt(ip,knew) = ((dpsdt(ippij)-dpsdt(ippi)) +
     &                              (dpsdt(ippj)-dpsdt(ip)))*ddmy2r(ip)
c
               testval = max(testval,
     &                       (abs(d2psdxdt(ip,knew)-d2psdxdt(ip,kold))
     &                        -(rtol*abs(d2psdxdt(ip,knew))+atol)),
     &                       (abs(d2psdydt(ip,knew)-d2psdydt(ip,kold))
     &                        -(rtol*abs(d2psdydt(ip,knew))+atol)) )
c
  40        continue
#ifdef coast
  50     continue
# endif
  60  continue
c
      isconv = testval .le. c0
c
      return
c
c-----------------------------------------------------------------------
c  Entry to initialize rolling storage.
c-----------------------------------------------------------------------
c
      entry gcinit
c
      do 80 n = 0, 1
         do 70 ip = 1, im*jm
            d2psdxdt(ip,n) = c0
            d2psdydt(ip,n) = c0
  70     continue
  80  continue
c
      if (first) then
         if(coord.eq.0)then
            dxx=dble(dx)
            dyy=dble(dy)
         else
            dxx=dble(dx*deg2cm)
            dyy=dble(dy*deg2cm)
         endif
         rdxx = 0.5d0/dxx
         rdyy = 0.5d0/dyy
         do 90 ip = 1, im*jm
            ddmx2r(ip) = rdxx/dble(vmetx(ip))
            ddmy2r(ip) = rdxx/dble(vmety(ip))
  90     continue
         first = .false.
      end if
c
      return
      end
