      subroutine timediff
c
c=======================================================================
c                                                                    ===
c  This routine constructs time differences of barotropic vorticity  ===
c  or divergence fields already save in the output netCDF file.      ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /IOUNITS/                                                         ===
c                                                                    ===
c     STDOUT    Standard output unit.                 (integer)      ===
c                                                                    ===
c  /NDIMEN/                                                          ===
c                                                                    ===
c     IM   Number of points in the x-direction.       (integer)      ===
c     JM   Number of points in the y-direction.       (integer)      ===
c                                                                    ===
c  /PI_NETCDF/                                                       ===
c                                                                    ===
c     NCOUTID   NetCDF ID for output file.            (integer)      ===
c     TIMEDIM   Observation time dimension.           (integer)      ===
c                                                                    ===
c  /SWITCHES/                                                        ===
c                                                                    ===
c     PTYPE   Flag to select PE external state variables.   (integer)===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  HOPS:    DT2ND,   EXITUS,  GET_QBRY,  PUT_QBRY,  ROLLPTR          ===
c  NetCDF:  NCDINQ,  NCSNC                                           ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <param.h>
#include <iounits.h>
#include <ndimen.h>
#include <pi_netcdf.h>
#include <switches.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer mxby
      real    c0
      parameter (mxby=mx*36, c0=0.0)
c
      integer n,n1,notime
      integer ibdy(4),ptr(3)
      real    dtbry(mx,4,3),qbry(mx,4,3,0:2),times(0:2)
#ifdef surfpress
     &     ,pxbry(mx,4,3,0:2),pybry(mx,4,3,0:2)
c
      equivalence (qbry,pxbry)
#endif
c
      data ptr /0,1,2/
      data qbry /mxby*c0/
#ifdef surfpress
      data pybry /mxby*c0/
#endif
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Initialize boundary counters.
c-----------------------------------------------------------------------
c
      ibdy(west)  = jm
      ibdy(south) = im
      ibdy(east)  = jm
      ibdy(north) = im
c
c-----------------------------------------------------------------------
c  Synchronize NetCDF data to disk.
c-----------------------------------------------------------------------
c
      call ncsnc (ncoutid,rcode)
c
      if (rcode.ne.0) then
         write(stdout,900)
         call exitus ('TIMEDIFF')
      endif
c
c-----------------------------------------------------------------------
c  Determine temporal size of data.
c-----------------------------------------------------------------------
c
      call ncdinq (ncoutid,timedim,dimnam,notime,rcode)
c
      if (rcode.ne.0) then
         write(stdout,910) 'read dimension','time'
         call exitus ('TIMEDIFF')
      endif
c
c-----------------------------------------------------------------------
c  If only 1 temporal realization, set time derivative to zero.
c-----------------------------------------------------------------------
c
      if (notime.lt.2) then
#ifndef surfpress
         call put_qbry (qbry(1,1,1,0),1,ibdy,0)
#else
         if (ptype.gt.0) then
            call put_qbry (pxbry(1,1,1,0),1,ibdy,1)
            call put_qbry (pybry(1,1,1,0),1,ibdy,2)
          else
            call put_qbry (qbry(1,1,1,0),1,ibdy,0)
         endif
#endif
         return
      endif
c
c-----------------------------------------------------------------------
c  Initialize rolling data.
c-----------------------------------------------------------------------
c
      do 10 n = 1, 2
         n1 = n + 1
#ifndef surfpress
         call get_qbry (times(ptr(n1)),qbry(1,1,1,ptr(n1)),n,ibdy,0)
#else
         if (ptype.gt.0) then
            call get_qbry (times(ptr(n1)),pxbry(1,1,1,ptr(n1)),n,ibdy,1)
            call get_qbry (times(ptr(n1)),pybry(1,1,1,ptr(n1)),n,ibdy,2)
          else
            call get_qbry (times(ptr(n1)),qbry(1,1,1,ptr(n1)),n,ibdy,0)
         endif
#endif
  10  continue
c
c-----------------------------------------------------------------------
c  Loop over remaining temporal realizations, taking second order time
c  derivatives.
c-----------------------------------------------------------------------
c
c  Construct initial centered difference, assuming persistence beyond data.
c
#ifndef surfpress
      call dt2nd ('Persist',ptr,ibdy,times,qbry,dtbry)
      call put_qbry (dtbry,1,ibdy,0)
#else
      if (ptype.gt.0) then
         call dt2nd ('Persist',ptr,ibdy,times,pxbry,dtbry)
         call put_qbry (dtbry,1,ibdy,1)
         call dt2nd ('Persist',ptr,ibdy,times,pybry,dtbry)
         call put_qbry (dtbry,1,ibdy,2)
       else
         call dt2nd ('Persist',ptr,ibdy,times,qbry,dtbry)
         call put_qbry (dtbry,1,ibdy,0)
      endif
#endif
c
c  Loop over remaining times, reading new data and constructing
c  time derivatives.
c
      do 20 n = 3, notime
c
c  Read new time.
c
         call rollptr (3,ptr)
#ifndef surfpress
         call get_qbry (times(ptr(3)),qbry(1,1,1,ptr(3)),n,ibdy,0)
#else
         if (ptype.gt.0) then
            call get_qbry(times(ptr(3)),pxbry(1,1,1,ptr(3)),n,ibdy,1)
            call get_qbry(times(ptr(3)),pybry(1,1,1,ptr(3)),n,ibdy,2)
          else
            call get_qbry (times(ptr(3)),qbry(1,1,1,ptr(3)),n,ibdy,0)
         endif
#endif
c
c  Construct second order centered difference.
c
         n1 = n - 1
#ifndef surfpress
         call dt2nd ('Center',ptr,ibdy,times,qbry,dtbry)
         call put_qbry (dtbry,n1,ibdy,0)
#else
         if (ptype.gt.0) then
            call dt2nd ('Center',ptr,ibdy,times,pxbry,dtbry)
            call put_qbry (dtbry,n1,ibdy,1)
            call dt2nd ('Center',ptr,ibdy,times,pybry,dtbry)
            call put_qbry (dtbry,n1,ibdy,2)
          else
            call dt2nd ('Center',ptr,ibdy,times,qbry,dtbry)
            call put_qbry (dtbry,n1,ibdy,0)
         endif
#endif
  20  continue
c
c  Construct final centered difference, assuming persistence beyond data.
c
#ifndef surfpress
      call dt2nd ('Persist',ptr,ibdy,times,qbry,dtbry)
      call put_qbry (dtbry,notime,ibdy,0)
#else
      if (ptype.gt.0) then
         call dt2nd ('Persist',ptr,ibdy,times,pxbry,dtbry)
         call put_qbry (dtbry,notime,ibdy,1)
         call dt2nd ('Persist',ptr,ibdy,times,pybry,dtbry)
         call put_qbry (dtbry,notime,ibdy,2)
       else
         call dt2nd ('Persist',ptr,ibdy,times,qbry,dtbry)
         call put_qbry (dtbry,notime,ibdy,0)
      endif
#endif
c
      return
c
 900  format (/'***Error:  TIMEDIFF - unable to synchronize data to ',
     &        'disk.')
 910  format (/'***Error:  TIMEDIFF - unable to ',a,1x,1h",a,1h")
c
      end
