function masknew = clmsk_itr (landt);
%
% function masknew = clmsk_itr (landt);
%
% This function cleans the land mask:
%
%    (1)  Open Sea points which touch land
%         are changed to land points.
%
%    (2)  Boundary Sea points which don't
%         touch Open Sea are changed to land
%         points.
%
% ------
% Input:
% ------
%
%    LANDT.....Land mask at tracer points.
%
% -------
% Output:
% -------
%
%    MASKNEW...Corrected land mask at tracer points.

%-------------------------------------------------------------------------------
%  Get Land/Sea/Boundary mask.
%-------------------------------------------------------------------------------

[ny nx] = size (landt);

[landv,ismask] = v_mask (landt);
ismask = 2 - ismask;

clear landv;

%-------------------------------------------------------------------------------
%  Clear open sea points which touch land.
%-------------------------------------------------------------------------------

masknew  = landt;

%----------------------------
%--- Find open sea points ---
%----------------------------

[jsea isea] = find (ismask==2);

if ~isempty(jsea)

   neighbor = ones(size(jsea));

   %----------------------------------
   %--- Examine northern neighbor. ---
   %----------------------------------

   ind = find (jsea~=ny);

   if ~isempty(ind),
      ind2 = jsea(ind)+1 + (isea(ind)-1).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine northeastern neighbor. ---
   %--------------------------------------

   ind = find ((jsea~=ny)&(isea~=nx));

   if ~isempty(ind),
      ind2 = jsea(ind)+1 + isea(ind).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %---------------------------------
   %--- Examine eastern neighbor. ---
   %---------------------------------

   ind = find (isea~=nx);

   if ~isempty(ind),
      ind2 = jsea(ind) + isea(ind).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine southeastern neighbor. ---
   %--------------------------------------

   ind = find ((jsea~=1)&(isea~=nx));

   if ~isempty(ind),
      ind2 = jsea(ind)-1 + isea(ind).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %----------------------------------
   %--- Examine southern neighbor. ---
   %----------------------------------

   ind = find (jsea~=1);

   if ~isempty(ind),
      ind2 = jsea(ind)-1 + (isea(ind)-1).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine southwestern neighbor. ---
   %--------------------------------------

   ind = find ((jsea~=1)&(isea~=1));

   if ~isempty(ind),
      ind2 = jsea(ind)-1 + (isea(ind)-2).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %---------------------------------
   %--- Examine western neighbor. ---
   %---------------------------------

   ind = find (isea~=1);

   if ~isempty(ind),
      ind2 = jsea(ind) + (isea(ind)-2).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine northwestern neighbor. ---
   %--------------------------------------

   ind = find ((jsea~=ny)&(isea~=1));

   if ~isempty(ind),
      ind2 = jsea(ind)+1 + (isea(ind)-2).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %---------------------------------------------------
   %--- Eliminate open sea points which touch land. ---
   %---------------------------------------------------

   ind = find (neighbor==0);

   if ~isempty(ind),
      ind2 = jsea(ind) + (isea(ind)-1).*ny;
      masknew(ind2) = zeros(size(ind2));
   end;

end;

clear isea jsea;

%-------------------------------------------------------------------------------
%  Get revised Land/Sea/Boundary mask.
%-------------------------------------------------------------------------------

[landv,ismask] = v_mask (masknew);

clear landv;

%-------------------------------------------------------------------------------
%  Clear boundary sea points which do not touch open sea.
%-------------------------------------------------------------------------------

%--------------------------------
%--- Find boundary sea points ---
%--------------------------------

[jbnd ibnd] = find (ismask==1);

if ~isempty(jbnd)

   neighbor = ones(size(jbnd));

   %----------------------------------
   %--- Examine northern neighbor. ---
   %----------------------------------

   ind = find (jbnd~=ny);

   if ~isempty(ind),
      ind2 = jbnd(ind)+1 + (ibnd(ind)-1).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine northeastern neighbor. ---
   %--------------------------------------

   ind = find ((jbnd~=ny)&(ibnd~=nx));

   if ~isempty(ind),
      ind2 = jbnd(ind)+1 + ibnd(ind).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %---------------------------------
   %--- Examine eastern neighbor. ---
   %---------------------------------

   ind = find (ibnd~=nx);

   if ~isempty(ind),
      ind2 = jbnd(ind) + ibnd(ind).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine southeastern neighbor. ---
   %--------------------------------------

   ind = find ((jbnd~=1)&(ibnd~=nx));

   if ~isempty(ind),
      ind2 = jbnd(ind)-1 + ibnd(ind).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %----------------------------------
   %--- Examine southern neighbor. ---
   %----------------------------------

   ind = find (jbnd~=1);

   if ~isempty(ind),
      ind2 = jbnd(ind)-1 + (ibnd(ind)-1).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine southwestern neighbor. ---
   %--------------------------------------

   ind = find ((jbnd~=1)&(ibnd~=1));

   if ~isempty(ind),
      ind2 = jbnd(ind)-1 + (ibnd(ind)-2).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %---------------------------------
   %--- Examine western neighbor. ---
   %---------------------------------

   ind = find (ibnd~=1);

   if ~isempty(ind),
      ind2 = jbnd(ind) + (ibnd(ind)-2).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %--------------------------------------
   %--- Examine northwestern neighbor. ---
   %--------------------------------------

   ind = find ((jbnd~=ny)&(ibnd~=1));

   if ~isempty(ind),
      ind2 = jbnd(ind)+1 + (ibnd(ind)-2).*ny;
      neighbor(ind) = neighbor(ind).*ismask(ind2);
   end;

   %------------------------------------------------------------------
   %--- Eliminate boundary sea points which do NOT touch open sea. ---
   %------------------------------------------------------------------

   ind = find (neighbor~=0);

   if ~isempty(ind),
      ind2 = jbnd(ind) + (ibnd(ind)-1).*ny;
      masknew(ind2) = zeros(size(ind2));
   end;

end;

clear ibnd ind ind2 ismask jbnd nx ny;
