function [icst,jcst,inew,jnew] = coastline(ismask,i,j,xb,yb)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                      %
% function [icst,jcst,inew,jnew] = coastline(ismask,i,j,xb,yb)         %
%                                                                      %
% Given the Land/boundary/sea mask data and scalars and the starting   %
% positions  (XB,YB)  of the coastal segment, this function extracts   %
% the coastal segment contained in the boundary points in such a way   %
% that water is always to the left when looking from the position of   %
% the first point towards the second point of the coatal segment.      %
%                                                                      %
% On Input:                                                            %
%                                                                      %
%    ISMASK     Land/Boundary/Sea mask (integer matrix):               %
%               ISMASK=0 -> sea point.                                 %
%               ISMASK=1 -> boundary point.                            %
%               ISMASK=2 -> land point.                                %
%    I          I-coordinates of boundary points.                      %
%    J          J-coordinates of boundary points.                      %
%    XB         X-coordinate of starting point (grid index).           %
%    YB         Y-coordinate of starting point (grid index).           %
%                                                                      %
% On Output:                                                           %
%                                                                      %
%    ICST       I-coordinates of coastline (integer vector).           %
%    JCST       J-coordinates of coastline (integer vector).           %
%    INEW       Updated I-coordinates of boundary points.              %
%    JNEW       Updated J-coordinates of boundary points.              %
%                                                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----------------------------------------------------------------------
%  Find begining of the string.
%-----------------------------------------------------------------------

place=find(i==xb & j==yb);
ind=0;

%-----------------------------------------------------------------------
%  If starting point is in boundary nodes,
%-----------------------------------------------------------------------

if ~isempty(place),

%-----------------------------------------------------------------------
% Set  interior wet points.
%-----------------------------------------------------------------------

   [jw iw]=find(ismask==0);

%-----------------------------------------------------------------------
%  Add point to string, remove point from list of boundary nodes.
%-----------------------------------------------------------------------

   ind=ind+1;
   icst(ind)=i(place);
   jcst(ind)=j(place);
   i(place)=[];
   j(place)=[];

%-----------------------------------------------------------------------
%  Setup neighborhood.
%-----------------------------------------------------------------------

   star=[1 0; 0 1; -1 0; 0 -1];
   goon=1;

%-----------------------------------------------------------------------
%  Loop while still finding coastal points.
%-----------------------------------------------------------------------

   while goon==1,

      x0=icst(ind);
      y0=jcst(ind);
      k=1;
      done=0;

%-----------------------------------------------------------------------
%  Loop until next coastal point is found or possibilities exhausted.
%-----------------------------------------------------------------------

      while k<5 & done==0,

         x=x0+star(k,1);
         y=y0+star(k,2);
         place=find(i==x & j==y);

         if ~isempty(place),

%  [X Y] belongs to the set: IDMAT==1 minus the points already included
%  in the  segment. Examine the head of the segment.

             yw=y0+(x-x0);
             xw=x0-(y-y0);
             wet=find(iw==xw & jw== yw );

             if isempty(wet),

%  The head does not have interior wet to the left examine the tail of
%  the segment

                yw=y+(x-x0);
                xw=x-(y-y0);
                wet=find(iw==xw & jw==yw);
             end,

             if ~isempty(wet),

%  Add point to the segment.

                done = 1;
                ind=ind+1;
                icst(ind)=i(place);
                jcst(ind)=j(place);
                i(place)=[];
                j(place)=[];

            else

%  This interval is not connected to the interior wet points
%  go the next neigbour.

                k=k+1;
            end,

         else

%  The neighbour does not belong to the set: IDMAT==1 minus the points
%  already included in the segment.  Go the next neigbour.

            k=k+1;
         end,
      end,
      if done==0, goon=0;end,
    end,
 else
   icst=[];
   jcst=[];
end,

inew = i;
jnew = j;

return

