%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Matlab script to process PE model Land/Sea Mask Data.                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-------------------------------------------------------------------------
%  Set default values:
%
%  ZMSK_DEF    Default transition isobath between land and sea points.
%  MCSEG_DEF   Default maximum number of coastal segments.
%  MPSEG_DEF   Default maximum number of points per coastal segment.
%  MISLE_DEF   Default maximum number of islands.
%
%-------------------------------------------------------------------------

zmsk_def=0;
mcseg_def=20;
mpseg_def=2000;
misle_def=20;

%-------------------------------------------------------------------------
%  Get GRIDS NetCDF file name.
%-------------------------------------------------------------------------

[filename,pathname] = uigetfile ({'*.nc'; '*.cdf'},'GRIDS NetCDF file');

if (isstr(filename) & isstr(pathname))
   grdname = fullfile (pathname,filename);
elseif isstr(filename)
   grdname = filename;
else
   disp (' ');
   disp ('***Error:  PE_MASK - unspecified GRIDS NetCDF file');
   disp (' ');
   return;
end;

%-------------------------------------------------------------------------
%  Get Map coastline file name.
%-------------------------------------------------------------------------

fspec = {'*.dat';'*.nc';'*.cdf'};

[filename,pathname] = uigetfile (fspec,'Map coastline file');

if (isstr(filename) & isstr(pathname))
   cstname = fullfile (pathname,filename);
elseif isstr(filename)
   cstname = filename;
else
   disp (' ');
   disp ('***Error:  PE_MASK - unspecified Map coastline file');
   disp (' ');
   return;
end;

clear filename fspec pathname

%-------------------------------------------------------------------------
%  Read in domain parameters, raw bathymetry, and Land/Sea mask (if any)
%  from GRIDS NetCDF file.
%-------------------------------------------------------------------------

[domdat,bath,mask,mcseg,mpseg,misle] = read_mask (grdname);

%  If LANDT is flagged out, inquire the value of the transition isobath
%  and then set the mask data.

maxmask=max(max(mask));
minmask=min(min(mask));
if ((minmask == 0 & maxmask == 0) | isempty(mask)),
   mask = init_mask (bath,zmsk_def);
end
clear maxmask minmask

%-------------------------------------------------------------------------
%  Inquire about maximal dimensions to reserve for undefined coastal
%  variables.
%-------------------------------------------------------------------------

if ((mcseg == 0) | isempty(mcseg)),
  mcseg = input (['Enter maximum number of coastal segments to define [', ...
                   num2str(mcseg_def),'] : ']);
  if (isempty(mcseg)), mcseg=mcseg_def; end
end

if ((mpseg == 0) | isempty(mpseg)),
  mpseg = input (['Enter maximum number of points per coastal segments ', ...
                  'to define [',num2str(mpseg_def),']: ']);
  if (isempty(mpseg)), mpseg=mpseg_def; end
end

if ((misle == 0) | isempty(misle)),
  misle = input (['Enter maximum number of islands to define [', ...
                  num2str(misle_def),'] : ']);
  if (isempty(misle)), misle=misle_def; end
end

%-------------------------------------------------------------------------
%  Process land/sea mask: create a new mask; mask/unmask points.
%-------------------------------------------------------------------------

[landt,landv,icoast,jcoast,lencoast,isis,ieis,jsis,jeis,abort] = ...
                        set_mask (mask,mcseg,mpseg,misle,cstname,domdat);

%-------------------------------------------------------------------------
%  Write out mask data into GRIDS NetCDF file.
%-------------------------------------------------------------------------

if (~abort),
   status = write_mask (grdname,landt,landv,icoast,jcoast,lencoast, ...
                        isis,ieis,jsis,jeis);
end;
