function plot_cst_xy (ncid,type,lineclr,domdat);
%
% function plot_cst_xy (ncid,type,lineclr,domdat);
%
% This function adds one type of coast line to the current plot.
%
% ------
% Input:
% ------
%
%    NCID      NetCDF identifier for opened coastline dataset.
%    TYPE      Type of coastline to add.
%               "Land"     coastlines separating land from ocean
%               "Lake"     coastlines defining land-locked lakes
%               "Island"   coastlines defining islands within lakes
%               "Pond"     coastlines defining ponds within islands within lakes
%    LINECLR   Color to use in drawing coastlines.
%    DOMDAT    Vector containing domain definition parameters.
%                 DOMDAT( 1) = COORD - grid type
%                 DOMDAT( 2) = LONC  - longitude trans. center
%                 DOMDAT( 3) = LATC  - latitude trans. center
%                 DOMDAT( 4) = THETA - grid rotation angle
%                 DOMDAT( 5) = IM    - number of x-grid points
%                 DOMDAT( 6) = JM    - number of y-grid points
%                 DOMDAT( 7) = DX    - x-grid spacing
%                 DOMDAT( 8) = DY    - y-grid spacing
%                 DOMDAT( 9) = DELX  - x-offset grid-trans. centers
%                 DOMDAT(10) = DELY  - y-offset grid-trans. centers

%-----------------------------------------------------------------------
% Extract domain data.
%-----------------------------------------------------------------------

coord = domdat( 1);
lonc  = domdat( 2);
latc  = domdat( 3);
theta = domdat( 4);
im    = domdat( 5);
jm    = domdat( 6);
dx    = domdat( 7);
dy    = domdat( 8);
delx  = domdat( 9);
dely  = domdat(10);

%-------------------------------------------------------------------------------
% Get current plot limits.
%-------------------------------------------------------------------------------

[lon,lat]   = xy2ll ([1 1 im im],[1 jm 1 jm], ...
                     coord,im,jm,dx,dy,lonc,latc,delx,dely,theta);

lonlim = [min(lon(:)) max(lon(:))];
latlim = [min(lat(:)) max(lat(:))];

%-------------------------------------------------------------------------------
% Look for bounding box.  If not found, assume the requested coastal type
% is not included in file.
%-------------------------------------------------------------------------------

box = ncid{[lower(type),'box']}(:);

if isempty(box)
   return;
end;

[ncst, nside] = size(box);

%-------------------------------------------------------------------------------
% Examine each box.  If it intersects the plot window, add the coastline.
%-------------------------------------------------------------------------------

nant = ncid.antarctica(:);

for n = 1:ncst

   if ~( (box(n,2)<lonlim(1)) | (box(n,1)>lonlim(2)) | ...
         (box(n,4)<latlim(1)) | (box(n,3)>latlim(2)) )
      deltas = ncid{[lower(type),'index']}(n,1:2);
      pos    = ncid{lower(type)}((deltas(1)+(1:deltas(2))),:);
      if (n==nant)
         [npt nvc] = size(pos);
         pos(npt+1,:) = [180 -90];
         pos(npt+1,:) = [-180 -90];
      end;
      [x,y]   = ll2xy (pos(:,1),pos(:,2), ...
                       coord,im,jm,dx,dy,lonc,latc,delx,dely,theta);
      h = plot (x,y);
      set (h,'Color',lineclr);
   end;

end;
