function [landt,landv,icoast,jcoast,lencoast,isis,ieis,jsis,jeis,abort] = ...
                       set_mask (mask,mcseg,mpseg,misle,cstname,domdat);
%
% function [landt,landv,icoast,jcoast,lencoast,isis,ieis,jsis,jeis,abort] = ...
%                         set_mask (mask,mcseg,mpseg,misle,cstname,domdat);
%
% This procedure mask/unmask Land/Sea points in the coastal mask file.
% The coastline data is overlayed to facilitate masking.  It zooms in
% the are specified by the User.
%
% ------
% Input:
% ------
%
%    MASK         Original Land/Sea mask (integer matrix):
%                    MASK=0 land, Mask=1 Sea.
%    MCSEG        Maximum number of coastal segments.
%    MPSEG        Maximum number of points per coastal segment.
%    MISLE        Maximum number of islands.
%    CSTNAME      Name of coastline data file.
%    DOMDAT       Vector containing domain definition parameters.
%                    DOMDAT( 1) = COORD - grid type
%                    DOMDAT( 2) = LONC  - longitude trans. center
%                    DOMDAT( 3) = LATC  - latitude trans. center
%                    DOMDAT( 4) = THETA - grid rotation angle
%                    DOMDAT( 5) = IM    - number of x-grid points
%                    DOMDAT( 6) = JM    - number of y-grid points
%                    DOMDAT( 7) = DX    - x-grid spacing
%                    DOMDAT( 8) = DY    - y-grid spacing
%                    DOMDAT( 9) = DELX  - x-offset grid-trans. centers
%                    DOMDAT(10) = DELY  - y-offset grid-trans. centers
%
% -------
% Output:
% -------
%
%    LANDT        modified Land/Sea mask (integer matrix).

%-------------------------------------------------------------------------------
%  Determine MATLAB version number.
%-------------------------------------------------------------------------------

matver = version;
matvnum = str2num(matver(1));
if (matvnum<5),
   bndclr = 'm';
   seaclr = 'c';
   cstclr = 'y';
   mrkclr = 'w';
 else
   bndclr = 'r';
   seaclr = 'b';
   cstclr = 'r';
   mrkclr = 'k';
end;

land_color  = 0.75.*[0.86 0.58 0.44];
%land_color  = [0.86 0.58 0.44];
bkgndcolor = [0.75 0.875 1];

%--------------------------------------------------------------------------
%  Keep user waiting until ready.
%--------------------------------------------------------------------------

disp(' ');
disp('---------------------------------------------------------');
disp('------------- Processing Land/Sea Mask Data -------------');
disp('---------------------------------------------------------');
disp(' ');
disp('Please wait a moment...');

%--------------------------------------------------------------------------
%  Copy the data into the working array.  Find data limits.
%  Differentiate between land and sea points.  Create plot.
%--------------------------------------------------------------------------

landt = mask;
[landv,ismask] = v_mask(landt);

[jmax,imax] = size(mask);
ilim = [1 imax];
jlim = [1 jmax];

[jland,iland] = find(landt==0);
[jbndy,ibndy] = find(ismask==1);
[jsea,isea]   = find((landt==1) & (ismask~=1));

if isempty(iland)
   iland = NaN;
   jland = NaN;
end;
if isempty(ibndy)
   ibndy = NaN;
   jbndy = NaN;
end;
if isempty(isea)
   isea = NaN;
   jsea = NaN;
end;

%--------------------------------------------------------------------------
%  Create initial plot.
%--------------------------------------------------------------------------

set (gca,'XLim',ilim,'YLim',jlim,'FontName','Times');
hold on;
status = add_cst_xy (cstname,land_color,bkgndcolor,domdat);

if (status~=0)
   hold off;clf;
   [clon,clat] = rcoastline (cstname);
   set (gca,'XLim',ilim,'YLim',jlim,'FontName','Times');
   hold on;
end

hnd_lbs = plot (iland,jland,'g*', ...
                ibndy,jbndy,[bndclr,'+'], ...
                isea,jsea,[seaclr,'.']); ...
if (matvnum>=5),
   set (gca,'FontSize',14);
end;
title ('Mouse:  LEFT-select   MIDDLE-zoom in   RIGHT-zoom out'); ...
if (status~=0)
   draw_cstp (clon,clat,domdat);
end;

if (matvnum>=5),
   set (gcf,'WindowButtonUpFcn','pemask_uifn (''Dummy'');');
end;

%--------------------------------------------------------------------------
%  Set control buttons and initialize states.
%--------------------------------------------------------------------------

pemask_uifn ('Initialize');
figure (gcf);
States = get_states;

%--------------------------------------------------------------------------
%  Write instructions.
%--------------------------------------------------------------------------

disp(' ');
disp('-----------------------------------------------------------------------');
disp(' ');
disp('The newly created figure displays the current land mask with the');
disp('following symbols:');
disp(' ');
disp('   *  Land points');
disp('   +  Boundary sea points');
disp('   .  Open sea points');
disp(' ');
disp('For modification purposes, Boundary & Open Sea points are lumped ')
disp('together as Sea.  Use the following guidelines in setting the mask:');
disp(' ');
disp('   -- Channels and bays should be at least 2 open sea points wide.');
disp(' ');
disp('   -- Boundary sea points must touch both land and open sea points.');
disp(' ');
disp('   -- Open sea points must touch only boundary sea and open sea points.');
disp(' ');
disp('-----------------------------------------------------------------------');
disp(' ');
disp('Carefully select mask points to change:');
disp(' ');
disp('   Use the LEFT mouse button to select a single point or');
disp('      two opposing corners of a rectangle for an area to change.');
disp('   Use the MIDDLE mouse button to select two opposing');
disp('      corners of an area for zooming.');
disp('   Use the RIGHT mouse button to unzoom.');
disp(' ');
disp('Use the cyan graph buttons to change from flipping to');
disp('      filling with land or sea.');
disp(' ');
disp('Use the yellow graph buttons to select between changing');
disp('      a point or an entire area.');
disp(' ');
disp('Use the purple graph button to cancel an area');
disp('      manipulation or zoom-in operation.');
disp(' ');
disp('When done, use the green graph button to stop.');
disp(' ');
disp('To abort: use the red graph botton.');
disp(' ');

%--------------------------------------------------------------------------
% Modify land mask according to user's instructions.
% Loop until the DONE button is pressed.
%--------------------------------------------------------------------------

[xpt,ypt,butt,States] = get_point ('UnCancelable');

while ~States(pemsk_ind('Stop')),

%   --------------------------------------------
%   --- Select action based on button value. ---
%   --------------------------------------------

   if (butt==1),

%      -------------------------------------------------------
%      --- Change point(s) based on current button values. ---
%      -------------------------------------------------------

      iwk  = max([min([round(xpt) imax]) 1]);
      jwk  = max([min([round(ypt) jmax]) 1]);
      htmp = [];

      if States(pemsk_ind('Point')),

%         ------------------------------
%         --- Change a single point. ---
%         ------------------------------

         if States(pemsk_ind('Land')),
            landt(jwk,iwk)=0;
          elseif States(pemsk_ind('Sea')),
            landt(jwk,iwk)=1;
          else
            landt(jwk,iwk)=1-landt(jwk,iwk);
         end,

       else

%         --------------------------------
%         --- Change rectangular area. ---
%         --------------------------------

         htmp = plot(iwk,jwk,[mrkclr,'x']);
         xlabel('AREA Change:  Select second corner'); ...
         set(get(gca,'XLabel'),'Color',seaclr);
         figure(gcf);
         [xpt,ypt,butt,States] = get_point ('Cancelable');

         if ( States(pemsk_ind('Area')) & ...
             ~States(pemsk_ind('Stop')) & ...
             ~States(pemsk_ind('Cancel'))),
            iwk2 = max([min([round(xpt) imax]) 1]);
            jwk2 = max([min([round(ypt) jmax]) 1]);
            iwnd = min([iwk iwk2]):max([iwk iwk2]);
            jwnd = min([jwk jwk2]):max([jwk jwk2]);
            if States(pemsk_ind('Land')),
               landt(jwnd,iwnd) = zeros(size(landt(jwnd,iwnd)));
             elseif States(pemsk_ind('Sea')),
               landt(jwnd,iwnd) = ones(size(landt(jwnd,iwnd)));
             else
               landt(jwnd,iwnd) = 1-landt(jwnd,iwnd);
            end,
           elseif States(pemsk_ind('Cancel')),
            States(pemsk_ind('Cancel')) = 0;
            put_states (States);
         end,

      end,

      [landv,ismask] = v_mask (landt);
      [jland,iland]  = find (landt==0);
      [jbndy,ibndy]  = find (ismask==1);
      [jsea,isea]    = find ((landt==1) & (ismask~=1));


     elseif (butt==2),

%      ------------------------
%      --- Choose new zoom. ---
%      ------------------------

      iwk = round(xpt);
      jwk = round(ypt);
      htmp = plot(iwk,jwk,[mrkclr,'o']);
      xlabel('ZOOM In:  Select second corner');
      set(get(gca,'XLabel'),'Color','y');
      figure(gcf);
      [xpt,ypt,butt,States] = get_point ('Cancelable');

      if (~States(pemsk_ind('Stop')) & ~States(pemsk_ind('Cancel'))),
         iwk2 = round(xpt);
         jwk2 = round(ypt);
         if ((iwk~=iwk2)&(jwk~=jwk2)),
            ilim = [min([iwk iwk2]) max([iwk iwk2])];
            jlim = [min([jwk jwk2]) max([jwk jwk2])];
           else,
            xlabel('Bad zoom window, try again.');
            set(get(gca,'XLabel'),'Color',[1.0 0.25 0.25]);
            figure(gcf);
            pause(1);
         end;
        elseif States(pemsk_ind('Cancel')),
         States(pemsk_ind('Cancel')) = 0;
         put_states (States);
      end;

     elseif (butt==3),

%      ----------------------------
%      --- Return to full plot. ---
%      ----------------------------

      ilim = [1 imax];
      jlim = [1 jmax];
      htmp = [];

   end;

%   -------------------------------------------
%   --- Redraw and wait for next selection. ---
%   -------------------------------------------

   if ~isempty(htmp)
      delete(htmp);
   end;
   set (hnd_lbs(1),'XData',iland,'YData',jland); ...
   set (hnd_lbs(2),'XData',ibndy,'YData',jbndy); ...
   set (hnd_lbs(3),'XData',isea,'YData',jsea);
   xlabel('');
   set (gca,'XLim',ilim,'YLim',jlim);

   figure(gcf);
   [xpt,ypt,butt,States] = get_point ('UnCancelable');

end;

%--------------------------------------------------------------------------
%  Save abort flag, clear workspace.
%--------------------------------------------------------------------------

abort  = States(pemsk_ind('Abort'));

clear butt i iland ilim imax isea j jland jlim jmax jsea
clear States xpt ypt

%-------------------------------------------------------------------------
%  Set Land/Sea mask at the velocity points: LANDU, and ISMASK.
%-------------------------------------------------------------------------

if (~abort),
   maskold = landt;
   landt = clean_mask (maskold,'Partial');
   clear maskold;
   [landv,ismask] = v_mask (landt);
end;

%-------------------------------------------------------------------------
%  Extract coastal segments and the corners of the islands box, if any.
%-------------------------------------------------------------------------

if (~abort),
   [ibrywk,jbrywk,ncseg,icoast,jcoast,lencoast] = ...
                                   xtr_seg (ismask,mcseg,mpseg,seaclr,cstclr);
end;

if (~abort),
   icst = icoast;
   jcst = jcoast;
   lncst = lencoast;
   [icoast,jcoast,lencoast,isis,ieis,jsis,jeis] = ...
                              xtr_box (misle,ibrywk,jbrywk,ncseg,icst,jcst, ...
                                       lncst,ismask,landt,seaclr,cstclr);
end;

if (~abort),

   ncseg = max(find(lencoast>0));
   if (isempty(ncseg))
      ncseg = 0;
   end,

   nisle = max(find(isis>0));
   if (isempty(nisle)),
      nisle = 0;
   end,

   disp (' ');
   disp ('  ------------------------------');
   disp ('  Coastal Segment(s) Extraction:');
   disp ('  ------------------------------');
   disp (' ');
   disp (['    Found ',num2str(ncseg),' coastlines:']);
   disp (['       ',num2str(ncseg-nisle),' exterior coasts']);
   disp (['       ',num2str(nisle),' islands']);

   ist = num2str(icoast(1:ncseg,1));
   jst = num2str(jcoast(1:ncseg,1));

   cstid = num2str((1:ncseg)');
   stloc = [setstr(ones(ncseg,1)*abs('(')) ist  ...
            setstr(ones(ncseg,1)*abs(', '))     ...
            jst setstr(ones(ncseg,1)*abs(')'))];
   typlb = [setstr(ones((ncseg-nisle),1)*abs('(exterior)')); ...
            setstr(ones(nisle,1)*abs(' (island) '))];

   [nln idln] = size(cstid);
   [nln stln] = size(stloc);
   [nln tpln] = size(typlb);

   csthd = 'coast';
   sthd  = 'starting point';
   typhd = 'type';

   idhln = length(csthd);
   sthln = length(sthd);
   tphln = length(typhd);

   idiff = idln-idhln;
   sdiff = stln-sthln;
   tdiff = tpln-tphln;

   disp (' ');
   disp ('    Starting locations:');
   disp (' ');

   disp ([setstr(ones(1,6+max([0 fix(0.5*idiff)])).*abs(' ')) csthd ...
          setstr(ones(1,5+max([0 (idiff-fix(0.5*idiff))])).*abs(' ')) ...
          setstr(ones(1,max([0 fix(0.5*sdiff)])).*abs(' ')) sthd ...
            setstr(ones(1,5+max([0 (sdiff-fix(0.5*sdiff))])).*abs(' ')) ...
          setstr(ones(1,max([0 fix(0.5*tdiff)])).*abs(' ')) typhd ...
            setstr(ones(1,5+max([0 (tdiff-fix(0.5*tdiff))])).*abs(' '))]);

   disp ([setstr(ones(nln,6+max([0 fix(0.5*(-idiff))])).*abs(' ')) cstid ...
          setstr(ones(nln,5+max([0 ((-idiff)-fix(0.5*(-idiff)))])) ...
                                                             .*abs(' ')) ...
          setstr(ones(nln,max([0 fix(0.5*(-sdiff))])).*abs(' ')) stloc ...
            setstr(ones(nln,5+max([0 ((-sdiff)-fix(0.5*(-sdiff)))])) ...
                                                              .*abs(' ')) ...
          setstr(ones(nln,max([0 fix(0.5*(-tdiff))])).*abs(' ')) typlb ...
            setstr(ones(nln,5+max([0 ((-tdiff)-fix(0.5*(-tdiff)))])) ...
                                                            .*abs(' '))]);

end;
