function [landv,ismask]=v_mask(landt)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% function [landv,ismask]=v_mask(landt)                                   %
%                                                                         %
% This function computes the Land/Sea mask at velocity points form        %
% Land/Sea mask at tracer points.                                         %
%                                                                         %
% On Input:                                                               %
%                                                                         %
%    LANDT        Land/Sea mask tracer points (integer matrix):           %
%                 LANDT=0 land, LANDT=1 Sea.                              %
%                                                                         %
% On Output:                                                              %
%                                                                         %
%    LANDV        Land/Sea mask at velocity points (integer matrix):      %
%                 LANDV=0 land, LANDV=1 boundary, LANDV=2 sea.            %
%    ISMASK       Land/Sea/Boundary mask used in coastal segment          %
%                 extraction (integer matrix):                            %
%                 ISMASK=0 land, ISMASK=1 boundary, ISMASK=2 sea.         %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%--------------------------------------------------------------------------
% Determine size of matrices.
%--------------------------------------------------------------------------

[jm im]=size(landt);

%--------------------------------------------------------------------------
% Compute mask at velocity points which is used to compute ISMASK.
%--------------------------------------------------------------------------

landv=ones([jm im]);

indx=1:im-1;
indxp1=indx+1;

for j=1:jm-1,
   neighbor(:,indx)=[landt(j,indx)' landt(j+1,indx)' landt(j,indxp1)' ...
                     landt(j+1,indxp1)']';
   landv(j,indx)=min(neighbor);
end,

%  Set mask indices at the outer boundaries.

landv(jm,:)=landv(jm-1,:);
landv(:,im)=landv(:,im-1);

%--------------------------------------------------------------------------
%  Compute Land/Sea/Boundary mask used in the coastal segment extraction.
%--------------------------------------------------------------------------

ismask=zeros([jm im]);

jndx=1:jm;
jndxm1=[1 1:jm-1];
indx=1:im;
indxm1=[1 1:im-1];

isea=landv(jndx,indx).*landv(jndxm1,indx).*landv(jndx,indxm1).* ...
     landv(jndxm1,indxm1);
tstnd=find(isea==0);
ismask(tstnd)=ones(size(tstnd));

isea=landv(jndx,indx)+landv(jndxm1,indx)+landv(jndx,indxm1)+ ...
     landv(jndxm1,indxm1);
tstnd=find(isea==0);
ismask(tstnd)=2.*ones(size(tstnd));

%--------------------------------------------------------------------------
%  Compute Land/Sea mask at velocity points.
%--------------------------------------------------------------------------

landv=zeros([jm im]);

jndx=1:jm;
jndxp1=[2:jm jm];
indx=1:im;
indxp1=[2:im im];

isea=landt(jndx,indx).*landt(jndxp1,indx).*landt(jndx,indxp1).* ...
     landt(jndxp1,indxp1);
tstnd=find(isea==0);
landv(tstnd)=ones(size(tstnd));

isea=landt(jndx,indx)+landt(jndxp1,indx)+landt(jndx,indxp1)+ ...
     landt(jndxp1,indxp1);
tstnd=find(isea==0);
landv(tstnd)=2.*ones(size(tstnd));

%  Set mask indices at the outer boundaries.

landv(:,im)=landv(:,im-1);
landv(jm,:)=landv(jm-1,:);

%  Reset indices.

landv=2-landv;

return
