function status=write_mask(gname,landt,landv,icoast,jcoast,lencoast,...
                           isis,ieis,jsis,jeis)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                          %
% function status=write_mask(gname,landt,landv,icoast,jcoast,lencoast,     %
%                            isis,ieis,jsis,jeis)                          %
%                                                                          %
% This routine writes out mask data into GRIDS NetCDF file.                %
%                                                                          %
% On Input:                                                                %
%                                                                          %
%    GNAME      GRIDS NetCDF file name (character string).                 %
%    LANDT      Land/Sea mask at the tracer points (integer matrix).       %
%    LANDV      Land/Sea mask at the velocity points (integer maxtrix).    %
%    ICOAST     coastal segments x-index coordinate (integer matrix).      %
%    JCOAST     coastal segments y-index coordinate (integer matrix).      %
%    LENCOAST   length of coastal segments (integer vector).               %
%    ISIS       starting x-index of islands box (integer vector).          %
%    IEIS       ending x-index of islands box (integer vector).            %
%    JSIS       starting y-index of islands box (integer vector).          %
%    JEIS       ending y-index of islands box (integer vector).            %
%                                                                          %
% Calls:   MEXCDF (Interface to NetCDF library using Matlab).              %
%          DATE_STAMP                                                      %
%          PE_SEASEG                                                       %
%                                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-------------------------------------------------------------------------------
%  Set history attribute.
%-------------------------------------------------------------------------------

pemsk_ver = ['Land/Sea Mask - PE_MASK (Matlab), Version 4.1 ', date_stamp];

%-------------------------------------------------------------------------------
%  Open GRIDS NetCDF file.
%-------------------------------------------------------------------------------

nc_write = mexcdf ('parameter','nc_write');
ncfid    = mexcdf ('open',gname,nc_write);
if (ncfid == -1),
  error(['WRITE_MASK: ncopen - unable to open file: ', gname])
  return
end

%-------------------------------------------------------------------------------
%  Define NetCDF parameters.
%-------------------------------------------------------------------------------

ncverbose = mexcdf ('parameter','nc_verbose');
ncglobal  = mexcdf ('parameter','nc_global');
nclong    = mexcdf ('parameter','nc_long');
ncchar    = mexcdf ('parameter','nc_char');

%-------------------------------------------------------------------------------
%  Supress error messages from NetCDF.
%-------------------------------------------------------------------------------

ncopts = mexcdf ('setopts',0);

%-------------------------------------------------------------------------------
%  Inquire dimension IDs.  A value of -1 is returned if the dimension is
%  not defined.
%-------------------------------------------------------------------------------

tlondim  = mexcdf ('dimid',ncfid,'tlon');
tlatdim  = mexcdf ('dimid',ncfid,'tlat');
vlondim  = mexcdf ('dimid',ncfid,'vlon');
vlatdim  = mexcdf ('dimid',ncfid,'vlat');
ncsegdim = mexcdf ('dimid',ncfid,'ncseg');
lcsegdim = mexcdf ('dimid',ncfid,'lcseg');
isledim  = mexcdf ('dimid',ncfid,'island');

%-------------------------------------------------------------------------------
%  Set local parameters.
%-------------------------------------------------------------------------------

[jm im] = size(landt);
lcseg = max(lencoast); if (isempty(lcseg)), lcseg=0; end,
ncseg = max(find(lencoast>0)); if (isempty(ncseg)), ncseg=0; end,
nisle = max(find(isis>0)); if (isempty(nisle)), nisle=0; end,
[mcseg mpseg] = size(icoast);
[misle] = length (isis);

%===========================================================================
%  Define mask data into GRIDS NetCDF file.
%===========================================================================

status = mexcdf ('redef',ncfid);
if (status == -1),
  error(['WRITE_MASK: ncrefdef - unable to put into define mode.'])
  return
end

%---------------------------------------------------------------------------
%  Define dimensions.
%---------------------------------------------------------------------------

if (ncsegdim == -1),
  ncsegdim = mexcdf ('dimdef',ncfid,'ncseg',mcseg);
  if (ncsegdim == -1),
    error(['WRITE_MASK: ncdimdef - unable to define dimension: ncseg.'])
  end
end

if (lcsegdim == -1),
  lcsegdim = mexcdf ('dimdef',ncfid,'lcseg',mpseg);
  if (lcsegdim == -1),
    error(['WRITE_MASK: ncdimdef - unable to define dimension: lcseg.'])
  end
end

if (isledim == -1 & nisle > 0),
  isledim = mexcdf ('dimdef',ncfid,'island',misle);
  if (isledim == -1),
    error(['WRITE_MASK: ncdimdef - unable to define dimension: island.'])
  end
end

%---------------------------------------------------------------------------
%  Write new global attribute.
%---------------------------------------------------------------------------

lenstr = length (pemsk_ver);
status = mexcdf ('attput',ncfid,ncglobal,'mask',ncchar,lenstr,pemsk_ver);
if (status == -1),
  error(['WRITE_MASK: ncattput - unable to global attribure: history.'])
  return
end

%-----------------------------------------------------------------------------
%  Define mask data variables.
%-----------------------------------------------------------------------------

%------------------------------------------
%--- Define number of coastal segments. ---
%------------------------------------------

varname = 'ncseg';
ncsegid = mexcdf ('varid',ncfid,varname);
if (ncsegid == -1),
  vid = mexcdf ('vardef',ncfid,varname,nclong,0,0);
  ncsegid = vid;
  if (ncsegid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'number of coastal segments';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end

%---------------------------------------------------
%--- Define maximum length for coastal segments. ---
%---------------------------------------------------

varname = 'lcseg';
lcsegid = mexcdf ('varid',ncfid,varname);
if (lcsegid == -1),
  vid = mexcdf ('vardef',ncfid,varname,nclong,0,0);
  lcsegid = vid;
  if (lcsegid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'maximum length of coastal segments';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end

%----------------------------------------------
%--- Define number of islands in the basin. ---
%----------------------------------------------

if (isledim ~= -1),
  varname = 'nisle';
  nisleid = mexcdf ('varid',ncfid,varname);
  if (nisleid == -1),
    vid = mexcdf ('vardef',ncfid,varname,nclong,0,0);
    nisleid = vid;
    if (nisleid == -1),
      error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
    end
    attname = 'long_name';
    attval  = 'number of islands';
    attlen  = length(attval);
    atttyp  = ncchar;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
  end
end

%------------------------------------------
%--- Define length of coastal segments. ---
%------------------------------------------

varname  = 'lencoast';
lensegid = mexcdf ('varid',ncfid,varname);
if (lensegid == -1),
  vdims = ncsegdim;
  nvdims = length(vdims);
  vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
  lensegid = vid;
  if (lensegid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'length of coastal segments';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end

%---------------------------------------
%--- Define x-index coastal segment. ---
%---------------------------------------

varname = 'icoast';
csegxid = mexcdf ('varid',ncfid,varname);
if (csegxid == -1),
  vdims = [ncsegdim lcsegdim];
  nvdims = length(vdims);
  vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
  csegxid = vid;
  if (csegxid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'coastal segment x-index coordinate';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'valid_min';
  attval  = 1;
  attlen  = length(attval);
  atttyp  = nclong;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'valid_max';
  attval  = im;
  attlen  = length(attval);
  atttyp  = nclong;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = '_FillValue';
  attval  = 0;
  attlen  = length(attval);
  atttyp  = nclong;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end

%---------------------------------------
%--- Define y-index coastal segment. ---
%---------------------------------------

varname = 'jcoast';
csegyid = mexcdf ('varid',ncfid,varname);
if (csegyid == -1),
  vdims = [ncsegdim lcsegdim];
  nvdims = length(vdims);
  vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
  csegyid = vid;
  if (csegyid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'coastal segment y-index coordinate';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'valid_min';
  attval  = 1;
  attlen  = length(attval);
  atttyp  = nclong;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'valid_max';
  attval  = jm;
  attlen  = length(attval);
  atttyp  = nclong;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = '_FillValue';
  attval  = 0;
  attlen  = length(attval);
  atttyp  = nclong;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end

%--------------------------------------------------
%--- Define Land/Sea mask at the tracer points. ---
%--------------------------------------------------

varname = 'landt';
landtid = mexcdf ('varid',ncfid,varname);
if (landtid == -1),
  vdims = [tlatdim tlondim];
  nvdims = length(vdims);
  vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
  landtid = vid;
  if (landtid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'land/sea mask at tracer points';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'option_0';
  attval  = 'land';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'option_1';
  attval  = 'sea';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'field';
  attval  = 'Tmask, scalar';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'positions';
  attval  = 'tgrid2';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end
vid   = landtid;
attname = 'pe_seaseg';
attval  = pe_seaseg(landt);
attlen  = length(attval);
atttyp  = nclong;
status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
if (status == -1),
  error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
         setstr(34),varname,':',attname,setstr(34)])
end

%----------------------------------------------------
%--- Define Land/Sea mask at the velocity points. ---
%----------------------------------------------------

varname = 'landv';
landvid = mexcdf ('varid',ncfid,varname);
if (landvid == -1),
  vdims = [vlatdim vlondim];
  nvdims = length(vdims);
  vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
  landvid = vid;
  if (landvid == -1),
    error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
  end
  attname = 'long_name';
  attval  = 'land/sea mask at velocity points';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'option_0';
  attval  = 'land';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'option_1';
  attval  = 'boundary';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'option_2';
  attval  = 'sea';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'field';
  attval  = 'Vmask, scalar';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
  attname = 'positions';
  attval  = 'vgrid2';
  attlen  = length(attval);
  atttyp  = ncchar;
  status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
  if (status == -1),
    error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
           setstr(34),varname,':',attname,setstr(34)])
  end
end

%---------------------------------------
%--- Define corners of island boxes. ---
%---------------------------------------

if (isledim ~= -1),

  %------------------------------------------------------------
  %--- Define starting x-index coordinate for island boxes. ---
  %------------------------------------------------------------

  varname = 'isis';
  isisid  = mexcdf ('varid',ncfid,varname);
  if (isisid == -1),
    vdims = isledim;
    nvdims = length(vdims);
    vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
    isisid = vid;
    if (isisid == -1),
      error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
    end
    attname = 'long_name';
    attval  = 'starting x-index coordinate of island box';
    attlen  = length(attval);
    atttyp  = ncchar;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_min';
    attval  = 1;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_max';
    attval  = im;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
  end

  %----------------------------------------------------------
  %--- Define ending x-index coordinate for island boxes. ---
  %----------------------------------------------------------

  varname = 'ieis';
  ieisid  = mexcdf ('varid',ncfid,varname);
  if (ieisid == -1),
    vdims = isledim;
    nvdims = length(vdims);
    vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
    ieisid = vid;
    if (ieisid == -1),
      error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
    end
    attname = 'long_name';
    attval  = 'ending x-index coordinate of island box';
    attlen  = length(attval);
    atttyp  = ncchar;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_min';
    attval  = 1;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_max';
    attval  = im;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
  end

  %------------------------------------------------------------
  %--- Define starting y-index coordinate for island boxes. ---
  %------------------------------------------------------------

  varname = 'jsis';
  jsisid  = mexcdf ('varid',ncfid,varname);
  if (jsisid == -1),
    vdims = isledim;
    nvdims = length(vdims);
    vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
    jsisid = vid;
    if (jsisid == -1),
      error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
    end
    attname = 'long_name';
    attval  = 'starting y-index coordinate of island box';
    attlen  = length(attval);
    atttyp  = ncchar;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_min';
    attval  = 1;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_max';
    attval  = jm;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
  end

  %----------------------------------------------------------
  %--- Define ending y-index coordinate for island boxes. ---
  %----------------------------------------------------------

  varname = 'jeis';
  jeisid  = mexcdf ('varid',ncfid,varname);
  if (jeisid == -1),
    vdims = isledim;
    nvdims = length(vdims);
    vid = mexcdf ('vardef',ncfid,varname,nclong,nvdims,vdims);
    jeisid = vid;
    if (jeisid == -1),
      error(['WRITE_MASK: ncvardef - unable to define variable: ',varname])
    end
    attname = 'long_name';
    attval  = 'ending y-index coordinate of island box';
    attlen  = length(attval);
    atttyp  = ncchar;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_min';
    attval  = 1;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
    attname = 'valid_max';
    attval  = jm;
    attlen  = length(attval);
    atttyp  = nclong;
    status  = mexcdf ('attput',ncfid,vid,attname,atttyp,attlen,attval);
    if (status == -1),
      error(['WRITE_MASK: ncattput - unable to define attribute: ', ...
             setstr(34),varname,':',attname,setstr(34)])
    end
  end
end

%---------------------------------------------------------------------------
%  Leave definition mode.
%---------------------------------------------------------------------------

%----------------------------
%--- End definition mode. ---
%----------------------------

status = mexcdf ('endef',ncfid);
if (status == -1),
  error(['WRITE_MASK: ncendef - unable to leave definition mode.'])
end

%===========================================================================
%  Write out mask data into GRIDS NetCDF file.
%===========================================================================

%---------------------------------------------
%--- Write out number of coastal segments. ---
%---------------------------------------------

status = mexcdf ('varput1',ncfid,ncsegid,[0],ncseg);
if (status == -1),
  error(['WRITE_MASK: ncvarput1 - error while writing variable: ncseg.'])
end

%-----------------------------------------------------
%--- Write out maximum length of coastal segments. ---
%-----------------------------------------------------

status = mexcdf ('varput1',ncfid,lcsegid,[0],lcseg);
if (status == -1),
  error(['WRITE_MASK: ncvarput1 - error while writing variable: lcseg.'])
end

%------------------------------------
%--- Write out number of islands. ---
%------------------------------------

if (isledim ~= -1),
  status = mexcdf ('varput1',ncfid,nisleid,[0],nisle);
  if (status == -1),
    error(['WRITE_MASK: ncvarput1 - error while writing variable: nisle.'])
  end
end

%---------------------------------------------
%--- Write out length of coastal segments. ---
%---------------------------------------------

status = mexcdf ('varput',ncfid,lensegid,[0],[mcseg],lencoast);
if (status == -1),
  error(['WRITE_MASK: ncvarput - error while writing variable: lencoast.'])
end

%------------------------------------------------------
%--- Write out coastal segments x-index coordinate. ---
%------------------------------------------------------

status = mexcdf ('varput',ncfid,csegxid,[0 0],[mcseg mpseg],icoast');
if (status == -1),
  error(['WRITE_MASK: ncvarput - error while writing variable: icoast.'])
end

%------------------------------------------------------
%--- Write out coastal segments y-index coordinate. ---
%------------------------------------------------------

status = mexcdf ('varput',ncfid,csegyid,[0 0],[mcseg mpseg],jcoast');
if (status == -1),
  error(['WRITE_MASK: ncvarput - error while writing variable: jcoast.'])
end

%-------------------------------------------------------------
%--- Write out coastal Land/Sea mask at the tracer points. ---
%-------------------------------------------------------------

status = mexcdf ('varput',ncfid,landtid,[0 0],[jm im],landt');
if (status == -1),
  error(['WRITE_MASK: ncvarput - error while writing variable: landt.'])
end

%---------------------------------------------------------------
%--- Write out coastal Land/Sea mask at the velocity points. ---
%---------------------------------------------------------------

status = mexcdf ('varput',ncfid,landvid,[0 0],[jm im],landv');
if (status == -1),
  error(['WRITE_MASK: ncvarput - error while writing variable: landv.'])
end

%-------------------------------------------
%--- Write out corners of islands boxes. ---
%-------------------------------------------

if (isledim ~= -1),
  status = mexcdf ('varput',ncfid,isisid,[0],[misle],isis);
  if (status == -1),
    error(['WRITE_MASK: ncvarput - error while writing variable: isis.'])
  end
  status = mexcdf ('varput',ncfid,ieisid,[0],[misle],ieis);
  if (status == -1),
    error(['WRITE_MASK: ncvarput - error while writing variable: ieis.'])
  end
  status = mexcdf ('varput',ncfid,jsisid,[0],[misle],jsis);
  if (status == -1),
    error(['WRITE_MASK: ncvarput - error while writing variable: jsis.'])
  end
  status = mexcdf ('varput',ncfid,jeisid,[0],[misle],jeis);
  if (status == -1),
    error(['WRITE_MASK: ncvarput - error while writing variable: jeis.'])
  end
end

%---------------------------------------------------------------------------
%  Close NetCDF file.
%---------------------------------------------------------------------------

status = mexcdf ('close',ncfid);
if (status == -1),
  error(['WRITE_MASK: ncclose - unable to close NetCDF file: ', gname])
end

return
