function [icoast,jcoast,lencoast,isis,ieis,jsis,jeis] = ...
                              xtr_box (misle,ibrywk,jbrywk,ncseg,icst,jcst, ...
                                       lncst,ismask,landt,seaclr,cstclr);
%
% function [icoast,jcoast,lencoast,isis,ieis,jsis,jeis] = ...
%                             xtr_box (misle,ibrywk,jbrywk,ncseg,icst,jcst, ...
%                                      lncst,ismask,landt,seaclr,cstclr);
%
% This procedure extracts island boxes from ICOAST and JCOAST.
%
% ------
% Input:
% ------
%
%    MISLE        maximum number of islands.
%    IBRYWK       list of unused boundary x-points (islands).
%    JBRYWK       list of unused boundary y-points (islands).
%    NCSEG        number of coastal segments (w/o islands).
%    ICST         coastal segment x-index coordinate (w/o islands).
%    JCST         coastal segment y-index coordinate (w/o islands).
%    LNCST        length of coastal segment (w/o islands).
%    ISMASK       land/sea/boundary mask.
%    LANDT        modified Land/Sea mask (integer matrix).
%    SEACLR       color used to mark sea points.
%    CSTCLR       color used to draw coastlines.
%
% -------
% Output:
% -------
%
%    ICOAST       coastal segment x-index coordinate.
%    JCOAST       coastal segment y-index coordinate.
%    LENCOAST     length of coastal segment.
%    ISIS         island box starting x-index coordinate.
%    IEIS         island box ending x-index coordinate.
%    JSIS         island box starting y-index coordinate.
%    JEIS         island box ending y-index coordinate.

%-------------------------------------------------------------------
%  Initialize island data.
%-------------------------------------------------------------------

nisle = 0;
isis  = zeros(1,misle);
ieis  = isis;
jsis  = isis;
jeis  = isis;

vstr = version;
vnum = str2num(vstr(1));

%-------------------------------------------------------------------
%  Transfer data.
%-------------------------------------------------------------------

lencoast = lncst;
icoast   = icst;
jcoast   = jcst;

%-------------------------------------------------------------------
%  Clean and redraw plot.
%-------------------------------------------------------------------

[jmax,imax]=size(ismask);

[jland,iland]=find(ismask==2);
[jbry,ibry]=find(ismask==1);
[jsea,isea]=find(ismask==0);

clf;
plot(iland,jland,'g*',isea,jsea,[seaclr,'.'],ibry,jbry,'m+');
set(gca,'XLim',[1 imax],'YLim',[1 jmax],'FontName','Times');
if (vnum>4),
   set(gca,'FontSize',14);
end;
xlabel('Finding islands:  Please wait...');
set(get(gca,'XLabel'),'Color',seaclr);
hold on;

ind=find(icoast(:,1)~=0);
[ncseg,dum]=size(ind);
for n=1:ncseg,
   ind = find(icoast(n,:)~=0);
   plot(icoast(n,ind)',jcoast(n,ind)',cstclr);
end;
figure(gcf);

%-------------------------------------------------------------------
% Loop until list of boundary points is exhausted.
%-------------------------------------------------------------------

while ( (~isempty(ibrywk)) & (~isempty(jbrywk)) ),

%   ---------------------------------
%   --- Get new island coastline. ---
%   ---------------------------------

   ipt = ibrywk(1);
   jpt = jbrywk(1);
   ibold = ibrywk;
   jbold = jbrywk;

   [itmp,jtmp,ibrywk,jbrywk] = coastline (ismask,ibold,jbold,ipt,jpt);

%   ---------------------------
%   --- Set new island box. ---
%   ---------------------------

   nisle = nisle + 1;
   isis(nisle) = min(itmp);
   ieis(nisle) = max(itmp);
   jsis(nisle) = min(jtmp);
   jeis(nisle) = max(jtmp);

%   ------------------------------
%   --- Add coastline to list. ---
%   ------------------------------

   if (~isempty(itmp)),
      ncseg = ncseg + 1;
      ntmp = length (itmp);
      icoast(ncseg,1:ntmp) = itmp;
      jcoast(ncseg,1:ntmp) = jtmp;
      lencoast(ncseg)=ntmp;
      clear ntmp;
   end;

%   ---------------------------
%   --- Draw new coastline. ---
%   ---------------------------

   iwk = [isis(nisle) ieis(nisle) ieis(nisle) isis(nisle) isis(nisle)];
   jwk = [jsis(nisle) jsis(nisle) jeis(nisle) jeis(nisle) jsis(nisle)];
   plot (itmp,jtmp,cstclr,iwk,jwk,'m-.');
   clear itmp iwk jtmp jwk;
   figure(gcf);

end;

%-------------------------------------------------------------------
%  Clean titles and buttons.  Make sure entire mask shows
%-------------------------------------------------------------------

clf;
[jland,iland]=find(landt==0);
[jsea,isea]=find(landt==1);
plot(iland,jland,'g*',isea,jsea,[seaclr,'.']); hold on;
if (vnum>4),
   set(gca,'FontSize',14);
end;
set(gca,'xlim',[1 imax],'ylim',[1 jmax],'FontName','Times');
for n=1:ncseg,
   ind = find(icoast(n,:)~=0);
   plot(icoast(n,ind)',jcoast(n,ind)',cstclr);
end;
for n=1:nisle,
   iwk = [isis(n) ieis(n) ieis(n) isis(n) isis(n)];
   jwk = [jsis(n) jsis(n) jeis(n) jeis(n) jsis(n)];
   plot(iwk,jwk,'m-.');
end;
figure(gcf);

%  Clear workspace.

clear ibold ibry ibrywk iland ind ipt isea jbold jbry jbrywk jland ...
      jpt jsea n;
