function [ibrywk,jbrywk,ncseg,icoast,jcoast,lencoast] = ...
                                    xtr_seg (ismask,mcseg,mpseg,seaclr,cstclr);
%
%
% function [ibrywk,jbrywk,ncseg,icoast,jcoast,lencoast] = ...
%                                   xtr_seg (ismask,mcseg,mpseg,seaclr,cstclr);
%
% This procedure extracts coastal segments from ISMASK.
%
% ------
% Input:
% ------
%
%    ISMASK       land/sea/boundary mask.
%    MCSEG        maximum number of coastal segments.
%    MPSEG        maximum number of points per coastal segment.
%    SEACLR       color used to mark sea points.
%    CSTCLR       color used to draw coastlines.
%
% -------
% Output:
% -------
%
%    IBRYWK       list of unused boundary x-points (islands).
%    JBRYWK       list of unused boundary y-points (islands).
%    NCSEG        number of coastlines found.
%    ICOAST       coastal segment x-index coordinate.
%    JCOAST       coastal segment y-index coordinate.
%    LENCOAST     length of coastal segment.
%

%-------------------------------------------------------------------
%  Initialize coastline data.
%-------------------------------------------------------------------

ncseg    = 0;
icoast   = zeros(mcseg,mpseg);
jcoast   = icoast;
lencoast = zeros(1,mcseg);
ibrywk   = [];
jbrywk   = [];

vstr = version;
vnum = str2num(vstr(1));

%-------------------------------------------------------------------
%  Find break-up of points.  Abort if only sea.
%-------------------------------------------------------------------

[jmax,imax]=size(ismask);

[jland,iland]=find(ismask==2);
[jbry,ibry]=find(ismask==1);
[jsea,isea]=find(ismask==0);

if (isempty(ibry) & isempty(iland)),
   return;
end;

%-------------------------------------------------------------------
%  Clean and redraw plot.
%-------------------------------------------------------------------

clf;
plot(iland,jland,'g*',isea,jsea,[seaclr,'.'],ibry,jbry,'m+');
set(gca,'XLim',[1 imax],'YLim',[1 jmax],'FontName','Times');
if (vnum>4),
   set(gca,'FontSize',14);
end;
xlabel('Finding coastline:  Please wait...');
set(get(gca,'XLabel'),'Color',seaclr);
hold on;

ind=find(icoast(:,1)~=0);
[ncseg,dum]=size(ind);
for n=1:ncseg,
   ind = find(icoast(n,:)~=0);
   plot(icoast(n,ind)',jcoast(n,ind)',cstclr);
end;
figure(gcf);

%-------------------------------------------------------------------
%  Set initial working arrays.
%-------------------------------------------------------------------

ibrywk = ibry;
jbrywk = jbry;

iouter = [ 1:imax ...
           imax.*ones(1,(jmax-1)) ...
           (imax-1):-1:1 ...
           ones(1,(jmax-2)) ];

jouter = [ ones(1,imax) ...
           2:jmax ...
           jmax.*ones(1,(imax-1))  ...
           (jmax-1):-1:2 ];

%-------------------------------------------------------------------
%  Loop around outer edge of grid, with interior to left.
%-------------------------------------------------------------------

while ( (~isempty(iouter)) & (~isempty(jouter)) ),

%   --------------------------------------------
%   --- Select first point of outer vectors. ---
%   --------------------------------------------

   ipt = iouter(1);
   jpt = jouter(1);

   npt = length (iouter);
   ipt2 = iouter(min([2 npt]));
   jpt2 = jouter(min([2 npt]));

%   -------------------------------------------------
%   --- Determine if point is start of coastline. ---
%   -------------------------------------------------

   ind = find ( (ibrywk==ipt) & (jbrywk==jpt) );

   if ( (~isempty(ind)) & (ismask(jpt2,ipt2)==2) ),

%      -----------------------------------------------
%      --- Find coastline starting at given point. ---
%      -----------------------------------------------

      ibold = ibrywk;
      jbold = jbrywk;
      [itmp,jtmp,ibrywk,jbrywk] = coastline (ismask,ibold,jbold,ipt,jpt);

%      -------------------------------------
%      --- Update outer boundary arrays. ---
%      -------------------------------------

      ntmp = length (itmp);
      ind = find ( (iouter==itmp(ntmp)) & (jouter==jtmp(ntmp)) );
      if (~isempty(ind)),
         iouter(1:ind(1)) = [];
         jouter(1:ind(1)) = [];
      end;

%      ------------------------------
%      --- Add coastline to list. ---
%      ------------------------------

      if (~isempty(itmp)),
         ncseg = ncseg + 1;
         itmp(ntmp)=[];
         jtmp(ntmp)=[];
         ntmp = ntmp-1;
         icoast(ncseg,1:ntmp) = itmp;
         jcoast(ncseg,1:ntmp) = jtmp;
         lencoast(ncseg)=ntmp;
         clear ntmp;
      end;

%      ---------------------------
%      --- Draw new coastline. ---
%      ---------------------------

      plot (itmp,jtmp,cstclr);
      clear itmp jtmp;
      figure(gcf);

     else

%      -------------------------------------
%      --- Update outer boundary arrays. ---
%      -------------------------------------

      iouter(1) = [];
      jouter(1) = [];

   end;

end;

%-------------------------------------------------------------------
%  Clean plot and workspace.
%-------------------------------------------------------------------

xlabel(' ');

clear dum ibold ibry iland ind iouter ipt ipt2 isea jbold jbry jland ...
      jouter jpt jpt2 jsea n npt;
