function [lon,lat] = xy2ll (x,y,coord,nx,ny,gridx,gridy,rlngd,rlatd, ...
                                                   delx,dely,thetad);
%
%=======================================================================
%                                                                    ===
% function [lon,lat] = xy2ll (x,y,coord,nx,ny,gridx,gridy,rlngd, ... ===
%                                        rlatd,delx,dely,thetad);    ===
%                                                                    ===
% This routine computes the longitude and latitude of a given        ===
% point from the supplied grid coordinates.                          ===
%                                                                    ===
% ------                                                             ===
% Input:                                                             ===
% ------                                                             ===
%                                                                    ===
%     X, Y      the grid coordinates                                 ===
%     COORD     flag for coordinate type                             ===
%                  [0]  Cartesian (tangent plane) grid.              ===
%                  [1]  unrotated spherical grid.                    ===
%                  [2]  rotated spherical grid.                      ===
%     DELX,                                                          ===
%     DELY,     Offset between transform & grid centers.   (m | deg) ===
%     GRIDX     grid spacing in x-direction                (m | deg) ===
%     GRIDY     grid spacing in y-direction                (m | deg) ===
%     NX, NY    number of grid points in x & y directions            ===
%     RLNGD     longitude at center of transformation      (deg)     ===
%     RLATD     latitude at center of transformation       (deg)     ===
%     THETAD    angle grid is rotates with respect to east (deg)     ===
%                                                                    ===
% -------                                                            ===
% Output:                                                            ===
% -------                                                            ===
%                                                                    ===
%     LON, LAT  the longitude and latitude of the point    (deg)     ===
%                                                                    ===
%=======================================================================

%=======================================================================
%                                                                    ===
% Calls:  rotparm                                                    ===
%                                                                    ===
%=======================================================================

c180    = 180.0;
deg2rad = pi./c180;
xc      = (nx+1)./2 - (delx/gridx);
yc      = (ny+1)./2 - (dely/gridy);

%-----------------------------------------------------------------------
%  Cartesian (tangent plane) system.
%-----------------------------------------------------------------------

if (coord==0),
   re_d2r = 6371315.0.*deg2rad;
   rf1x   = gridx./re_d2r;
   rf1y   = gridy./re_d2r;
   rf2x   = rf1x./cos(rlatd.*deg2rad);
   rf2y   = rf1y./cos(rlatd.*deg2rad);
   sterm  = sin(thetad.*deg2rad);
   cterm  = cos(thetad.*deg2rad);
   dx     = x-xc;
   dy     = y-yc;
   lon    = rlngd+rf2x.*dx.*cterm-rf2y.*dy.*sterm;
   lat    = rlatd+rf1y.*dy.*cterm+rf1x.*dx.*sterm;

%-----------------------------------------------------------------------
%  Unrotated spherical system.
%-----------------------------------------------------------------------

  elseif (coord==1),
   lon = (x-xc).*gridx;
   lat = (y-yc).*gridy;

%-----------------------------------------------------------------------
%  Rotated spherical system.
%-----------------------------------------------------------------------

  else,
   rad2deg = c180./pi;
   [plon,plat,rnlon] = rotparm(rlngd.*deg2rad,rlatd.*deg2rad,thetad.*deg2rad);
   cosp = cos(plat);
   sinp = sin(plat);
   rlon = (x-xc).*gridx;
   rlat = (y-yc).*gridy;
   cosr = cos(rlat.*deg2rad);
   sinr = sin(rlat.*deg2rad);
   lat  = asin(sinr.*sinp+cosr.*cosp.*cos(rlon.*deg2rad-rnlon));
   sin1 = cosr.*sin(rlon.*deg2rad-rnlon);
   cos1 = sinr.*cosp-sinp.*cosr.*cos(rlon.*deg2rad-rnlon);
   lon  = plon-atan2(sin1,cos1);
   ind  = find(abs(lon)>pi);
   if (~isempty(ind)),
      lon(ind) = sign(lon(ind)).*(rem(abs(lon(ind)),2.*pi) - ...
                             2.*pi.*fix(rem(abs(lon(ind)),2.*pi)./pi));
   end;
   lat = lat.*rad2deg;
   lon = lon.*rad2deg;

end;
