      subroutine external(u,v,psi,h,im,jm,ncinpid)
c
c=======================================================================
c                                                                    ===
c  This routine computes horizontal velocity components (U,V) from   ===
c  quasi-geostrophic streamfunction PSI.                             ===
c                                                                    ===
c  On input:                                                         ===
c                                                                    ===
c     PSI      transport streamfunction field (real array).          ===
c     H        depth at tracer points (real array).                  ===
c     IM       number of points in the x-direction (integer).        ===
c     JM       number of points in the y-direction (integer).        ===
c     NCINPID  NetCDf ID for input file (integer).                   ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     U        zonal velocity component (real array; cm/s).          ===
c     V        meridional velocity component (real array; cm/s).     ===
c                                                                    ===
c  Required common blocks:                                           ===
c                                                                    ===
c     NETCDF,  PLTIO                                                 ===
c                                                                    ===
c  Calls:                                                            ===
c                                                                    ===
c     NCVID   (NetCDF library)                                       ===
c     EXITUS, NCVGET, NCVGET1                                        ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <pconst.h>
#include <netcdf.inc>
#include <pltio.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,im,j,jm,ncinpid,rcode,varid
      real diag1,diag2,facx,facy,gridx
      real psi(im,jm),h(im,jm),u(im,jm),v(im,jm)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Read in scaling parameters.  They are read each time to facilitate
c  generality.
c
      varid=ncvid(ncinpid,'dyt',rcode)
      if(rcode.eq.0) then
        call ncvget(ncinpid,varid,1,1,gridx,rcode)
      else
        varid=ncvid(ncinpid,'gridx',rcode)
        if(rcode.eq.0) then
          call ncvget1(ncinpid,varid,1,gridx,rcode)
        else
          write(stdout,900) 'dyt & gridx',ncinpid
          call exitus('EXTERNAL')
        endif
      endif
c
c  Compute depths at velocity points.  Use U array for temporary
c  storage.
c
      do 10 j=1,jm-1
      do 10 i=1,im-1
        u(i,j)=p25*(h(i,j)+h(i+1,j)+h(i,j+1)+h(i+1,j+1))
  10  continue
      do 20 j=1,jm-1
        u(im,j)=p5*(h(im,j)+h(im,j+1))
  20  continue
      do 30 i=1,im-1
        u(i,jm)=p5*(h(i,jm)+h(i+1,jm))
  30  continue
      u(im,jm)=p5*(u(im-1,jm)+u(im,jm-1))
c
c  Load reciprocal into depth array.
c
      do 40 j=1,jm
      do 40 i=1,im
        h(i,j)=c1/abs(u(i,j)*m2cm)
  40  continue
c
c  Compute velocity components.
c
      facx=p5/gridx
      facy=p5/gridx
      do 50 j=1,jm-1
      do 50 i=1,im-1
        diag1=psi(i+1,j+1)-psi(i  ,j)
        diag2=psi(i  ,j+1)-psi(i+1,j)
        u(i,j)=-(diag1+diag2)*h(i,j)*facy
        v(i,j)= (diag1-diag2)*h(i,j)*facx
  50  continue
c
c  Set eastern and northern boundary values.
c
      do 60 i=1,im
        u(i,jm)=u(i,jm-1)
        v(i,jm)=v(i,jm-1)
  60  continue
      do 70 j=1,jm
        u(im,j)=u(im-1,j)
        v(im,j)=v(im-1,j)
  70  continue
c
 900  format(/' EXTERNAL - cannot find variable: ',a,2x,
     *        ' in input NetCDF file ID = ',i2)
      return
      end
