      subroutine fdiag(f,ixm,im,jm,class,ifield,level,day,iunit)
c
c=======================================================================
c                                                                    ===
c  This routine computes field diagnostics for fields having PE      ===
c  or QG model field-classification.                                 ===
c                                                                    ===
c  On Input:                                                         ===
c                                                                    ===
c     F        field (real array)                                    ===
c     CLASS   field classification (integer).                        ===
c     IFIELD   field type, PE or QG classification (integer)         ===
c     IM       number of points in the x-direction                   ===
c     IXM      first dimension of F in the calling program (integer) ===
c     JM       number of points in the y-direction (integer)         ===
c     LEVEL    field depth level (integer)                           ===
c     DAY      day (real)                                            ===
c     IUNIT    output logical unit (integer)                         ===
c                                                                    ===
c  Required common blocks: (only relevant variables documented)      ===
c                                                                    ===
c     /FDIAGN/                                                       ===
c                                                                    ===
c     FADV     average deviation (real; output)                      ===
c     FAVG     mean (real; output)                                   ===
c     FKUR     kurtosis (real; output)                               ===
c     FMAX     maximum value (real; output)                          ===
c     FMIN     minimum value (real; output)                          ===
c     FRMS     root mean square (real; output)                       ===
c     FSDV     standard deviation (real; output)                     ===
c     FSKW     skewness (real; output)                               ===
c     FVAR     variance (real; output)                               ===
c                                                                    ===
c  Calls:  LNBLK, ID2FLDNAM                                          ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <pconst.h>
#include <fdiagn.h>
#include <inpcdf.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,class,ifield,imax,im,imin,ip,iunit,ixm,j,jm,jmax,jmin,
     *        lenstr,level,npts,vindx
      integer lnblk
      integer n,no_digit
      real day,famx,fval,p,s,small,sscl,term
      real f(*)
      logical      nograph
      character*39  fmt
      character*128 field,varnam
      parameter (small=c1em20)
c
      parameter (nograph=.false.)
c
      no_digit(n)=int(log10(float(max(abs(n),1))))+(1+max(-sign(1,n),0))
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Compute the mean and get minimum and maximum values.
c-----------------------------------------------------------------------
c
      npts=0
      favg=c0
      fmin=spval1
      imin=1
      jmin=1
      fmax=-spval1
      imax=1
      jmax=1
      do 20 j=1,jm
        do 10 i=1,im
          ip=i+(j-1)*ixm
          if(f(ip).lt.spval1) then
            if(f(ip).lt.fmin) then
              fmin=f(ip)
              imin=i
              jmin=j
            endif
            if(f(ip).gt.fmax) then
              fmax=f(ip)
              imax=i
              jmax=j
            endif
            npts=npts+1
            favg=favg+f(ip)
          endif
 10     continue
 20   continue
      if(npts.ne.0) favg=favg/float(npts)
c
c-----------------------------------------------------------------------
c  Compute the root mean square (FRMS), average deviation (FADEV),
c  standard deviation (FSDV), variance (FVAR), skewness (FSKEW), and
c  kurtosis (FKUR).
c-----------------------------------------------------------------------
c
      frms=c0
      fadv=c0
      fvar=c0
      fskw=c0
      fkur=c0
      npts=0
c
      famx = max( abs(fmin), abs(fmax) )
      if (famx.gt.c1) then
         sscl = c1/famx
       else
         sscl = c1
      end if
c
      do 40 j=1,jm
        do 30 i=1,im
          ip=i+(j-1)*ixm
          if(f(ip).lt.spval0) then
            npts=npts+1
            fval=f(ip)*sscl
            frms=frms+fval*fval
            s=fval-favg*sscl
            fadv=fadv+abs(s)
            p=s*s
            fvar=fvar+p
            p=p*s
            fskw=fskw+p
            p=p*s
            fkur=fkur+p
          endif
 30     continue
 40   continue
c
      if(npts.gt.0) then
        frms=sqrt(frms/float(npts))/sscl
        fadv=fadv/float(npts)/sscl
      else
        favg=spval1
        frms=spval1
        fadv=spval1
      endif
c
      if(npts.gt.1) then
        fvar=fvar/float(npts-1)
        fsdv=sqrt(fvar)
        term=float(npts)*fsdv**3
        if(abs(term).gt.abs(fskw*small)) then
          fskw=fskw/term
        else
          fskw=spval1
        endif
        term=float(npts)*fvar*fvar
        if(abs(term).gt.abs(fkur*small)) then
          fkur=fkur/term
        else
          fkur=spval1
        endif
        fvar=fvar/(sscl*sscl)
        fsdv=fsdv/sscl
      else
        fvar=spval1
        fsdv=spval1
        fskw=spval1
        fkur=spval1
      endif
c
c-----------------------------------------------------------------------
c  Get QG or PE field identification string.
c-----------------------------------------------------------------------
c
      if(class.eq.qgclass) then
        call id2fldnam(ifield,qgclass,nograph,varnam,vindx,field)
      elseif(class.eq.peclass) then
        call id2fldnam(ifield,peclass,nograph,varnam,vindx,field)
      endif
      lenstr=lnblk(field,len(field))
c
c-----------------------------------------------------------------------
c  Report diagnostics.
c-----------------------------------------------------------------------
c
      if(level.gt.0) then
        write(iunit,50) day,field(1:lenstr),level
      else
        write(iunit,60) day,field(1:lenstr)
        write(fmt,65) no_digit(abs(level))
        write(iunit,fmt) abs(level)
      endif
      write(iunit,70) npts,fmin,imin,jmin,fmax,imax,jmax,favg,frms,
     *                fadv,fvar,fsdv,fskw,fkur
 50   format(/,'  Field Diagnostics:',//
     *       10x,'                Day = ',f10.3,/,
     *       10x,'         Field type = ',a,/,
     *       10x,'        Model level = ',i5)
 60   format(/,'  Field Diagnostics:',//
     *       10x,'                Day = ',f10.3,/,
     *       10x,'         Field type = ',a)
 65   format('(10x,',1h','              Depth = ',1h',',i',i2,',',
     &       1h',' m',1h',')')
 70   format(10x,'   Number of points = ',i7,/,
     *       10x,'            Minimum = ',1pe15.8,
     *        5x,'( ',i3,',',i3,' )',/,
     *       10x,'            Maximum = ',1pe15.8,
     *        5x,'( ',i3,',',i3,' )',/,
     *       10x,'               Mean = ',1pe15.8,/,
     *       10x,'                RMS = ',1pe15.8,/,
     *       10x,'  Average Deviation = ',1pe15.8,/,
     *       10x,'           Variance = ',1pe15.8,/,
     *       10x,' Standard Deviation = ',1pe15.8,/,
     *       10x,'           Skewness = ',1pe15.8,/,
     *       10x,'           Kurtosis = ',1pe15.8,/)
      return
      end
