      subroutine sea_elev(f1,f2,im,jm,ncinpid,spv,spval)
c
c=======================================================================
c                                                                    ===
c  This routine computes sea surface elevation from streamfunction   ===
c  and top density.                                                  ===
c                                                                    ===
c  On input:                                                         ===
c                                                                    ===
c     F1       nondimensional streamfunction field (real array).     ===
c     F2       nondimensional top density (real array).              ===
c     IM       number of points in the x-direction (integer).        ===
c     JM       number of points in the y-direction (integer).        ===
c     NCINPID  NetCDf ID for input file (integer).                   ===
c     SPV      missing data/fill data value (real).                  ===
c     SPVAL    special value (real).                                 ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     F1       sea surface elevation (real array, meters)            ===
c                                                                    ===
c  Required common blocks:                                           ===
c                                                                    ===
c     NETCDF,  PLTIO                                                 ===
c                                                                    ===
c  Calls:                                                            ===
c                                                                    ===
c     NCVID   (NetCDF library)                                       ===
c     EXITUS, NCVGET, NCVGET1                                        ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <pconst.h>
#include <netcdf.inc>
#include <pltio.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer count,i,im,j,jm,ncinpid,start,rcode,varid
      real dhor,fval,f0,ht,hz1,r0,scale1,scale2,scsig,sigz1,spv,spval,
     *     t0,v0
      real f1(im,jm),f2(im,jm)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Read in scaling parameters.  They are read each time to facilitate
c  generality.
c
      varid=ncvid(ncinpid,'r0',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,r0,rcode)
      else
        write(stdout,900) 'r0',ncinpid
        call exitus('SEA_ELEV')
      endif
      varid=ncvid(ncinpid,'t0',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,t0,rcode)
      else
        write(stdout,900) 't0',ncinpid
        call exitus('SEA_ELEV')
      endif
      varid=ncvid(ncinpid,'dhor',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,dhor,rcode)
      else
        write(stdout,900) 'dhor',ncinpid
        call exitus('SEA_ELEV')
      endif
      varid=ncvid(ncinpid,'ht',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,ht,rcode)
      else
        write(stdout,900) 'ht',ncinpid
        call exitus('SEA_ELEV')
      endif
      varid=ncvid(ncinpid,'v0',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,v0,rcode)
      else
        write(stdout,900) 'v0',ncinpid
        call exitus('SEA_ELEV')
      endif
      varid=ncvid(ncinpid,'scsig',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,scsig,rcode)
      else
        write(stdout,900) 'scsig',ncinpid
        call exitus('SEA_ELEV')
      endif
      start=1
      count=1
      varid=ncvid(ncinpid,'hz',rcode)
      if(rcode.eq.0) then
        call ncvget(ncinpid,varid,start,count,hz1,rcode)
      else
        write(stdout,900) 'hz',ncinpid
        call exitus('SEA_ELEV')
      endif
      varid=ncvid(ncinpid,'sigz',rcode)
      if(rcode.eq.0) then
        call ncvget(ncinpid,varid,start,count,sigz1,rcode)
      else
        write(stdout,900) 'sigz',ncinpid
        call exitus('SEA_ELEV')
      endif
c
c  Compute sea surface elevation.
c
      f0=c1/(r0*t0)
      scale1=f0*v0*dhor/grav
      scale2=p5*(hz1/ht)/(scsig*sigz1)
      do 10 j=1,jm
      do 10 i=1,im
        if((f1(i,j).ge.spv).or.(f2(i,j).ge.spv)) then
          f1(i,j)=spval
        else
          fval=scale1*f1(i,j)+scale2*f2(i,j)
          f1(i,j)=fval
        endif
  10  continue
c
 900  format(/' SEA_ELEV - cannot find variable: ',a,2x,
     *        ' in input NetCDF file ID = ',i2)
      return
      end
