      function svan(s,t,p0,sigma)
c
c=======================================================================
c                                                                    ===
c  Specific volume anomaly (steric anomaly) based on 1980 Equation   ===
c  of State for Seawater and 1978 Practical Salinity Scale.          ===
c                                                                    ===
c  On Input:                                                         ===
c                                                                    ===
c     P0       pressure (real; decibars)                             ===
c     T        temperature (real; degree Celsius IPTS-68)            ===
c     S        salinity (real; PSU, IPSS-78)                         ===
c     SVAN     specific volume anomaly (real; 1.0e-8 * m^3/kg)       ===
c     SIGMA    density anomaly (real; kg/m^3)                        ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     SVAN     specific volume anomaly (real; 1.0e-8 * m^3/kg)       ===
c     SIGMA    density anomaly (real; kg/m^3)                        ===
c                                                                    ===
c  Check values:                                                     ===
c                                                                    ===
c     svan  = 981.30210 e-8 m^3/kg                                   ===
c     sigma =  59.82037 kg/m^3                                       ===
c                                                                    ===
c             for  s=40 (PSS-78), t=40 C, p=10000 decibars           ===
c                                                                    ===
c  References:                                                       ===
c                                                                    ===
c     Millero, et al., 1980: A new high-pressure equation of state   ===
c             for seawater. Deep-Sea Res., 27a, 255-264.             ===
c     Millero and Poisson, 1981: Summary of data treatment for the   ===
c             Unesco one atmosphere equation of state for seawater   ===
c             Deep-Sea Res., 28a, 625-629.                           ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      real a,aw,a1,b,bw,b1,c,d,d350,dk,dr35p,dr350,dvan,e,gam,kw,k0,
     *     k35,p,p0,pk,r1,r2,r3,r4,r3500,t,s,sig,sigma,sr,sva,svan,
     *     v350p
      data dr350,r3500,r4 /28.106331,1028.1063,4.8314e-4/
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Convert pressure to bars and take square root salinity.
c
      p=p0/10.
      sr=sqrt(abs(s))
c
c  Density of the reference pure water (Bigg, P.H., 1967, Br. J. Applied
c  Physics, 8, 521-537)
c
      r1=((((6.536332e-9*t-1.120083e-6)*t+1.001685e-4)*t
     *   -9.095290e-3)*t+6.793952e-2)*t-28.263737
c
c  Seawater density at atmospheric pressure, coefficients involving
c  salinity.
c
      r2=(((5.3875e-9*t-8.2467e-7)*t+7.6438e-5)*t-4.0899e-3)*t
     *   +8.24493e-1
      r3=(-1.6546e-6*t+1.0227e-4)*t-5.72466e-3
c
c-----------------------------------------------------------------------
c  International one-atmosphere equation of state of seawater.
c-----------------------------------------------------------------------
c
      sig=(r4*s+r3*sr+r2)*s+r1
c
c  Specific volume at atmospheric pressure.
c
      v350p=1.0/r3500
      sva=-sig*v350p/(r3500+sig)
      sigma=dr350-(sva/(v350p*(v350p+sva)))
c
c  Scale specific volume anamoly to normally reported units.
c
      svan=sva*1.0e+8
      if(p.eq.0.0) return
c
c-----------------------------------------------------------------------
c  New high pressure equation of state for seawater.
c-----------------------------------------------------------------------
c
c  Compute compression terms.
c
      e=(9.1697e-10*t+2.0816e-8)*t-9.9348e-7
      bw=(5.2787e-8*t-6.12293e-6)*t+3.47718e-5
      b=bw+e*s
c
      d=1.91075e-4
      c=(-1.6078e-6*t-1.0981e-5)*t+2.2838e-3
      aw=((-5.77905e-7*t+1.16092e-4)*t+1.43713e-3)*t-0.1194975
      a=(d*sr+c)*s+aw
c
      b1=(-5.3009e-4*t+1.6483e-2)*t+7.944e-2
      a1=((-6.1670e-5*t+1.09987e-2)*t-0.603459)*t+54.6746
      kw=(((-5.155288e-5*t+1.360477e-2)*t-2.327105)*t+148.4206)*t
     *   -1930.06
      k0=(b1*sr+a1)*s+kw
c
c  Evaluate pressure polynomial:
c
c   K is the secant bulk modulus of seawater
c   DK = K(s,t,p) - K(35,0,p)
c   K35 = K(35,0,p)
c
      dk=(b*p+a)*p+k0
      k35=(5.03217e-5*p+3.359406)*p+21582.27
      gam=p/k35
      pk=1.0-gam
      sva=sva*pk+(v350p+sva)*p*dk/(k35*(k35+dk))
c
c  Scale specific volume anamoly to normally reported units.
c
      svan=sva*1.0e+8
      v350p=v350p*pk
c
c-----------------------------------------------------------------------
c  Compute density anamoly with respect to 1000.0 kg/m**3:
c
c   1) dr350: density anamoly at 35 PSU, 0 deg. C, atmospheric pressure
c   2) dr35p: density anamoly 35 PSU, 0 deg. C, with pressure variation
c   3) dvan : density anamoly variations involving specific vol. anamoly
c-----------------------------------------------------------------------
c
      d350=gam/pk
      dr35p=r3500*d350
      dvan=sva/(v350p*(v350p+sva))
      sigma=dr350+dr35p-dvan
      return
      end
