      subroutine velocity(psi,u,v,im,jm,ncinpid,spv,spval)
c
c=======================================================================
c                                                                    ===
c  This routine computes horizontal velocity components (U,V) from   ===
c  quasi-geostrophic streamfunction PSI.                             ===
c                                                                    ===
c  On input:                                                         ===
c                                                                    ===
c     PSI      nondimensional streamfunction field (real array).     ===
c     IM       number of points in the x-direction (integer).        ===
c     JM       number of points in the y-direction (integer).        ===
c     NCINPID  NetCDf ID for input file (integer).                   ===
c     SPV      missing data/fill data value (real).                  ===
c     SPVAL    special value (real).                                 ===
c                                                                    ===
c  On Output:                                                        ===
c                                                                    ===
c     U        zonal velocity component (real array; cm/s).          ===
c     V        meridional velocity component (real array; cm/s).     ===
c                                                                    ===
c  Required common blocks:                                           ===
c                                                                    ===
c     NETCDF,  PLTIO                                                 ===
c                                                                    ===
c  Calls:                                                            ===
c                                                                    ===
c     NCVID   (NetCDF library)                                       ===
c     EXITUS, NCVGET1                                                ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <pconst.h>
#include <netcdf.inc>
#include <pltio.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i,im,j,jm,ncinpid,rcode,varid
      real dhor,dx,dy,facx,facy,scale,spv,spval,v0
      real psi(im,jm),u(im,jm),v(im,jm)
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c  Read in scaling parameters.  They are read each time to facilitate
c  generality.
c
      varid=ncvid(ncinpid,'dx',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,dx,rcode)
      else
        write(stdout,900) 'dx',ncinpid
        call exitus('VELOCITY')
      endif
      varid=ncvid(ncinpid,'dy',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,dy,rcode)
      else
        write(stdout,900) 'dy',ncinpid
        call exitus('VELOCITY')
      endif
      varid=ncvid(ncinpid,'dhor',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,dhor,rcode)
      else
        write(stdout,900) 'dhor',ncinpid
        call exitus('VELOCITY')
      endif
      varid=ncvid(ncinpid,'v0',rcode)
      if(rcode.eq.0) then
        call ncvget1(ncinpid,varid,1,v0,rcode)
      else
        write(stdout,900) 'v0',ncinpid
        call exitus('VELOCITY')
      endif
c
c  Compute velocity components.
c
      scale=v0*dhor
      facx=p5*scale/dx
      facy=p5*scale/dy
      do 10 j=1,jm-1
      do 10 i=1,im-1
        if(psi(i,j).ge.spv) then
          u(i,j)=spval
          v(i,j)=spval
        else
          u(i,j)=facy*((psi(i,j)+psi(i+1,j))-(psi(i,j+1)+psi(i+1,j+1)))
          v(i,j)=facx*((psi(i+1,j)+psi(i+1,j+1))-(psi(i,j)+psi(i,j+1)))
        endif
  10  continue
c
c  Set eastern and northern boundary values.
c
      do 20 i=1,im
        u(i,jm)=u(i,jm-1)
        v(i,jm)=v(i,jm-1)
  20  continue
      do 30 j=1,jm
        u(im,j)=u(im-1,j)
        v(im,j)=v(im-1,j)
  30  continue
c
 900  format(/' VELOCITY - cannot find variable: ',a,2x,
     *        ' in input NetCDF file ID = ',i2)
      return
      end
